'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

require('react-app-polyfill/ie11');
require('core-js/features/array/find');
require('core-js/features/array/includes');
require('core-js/features/number/is-nan');
require('core-js/features/array/at');
require('resize-observer-polyfill');
var dateFns = require('date-fns');
var React = require('react');
var solid = require('@heroicons/react/20/solid');
var outline = require('@heroicons/react/24/outline');
var reactResizeDetector = require('react-resize-detector');
var ReactTooltip = require('react-tooltip');
var react = require('@headlessui/react');
var reactFocusOn = require('react-focus-on');
var d3 = require('d3');
var chromaJs = require('chroma-js');
var Papa = require('papaparse');
var moment = require('moment');

const CUSTOM_METADATA_KEY = 'item count';
const countersStorageKey = 'oneAi-counters-cache';
const labelsStorageKey = 'oneAi-labels-cache';
const sizeAxisStorageKey = 'oneAi-sizeAxis-cache';
const colorAxisStorageKey = 'oneAi-colorAxis-cache';
const PAGE_SIZE = 25;

const resolveDomain = userInputDomain => {
  if (!userInputDomain) {
    return 'https://api.oneai.com';
  }
  if (userInputDomain.startsWith('http')) {
    return userInputDomain;
  }
  if (userInputDomain === 'prod') {
    return 'https://api.oneai.com';
  }
  if (userInputDomain === 'staging') {
    return 'https://staging.oneai.com';
  }
  return `https://${userInputDomain}`;
};

const COLLECTION_TYPE = 'Collection';
function getNodeText(node) {
  return node.type === 'Cluster' ? node.data.cluster_phrase : node.type === 'Phrase' ? node.data.text : node.type === 'Meta' ? node.data.meta_value : node.data.original_text;
}
function getNodeId(node) {
  return (node.type === 'Cluster' ? node.data.cluster_id : node.type === 'Phrase' ? node.data.phrase_id : node.type === 'Meta' ? node.data.meta_key + '$$' + node.data.meta_value : node.data.id).toString();
}
function getNodeItemsCount(node) {
  return node.type === 'Cluster' ? node.data.items_count : node.type === 'Phrase' ? node.data.items_count : node.type === 'Meta' ? node.data.items_count : 1;
}
function getNodeDetails(node, collection) {
  const currentNodeType = node?.type ?? COLLECTION_TYPE;
  const currentNodeId = node ? getNodeId(node) : collection;
  return {
    id: currentNodeId,
    type: currentNodeType
  };
}
function getNodeTrends(node) {
  if (!node) return [];
  return node.type === 'Cluster' ? node.data.trends ?? [] : node.type === 'Phrase' ? node.data.trends ?? [] : node.type === 'Meta' ? node.data.trends ?? [] : [];
}
const getNodeOriginalAndTranslatedText = node => {
  if (!node) return {
    originalText: undefined,
    translatedText: undefined
  };
  if (['Cluster', 'Phrase', 'Item'].includes(node.type)) {
    const cluster = node.data;
    return {
      originalText: cluster.item_original_text,
      translatedText: cluster.item_translated_text ?? cluster.item_original_text
    };
  }
  if (node.type === 'Meta') {
    const item = node.data;
    return {
      originalText: item.meta_value,
      translatedText: item.meta_value
    };
  }
  return {
    originalText: undefined,
    translatedText: undefined
  };
};

function calculateFontSize(height, width) {
  const area = height * width;
  let fontSize = 10 + 32 * area / (250000 - 15000);
  fontSize = Math.max(fontSize, 10);
  fontSize = Math.min(fontSize, 60);
  return fontSize;
}
function chunks(arr, size) {
  return Array.from(new Array(Math.ceil(arr.length / size)), (_, i) => arr.slice(i * size, i * size + size));
}
function getSecondsDiff(startDate, endDate) {
  const diff = endDate.getTime() - startDate.getTime();
  return Math.abs(diff / 1000);
}
function sum(numbers) {
  return numbers.reduce((partialSum, a) => partialSum + a, 0);
}
function uniqBy(arr, getKey) {
  let seen = new Set();
  return arr.filter(item => {
    let k = getKey(item);
    return seen.has(k) ? false : seen.add(k);
  });
}
function toLowerKeys(obj) {
  if (typeof obj === 'undefined') {
    return {};
  }
  return Object.keys(obj).reduce((accumulator, key) => {
    accumulator[key.toLowerCase()] = obj[key];
    return accumulator;
  }, {});
}
function percentageIncrease(previous, current) {
  let percent;
  if (current !== 0) {
    if (previous !== 0) {
      percent = (current - previous) / previous * 100;
    } else {
      percent = current * 100;
    }
  } else {
    percent = -previous * 100;
  }
  return Math.floor(percent);
}
const numberToFixed = num => Math.round((num + Number.EPSILON) * 100) / 100;
const customNumberToFixed = num => {
  const str = Math.abs(num).toString();
  const [integer, _] = str.split('.');
  if (integer.length < 3) {
    return numberToFixed(parseFloat(num.toFixed(2)));
  } else if (integer.length < 4) {
    return numberToFixed(parseFloat(num.toFixed(1)));
  } else {
    return parseFloat(num.toFixed(0));
  }
};
function getNumberDescription(numIn) {
  const num = customNumberToFixed(numIn);
  const prefix = num < 0 ? '-' : '';
  // Enumerate num abbreviations
  let abbrev = ['k', 'm', 'b', 't'];
  let strResult = num.toString();
  let result = Math.abs(num);
  let decPlaces = strResult.length > 3 ? 1 : 2;
  // 2 decimal places => 100, 3 => 1000, etc
  decPlaces = Math.pow(10, decPlaces);
  // Go through the array backwards, so we do the largest first
  for (let i = abbrev.length - 1; i >= 0; i--) {
    // Convert array index to "1000", "1000000", etc
    let size = Math.pow(10, (i + 1) * 3);
    // If the num is bigger or equal do the abbreviation
    if (size <= result) {
      // Here, we multiply by decPlaces, round, and then divide by decPlaces.
      // This gives us nice rounding to a particular decimal place.
      result = Math.round(result * decPlaces / size) / decPlaces;
      // Handle special case where we round up to the next abbreviation
      if (result === 1000 && i < abbrev.length - 1) {
        result = 1;
        i++;
      }
      // Add the letter for the abbreviation
      strResult = result.toString() + abbrev[i];
      // We are done... stop
      break;
    }
  }
  return prefix + strResult;
}
const objectToLowerCase = obj => {
  const entries = Object.entries(obj);
  return Object.fromEntries(entries.map(([key, value]) => {
    return [key.toLowerCase(), value];
  }));
};
const skillsArray = ['sentence', 'entity', 'enhance', 'highlight', 'keyword', 'summarize', 'emotion', 'article-topic', 'sentiment', 'extract-html', 'html-extract-text', 'html-extract-article', 'anonymize', 'embedding', 'clustering', 'action-item', 'business-entity', 'sales-insight', 'dialogue-segmentation', 'name', 'number', 'find', 'transcribe', 'translate', 'detect-language', 'debug', 'headline', 'subheading', 'skill-discovery', 'whisper-transcription', 'service-email-insight', 'pdf-extract-text', 'custom', 'fact-check', 'gpt', 'classify', 'collection-search', 'coda-query-classifier', 'social-toxicity', 'content-curation', 'subsentence', 'bizgpt', 'files-extract-text', 'query-rewriter'];

function topGroupPercentCalculation(metadataKeyValue, metadata, _, countersConfigurations, __) {
  const counter = getMetadataKeyValueConfiguration(metadataKeyValue, countersConfigurations);
  const groups = getCounterGroups(counter);
  if (!counter || groups.length === 0) return {
    counter: counter,
    result: 0
  };
  const sums = groups.map(group => calculateSumItemsInMetadata(group.members, [], metadata));
  const max = Math.max(...sums);
  const maxGroup = groups[sums.indexOf(max)];
  const groupsSum = sum(sums);
  return {
    result: groupsSum === 0 ? 0 : max / groupsSum * 100,
    counter: max > 0 ? maxGroup ?? counter : counter,
    metadataKey: maxGroup.label,
    value: undefined
  };
}
function topGroupCalculation(metadataKeyValue, metadata, _, countersConfigurations, __) {
  const counter = getMetadataKeyValueConfiguration(metadataKeyValue, countersConfigurations);
  const groups = getCounterGroups(counter);
  if (groups.length === 0) return {
    counter: counter,
    result: 0
  };
  const sums = groups.map(group => calculateSumItemsInMetadata(group.members, [], metadata));
  const max = Math.max(...sums);
  const maxGroup = groups[sums.indexOf(max)];
  return {
    result: max,
    counter: max > 0 ? maxGroup ?? counter : counter,
    metadataKey: maxGroup.label,
    value: undefined
  };
}
function topValuePercentCalculation(metadataKeyValue, metadata, _, countersConfigurations, __) {
  const counter = getMetadataKeyValueConfiguration(metadataKeyValue, countersConfigurations);
  if (!counter) return {
    counter: counter,
    result: 0
  };
  const max = getMaxItemValue(counter.members, counter.items, metadata);
  const sum = calculateSumItemsInMetadata(counter.members, counter.items, metadata);
  return {
    result: sum === 0 ? 0 : max.count / sum * 100,
    counter: getItemCounterConfiguration(max.metadata, max.value, countersConfigurations) ?? counter,
    metadataKey: max.metadata,
    value: max.value
  };
}
function topValueCalculation(metadataKeyValue, metadata, _, countersConfigurations, __) {
  const counter = getMetadataKeyValueConfiguration(metadataKeyValue, countersConfigurations);
  if (!counter) return {
    counter: counter,
    result: 0
  };
  const maxItem = getMaxItemValue(counter.members, counter.items, metadata);
  return {
    result: maxItem.count,
    counter: getItemCounterConfiguration(maxItem.metadata, maxItem.value, countersConfigurations) ?? counter,
    metadataKey: maxItem.metadata,
    value: maxItem.value
  };
}
function totalSumCalculation(metadataKeyValue, metadata, _, countersConfigurations, __) {
  const counter = getMetadataKeyValueConfiguration(metadataKeyValue, countersConfigurations);
  if (!counter) return {
    counter: counter,
    result: 0
  };
  return {
    result: calculateSumItemsInMetadata(counter.members, counter.items, metadata),
    counter: counter
  };
}
function percentOfItemsCalculation(metadataKeyValue, metadata, _, countersConfigurations, __) {
  if (!metadataKeyValue) return {
    counter: null,
    result: 0
  };
  const itemCounter = getMetadataKeyValueConfiguration(metadataKeyValue, countersConfigurations);
  const keyCounter = getMetadataKeyValueConfiguration({
    key: metadataKeyValue.key
  }, countersConfigurations);
  if (!itemCounter || !keyCounter) return {
    counter: null,
    result: 0
  };
  const itemCount = calculateSumItemsInMetadata(itemCounter.members, itemCounter.items, metadata);
  const keyCount = calculateSumItemsInMetadata(keyCounter.members, keyCounter.items, metadata);
  return {
    result: keyCount === 0 ? 0 : itemCount / keyCount * 100,
    counter: itemCounter ?? keyCounter,
    metadataKey: metadataKeyValue.key,
    value: metadataKeyValue.value
  };
}
function percentOfAllItemsCalculation(metadataKeyValue, metadata, _, countersConfigurations, totalItems) {
  if (!metadataKeyValue) return {
    counter: null,
    result: 0
  };
  const itemCounter = getMetadataKeyValueConfiguration(metadataKeyValue, countersConfigurations);
  const keyCounter = getMetadataKeyValueConfiguration({
    key: CUSTOM_METADATA_KEY
  }, countersConfigurations);
  if (!itemCounter || !keyCounter) return {
    counter: null,
    result: 0
  };
  const itemCount = calculateSumItemsInMetadata(itemCounter.members, itemCounter.items, metadata);
  const keyCount = calculateSumItemsInMetadata(keyCounter.members, keyCounter.items, metadata);
  let result;
  if (metadataKeyValue.key === CUSTOM_METADATA_KEY) {
    result = totalItems === 0 ? 0 : itemCount / totalItems * 100;
  } else {
    result = keyCount === 0 ? 0 : itemCount / keyCount * 100;
  }
  return {
    result: result,
    counter: itemCounter ?? keyCounter,
    metadataKey: metadataKeyValue.key,
    value: metadataKeyValue.value
  };
}
function trendCalculation(metadataKeyValue, _, trends, countersConfigurations) {
  if (!metadataKeyValue || trends.length < 2) return {
    counter: null,
    result: 0
  };
  const itemCounter = getMetadataKeyValueConfiguration(metadataKeyValue, countersConfigurations);
  const keyCounter = getMetadataKeyValueConfiguration({
    key: metadataKeyValue.key
  }, countersConfigurations);
  if (!itemCounter || !keyCounter) return {
    counter: null,
    result: 0
  };
  const trendsCounts = trends.map(trend => calculateSumItemsInMetadata(itemCounter.members, itemCounter.items, trend.metadata));
  const latestTrend = trendsCounts.at(0) ?? 0;
  const previousTrendsSum = sum(trendsCounts.slice(1, trendsCounts.length));
  const previousTrendsMean = previousTrendsSum > 0 ? previousTrendsSum / trendsCounts.length - 1 : 0;
  const trend = percentageIncrease(previousTrendsMean, latestTrend);
  return {
    result: trend,
    counter: itemCounter ?? keyCounter,
    metadataKey: metadataKeyValue.key,
    value: metadataKeyValue.value
  };
}
function percentOfAllUniqueItemsCalculation(metadataKeyValue, metadata, trend, countersConfigurations, totalItems, totalUniqueItemsStats, _, uniquePropertyName) {
  if (!uniquePropertyName) return percentOfAllItemsCalculation(metadataKeyValue, metadata, trend, countersConfigurations, totalItems);
  if (!metadataKeyValue || !totalUniqueItemsStats || !uniquePropertyName) return {
    counter: null,
    result: 0
  };
  const itemCounter = getMetadataKeyValueConfiguration(metadataKeyValue, countersConfigurations);
  if (!itemCounter) return {
    counter: null,
    result: 0
  };
  const itemCount = calculateSumItemsInMetadata(itemCounter.members, itemCounter.items, metadata);
  const keyCount = totalUniqueItemsStats.unique_values_count.find(uv => uv.meta_key === uniquePropertyName)?.unique_values_count ?? 0;
  const result = keyCount === 0 ? 0 : itemCount / keyCount * 100;
  return {
    result: result,
    counter: itemCounter,
    metadataKey: metadataKeyValue.key,
    value: metadataKeyValue.value
  };
}
function totalUniqueItemsCalculation(metadataKeyValue, metadata, trend, countersConfigurations, totalItems, _, uniqueItemsStats, uniquePropertyName) {
  if (!uniquePropertyName) return totalSumCalculation(metadataKeyValue, metadata, trend, countersConfigurations);
  if (!metadataKeyValue || !uniqueItemsStats) return {
    counter: null,
    result: 0
  };
  let itemCounter;
  if (metadataKeyValue.key === CUSTOM_METADATA_KEY) {
    itemCounter = getMetadataKeyValueConfiguration({
      key: uniquePropertyName
    }, countersConfigurations);
  } else {
    itemCounter = getMetadataKeyValueConfiguration(metadataKeyValue, countersConfigurations);
  }
  if (!itemCounter) return {
    counter: null,
    result: 0
  };
  const keyCount = uniqueItemsStats.unique_values_count.find(uv => uv.meta_key === (metadataKeyValue.key === CUSTOM_METADATA_KEY ? uniquePropertyName : metadataKeyValue.key))?.unique_values_count ?? 0;
  return {
    result: keyCount,
    counter: itemCounter,
    metadataKey: metadataKeyValue.key,
    value: metadataKeyValue.value
  };
}
function getMetadataKeyValueGroups(metadataKeyValue, countersConfigurations) {
  const counter = getMetadataKeyValueConfiguration(metadataKeyValue, countersConfigurations);
  return getCounterGroups(counter);
}
function getCounterGroups(counter) {
  if (!counter) return [];
  return counter.items?.filter(group => group.isGroup ?? false) ?? [];
}
function hasMultipleMembers(metadataKeyValue, countersConfigurations) {
  const counter = getMetadataKeyValueConfiguration(metadataKeyValue, countersConfigurations);
  if (!counter) return false;
  if (counter.isGroup) return true;
  let sum = 0;
  counter.members?.forEach(member => sum += member.values?.length ?? 2);
  return sum > 1;
}
function getMetadataKeyValueConfiguration(metadataKeyValue, countersConfigurations) {
  if (!metadataKeyValue) return null;
  const keyConfig = objectToLowerCase(countersConfigurations)[metadataKeyValue.key.toLowerCase()];
  if (!keyConfig) return null;
  if (!metadataKeyValue.value) return keyConfig;
  const memberConfig = keyConfig.items?.find(group => group.label?.toLowerCase() === metadataKeyValue.value?.toLowerCase());
  if (!memberConfig || !memberConfig.display) return keyConfig;
  return memberConfig;
}
function calculateSumItemsInMetadata(members, groups, metadata) {
  const items = getValuesAndCounts(members, groups, metadata);
  return sum(items.map(({
    count
  }) => count));
}
function getMaxItemValue(members, groups, metadata) {
  const items = getValuesAndCounts(members, groups, metadata);
  if (items.length === 0) return {
    count: 0
  };
  return items.reduce((prev, current) => prev.count > current.count ? prev : current);
}
function getValuesAndCounts(members, groups, metadata) {
  const lowerKeysMetadata = toLowerKeys(metadata);
  const membersData = filterRelevantValues(members, lowerKeysMetadata);
  if (membersData.length > 0) return membersData;
  return filterRelevantValues(groups?.flatMap(group => group.members ?? []) ?? [], lowerKeysMetadata);
}
function filterRelevantValues(members, metadata) {
  return members?.flatMap(member => {
    const values = metadata[member.metadataName ?? ''];
    if (!values) return [];
    return (member.values === undefined ? values : values.filter(k => member.values?.includes(k.value))).map(({
      value,
      count
    }) => {
      return {
        metadata: member.metadataName,
        value: value,
        count: count
      };
    });
  }) ?? [];
}
function getItemCounterConfiguration(metadata, value, countersConfigurations) {
  if (!metadata || !value) return null;
  const counterConfig = countersConfigurations[metadata];
  const valueCounterConfig = counterConfig.items?.find(group => group.label === value);
  if (!valueCounterConfig || !valueCounterConfig.display) return null;
  return valueCounterConfig;
}

function LabelDisplay({
  metadataKey,
  value,
  countersConfiguration,
  labelClicked,
  tooltip = '',
  width,
  maxWidth = '10ch',
  color
}) {
  const config = getMetadataKeyValueConfiguration({
    key: metadataKey,
    value: value
  }, countersConfiguration);
  return /*#__PURE__*/React.createElement("span", {
    "data-for": "global",
    "data-tip": tooltip !== '' ? tooltip + ': ' + value : value,
    className: "flex items-center cursor-pointer text-gray-500 hover:text-gray-300",
    style: {
      color
    },
    onClick: () => labelClicked(metadataKey, value)
  }, config && config.display && config.display.icon !== null && /*#__PURE__*/React.createElement("span", {
    className: `w-[1em] h-[1em] ${config.display.color === 'green' ? 'text-emerald-400' : config.display.color === 'red' ? 'text-red-400' : 'text-gray-500 dark:text-gray-300'}`
  }, config.display.icon), /*#__PURE__*/React.createElement("span", {
    style: {
      width,
      maxWidth,
      marginLeft: metadataKey === 'topic' ? '-1px' : '1px'
    },
    className: "italic truncate pr-1"
  }, value));
}

function LabelsFiltersSelect({
  selectedLabels,
  labelFilterDeleted,
  countersConfigurations,
  selectedMetadataKeyValueChange,
  filterOnlySkills
}) {
  const [position, setPosition] = React.useState(false);
  const ref = React.useRef(null);
  const onClick = () => {
    if (typeof window !== 'undefined') {
      setTimeout(() => {
        if (ref.current) {
          //@ts-ignore
          const popover = ref.current.getBoundingClientRect();
          if (popover.bottom >= window.innerHeight) {
            setPosition(true);
          } else {
            setPosition(false);
          }
        }
      }, 100);
    }
  };
  return /*#__PURE__*/React.createElement(react.Listbox, null, /*#__PURE__*/React.createElement("div", {
    className: "relative"
  }, /*#__PURE__*/React.createElement(react.Listbox.Button, {
    className: "relative rounded-lg  py-2 pl-3 pr-10 text-left focus:outline-none sm:text-sm",
    onClick: onClick
  }, /*#__PURE__*/React.createElement("span", {
    className: "block truncate lowercase first-letter:uppercase text-black dark:text-white !text-xl",
    style: {
      width: '1em',
      height: '1em'
    }
  }, /*#__PURE__*/React.createElement(outline.FunnelIcon, null)), /*#__PURE__*/React.createElement("span", {
    className: "pointer-events-none absolute inset-y-0 right-0 flex items-center pr-2"
  }, /*#__PURE__*/React.createElement(outline.ChevronUpDownIcon, {
    className: "h-5 w-5 text-gray-400",
    "aria-hidden": "true"
  }))), /*#__PURE__*/React.createElement(react.Transition, {
    as: React.Fragment,
    leave: "transition ease-in duration-100",
    leaveFrom: "opacity-100",
    leaveTo: "opacity-0"
  }, /*#__PURE__*/React.createElement(react.Listbox.Options, {
    ref: ref,
    className: `fixed ${position ? '-mt-[200px]' : 'mt-3'} z-10 max-h-60 scrollbar-thin scrollbar-thumb-[#747189] dark:scrollbar-track-[#272533] 
          overflow-y-scroll scrollbar-thumb-rounded-full scrollbar-track-rounded-full rounded-md bg-white
          dark:bg-[#272533] py-1 text-base shadow-lg ring-1 ring-gray-500 dark:ring-black ring-opacity-5 focus:outline-none sm:text-sm`
  }, uniqBy(Object.keys(countersConfigurations).sort((a, b) => a < b ? -1 : a > b ? 1 : 0), key => key).filter(meta => {
    if (filterOnlySkills) {
      return (countersConfigurations[meta]?.items?.length ?? 0) > 0 && skillsArray.includes(meta);
    }
    return (countersConfigurations[meta]?.items?.length ?? 0) > 0;
  }).map((key, i) => /*#__PURE__*/React.createElement(CascadedOption$1, {
    countersConfigurations: countersConfigurations,
    optionName: key,
    index: i,
    selected: selectedLabels ?? [],
    labelFilterDeleted: labelFilterDeleted,
    labelClicked: selectedMetadataKeyValueChange,
    key: i
  }))))));
}
function CascadedOption$1({
  countersConfigurations,
  optionName,
  index,
  selected,
  labelClicked,
  labelFilterDeleted
}) {
  const keySelected = selected.some(s => s.key === optionName);
  const [opened, setOpened] = React.useState(keySelected);
  const configData = countersConfigurations[optionName];
  const onClickLabel = e => {
    e.preventDefault();
    setOpened(!opened);
  };
  return /*#__PURE__*/React.createElement(React.Fragment, {
    key: index
  }, configData.items && configData.items.length > 0 ? /*#__PURE__*/React.createElement("button", {
    type: "button",
    className: "ml-auto w-full pr-3 flex items-center justify-between hover:bg-gray-400 hover:dark:bg-[#444154]",
    onClick: onClickLabel
  }, /*#__PURE__*/React.createElement(MetadataTitle, {
    label: optionName,
    key: optionName,
    setOpened: setOpened
  }), opened ? /*#__PURE__*/React.createElement(outline.ChevronUpIcon, {
    className: "h-4 w-4 text-gray-500 dark:text-gray-300"
  }) : /*#__PURE__*/React.createElement(outline.ChevronDownIcon, {
    className: "h-4 w-4 text-gray-500 dark:text-gray-300"
  }), /*#__PURE__*/React.createElement("span", {
    className: "sr-only"
  }, "Open or Close metadata")) : null, opened ? uniqBy(configData.items ?? [], group => group.label).map((group, i) => {
    return /*#__PURE__*/React.createElement(DropdownOption$4, {
      label: group.label ?? '',
      value: {
        key: optionName,
        value: group.label
      },
      selected: selected.some(keyVal => keyVal.key === optionName && keyVal.value === group.label),
      labelFilters: selected,
      labelClicked: labelClicked,
      labelFilterDeleted: labelFilterDeleted,
      pl: 2,
      key: optionName + group.label + i.toString()
    });
  }) : null);
}
function MetadataTitle({
  label,
  setOpened
}) {
  const onClickLabel = e => {
    e.preventDefault();
    setOpened(true);
  };
  return /*#__PURE__*/React.createElement(react.Listbox.Option, {
    style: {
      paddingLeft: '1rem'
    },
    className: 'relative select-none py-2 pr-4 cursor-pointer',
    onClick: onClickLabel,
    value: label
  }, /*#__PURE__*/React.createElement("span", {
    className: "w-full flex"
  }, /*#__PURE__*/React.createElement("div", {
    className: "flex items-center"
  }, /*#__PURE__*/React.createElement("label", {
    className: `text-sm ml-2 font-medium text-gray-900 dark:text-gray-300 block truncate lowercase first-letter:uppercase cursor-pointer`
  }, label))));
}
function DropdownOption$4({
  label,
  value,
  selected,
  labelFilters,
  labelClicked,
  labelFilterDeleted,
  pl = 1
}) {
  return /*#__PURE__*/React.createElement(react.Listbox.Option, {
    style: {
      paddingLeft: pl + 'rem'
    },
    className: ({
      active
    }) => `relative cursor-default select-none py-2 pr-4 ${active ? 'bg-gray-400 dark:bg-[#444154] text-white' : 'text-gray-300 dark:text-[#747189]'}`,
    onClick: () => {
      if (selected) {
        const index = labelFilters.findIndex(s => s.key === value.key && s.value === value.value);
        if (index >= 0) {
          labelFilterDeleted(index);
        }
        return;
      }
      labelClicked(value);
    },
    value: value
  }, /*#__PURE__*/React.createElement("span", {
    className: "w-full flex"
  }, /*#__PURE__*/React.createElement("div", {
    className: "flex items-center"
  }, /*#__PURE__*/React.createElement("input", {
    checked: selected,
    onChange: () => {},
    type: "checkbox",
    className: "w-4 h-4 text-[#4D4DFE] bg-gray-100 border-gray-300 focus:ring-[#4D4DFE] dark:ring-offset-gray-800 focus:ring-2 dark:bg-[#322F46] dark:border-[#322F46]"
  }), /*#__PURE__*/React.createElement("label", {
    className: `text-sm ml-2 font-medium text-gray-900 dark:text-gray-300 block truncate lowercase first-letter:uppercase ${selected ? 'font-medium' : 'font-normal'}`
  }, label))));
}

function SingleSelect$1({
  options,
  selectedLabel,
  onSelect,
  hideNavigationMenu = false
}) {
  return /*#__PURE__*/React.createElement(react.Listbox, null, /*#__PURE__*/React.createElement("div", {
    className: "relative"
  }, /*#__PURE__*/React.createElement(react.Listbox.Button, {
    className: "relative rounded-lg text-left focus:outline-none text-sm flex items-center"
  }, /*#__PURE__*/React.createElement("span", {
    className: "text-[#111111] dark:text-gray-300 text-sm mr-1"
  }, selectedLabel), hideNavigationMenu ? null : /*#__PURE__*/React.createElement("span", null, /*#__PURE__*/React.createElement(outline.ChevronDownIcon, {
    className: "h-4 w-4 text-gray-500 dark:text-gray-200",
    "aria-hidden": "true"
  }))), hideNavigationMenu ? null : /*#__PURE__*/React.createElement(react.Transition, {
    as: React.Fragment,
    leave: "transition ease-in duration-100",
    leaveFrom: "opacity-100",
    leaveTo: "opacity-0"
  }, /*#__PURE__*/React.createElement(react.Listbox.Options, {
    className: "fixed mt-1 z-10 p-1 max-h-60 scrollbar-thin scrollbar-thumb-[#747189] scrollbar-track-[#272533] overflow-y-scroll scrollbar-thumb-rounded-full scrollbar-track-rounded-full rounded-md bg-gray-200 dark:bg-[#272533] py-1 text-base shadow-lg ring-1 ring-gray-500 dark:ring-black ring-opacity-5 focus:outline-none sm:text-sm"
  }, options.map((key, i) => /*#__PURE__*/React.createElement(DropdownOption$3, {
    label: key,
    value: key,
    selected: selectedLabel === key,
    labelClicked: onSelect,
    key: i
  }))))));
}
function DropdownOption$3({
  label,
  value,
  selected,
  labelClicked
}) {
  return /*#__PURE__*/React.createElement(react.Listbox.Option, {
    className: ({
      active
    }) => `relative cursor-default select-none py-2 pr-4 ${active ? 'bg-gray-400 dark:bg-[#444154] text-white' : 'text-gray-300 dark:text-[#747189]'}`,
    onClick: () => labelClicked(value),
    value: value
  }, /*#__PURE__*/React.createElement("span", {
    className: "w-full flex"
  }, /*#__PURE__*/React.createElement("div", {
    className: "flex items-center"
  }, /*#__PURE__*/React.createElement("input", {
    checked: selected,
    onChange: () => {},
    type: "radio",
    className: "w-4 h-4 text-[#4D4DFE] bg-gray-100 border-gray-300 focus:ring-[#4D4DFE] dark:ring-offset-gray-800 focus:ring-2 dark:bg-[#322F46] dark:border-[#322F46]"
  }), /*#__PURE__*/React.createElement("label", {
    className: `text-sm ml-2 font-medium text-gray-900 dark:text-gray-300 block truncate lowercase first-letter:uppercase ${selected ? 'font-medium' : 'font-normal'}`
  }, label))));
}

const numberFormatter = /*#__PURE__*/new Intl.NumberFormat('en-US');
function getMetadataKeyValueDisplay(metadataKeyValue) {
  if (!metadataKeyValue.value) return metadataKeyValue.key;
  return `${metadataKeyValue.key}.${metadataKeyValue.value}`;
}
function getBackgroundColorLayers(colorAxis, metadata, trends, countersConfiguration, totalItems, totalUniqueItemsStats, uniqueItemsStats, uniquePropertyName) {
  return groupCounters(colorAxis, countersConfiguration).map(countersGroup => {
    const groups = countersGroup.map(counter => counter.calculationConfiguration.calculate(counter.metadataKeyValue, metadata, trends, countersConfiguration, totalItems, totalUniqueItemsStats, uniqueItemsStats, uniquePropertyName));
    if (!groups.some(group => group.result && group.result > 0)) return '';
    let backgroundString = '';
    let totalPercentage = 0;
    groups.forEach(group => {
      backgroundString = backgroundString + `,${group.counter?.display?.color ?? 'rgba(255,0,0,0)'} ${totalPercentage}% ${totalPercentage + group.result}%`;
      totalPercentage += group.result;
    });
    backgroundString = `${backgroundString},rgba(255,0,0,0) ${totalPercentage}% ${100 - totalPercentage}%`;
    return `linear-gradient(90deg${backgroundString})`;
  }).filter(str => str !== '');
}
function groupCounters(counters, countersConfiguration) {
  return counters.reduce((groups, counter) => {
    const index = groups.findIndex(group => group.length > 0 && areCountersCanBeGrouped(counter, group.at(0), countersConfiguration));
    if (index !== -1) {
      groups[index].push(counter);
    } else {
      groups.push([counter]);
    }
    return groups;
  }, []);
}
function areCountersCanBeGrouped(firstCounter, secondCounter, countersConfiguration) {
  if (!firstCounter.metadataKeyValue || !secondCounter || !secondCounter.metadataKeyValue) return false;
  if (firstCounter.calculationConfiguration.name !== secondCounter.calculationConfiguration.name) return false;
  return areKeyValuesSameLevel(firstCounter.metadataKeyValue, secondCounter.metadataKeyValue, countersConfiguration);
}
function areKeyValuesSameLevel(firstKeyValue, secondKeyValue, countersConfiguration) {
  if (firstKeyValue.key !== secondKeyValue.key) return false;
  const firstKeyValueHasGroups = getMetadataKeyValueGroups(firstKeyValue, countersConfiguration).length > 0;
  const secondKeyValueHasGroups = getMetadataKeyValueGroups(secondKeyValue, countersConfiguration).length > 0;
  if (firstKeyValueHasGroups !== secondKeyValueHasGroups) return false;
  const firstKeyValueHasMultipleMembers = hasMultipleMembers(firstKeyValue, countersConfiguration);
  const secondKeyValueHasMultipleMembers = hasMultipleMembers(secondKeyValue, countersConfiguration);
  if (firstKeyValueHasMultipleMembers !== secondKeyValueHasMultipleMembers) return false;
  return true;
}

function CounterDisplay({
  counter,
  metadata,
  countersConfiguration,
  trends,
  totalItems,
  width,
  maxWidth = '6ch',
  totalUniqueItemsStats,
  uniqueItemsStats,
  uniquePropertyName,
  hideSignals
}) {
  const metadataKeyValue = counter.metadataKeyValue;
  if (!metadataKeyValue) return /*#__PURE__*/React.createElement(React.Fragment, null);
  const displayResult = counter.calculationConfiguration.calculate(metadataKeyValue, metadata, trends, countersConfiguration, totalItems, totalUniqueItemsStats, uniqueItemsStats, uniquePropertyName);
  if (!displayResult.counter) return /*#__PURE__*/React.createElement(React.Fragment, null);
  const keysToHide = ['positive', 'negative'];
  if (hideSignals === true && keysToHide.includes(displayResult.metadataKey || '')) {
    return /*#__PURE__*/React.createElement(React.Fragment, null);
  }
  return /*#__PURE__*/React.createElement("span", {
    "data-tip": `${getMetadataKeyValueDisplay(metadataKeyValue)} - ${counter.calculationConfiguration.name}` + getMetadataValueTitle(displayResult.metadataKey, displayResult.value) + ' - ' + numberFormatter.format(customNumberToFixed(displayResult.result)),
    "data-for": "global",
    className: `flex items-center text-sm ${counter.calculationConfiguration.type !== 'trend' && displayResult.result < 1 ? 'text-gray-500 dark:text-gray-300' : displayResult.counter.display ? displayResult.counter.display.color === 'green' ? 'text-emerald-400' : displayResult.counter.display.color === 'red' ? 'text-red-400' : 'text-gray-500 dark:text-gray-300' : 'text-gray-500 dark:text-gray-300'}`
  }, counter.calculationConfiguration.type === 'trend' && displayResult.result > 0 ? /*#__PURE__*/React.createElement("span", {
    className: "text-emerald-400",
    style: {
      width: '1em',
      height: '1em'
    }
  }, /*#__PURE__*/React.createElement(solid.ArrowUpIcon, null)) : counter.calculationConfiguration.type === 'trend' && displayResult.result < 0 ? /*#__PURE__*/React.createElement("span", {
    className: "text-red-400",
    style: {
      width: '1em',
      height: '1em'
    }
  }, /*#__PURE__*/React.createElement(solid.ArrowDownIcon, null)) : counter.calculationConfiguration.type === 'trend' && /*#__PURE__*/React.createElement("span", {
    className: "text-gray-300",
    style: {
      width: '1em',
      height: '1em'
    }
  }, /*#__PURE__*/React.createElement(solid.ArrowsUpDownIcon, null)), displayResult.counter.display && displayResult.counter.display.icon !== null && /*#__PURE__*/React.createElement("span", {
    style: {
      width: '1em',
      height: '1em'
    }
  }, displayResult.counter.display.icon), /*#__PURE__*/React.createElement("span", {
    className: `${counter.calculationConfiguration.type === 'trend' ? displayResult.result > 0 ? 'text-emerald-400' : displayResult.result < 0 ? 'text-red-400' : 'text-gray-500 dark:text-gray-300' : ''}`,
    style: {
      width,
      maxWidth
    }
  }, getNumberDescription(displayResult.result), counter.calculationConfiguration.type === 'percentage' || counter.calculationConfiguration.type === 'trend' ? '%' : null));
}
function getMetadataValueTitle(metadata, value) {
  if (metadata) {
    if (value) return `(${metadata}.${value})`;
    return `(${metadata})`;
  }
  if (value) {
    return `(${value})`;
  }
  return '';
}

function MaxLabelDisplay({
  metadataKey,
  metadata,
  countersConfiguration,
  labelClicked,
  width,
  maxWidth
}) {
  const lowerKeysMetadata = toLowerKeys(metadata);
  const meta = lowerKeysMetadata[metadataKey.toLowerCase()]?.reduce((prev, current) => prev.count > current.count ? prev : current);
  if (!meta) return /*#__PURE__*/React.createElement(React.Fragment, null);
  return /*#__PURE__*/React.createElement(LabelDisplay, {
    metadataKey: metadataKey,
    value: meta.value,
    countersConfiguration: countersConfiguration,
    labelClicked: labelClicked,
    width: width,
    maxWidth: maxWidth
  });
}

function CountersLabelsDisplay({
  counters,
  labels,
  metadata,
  countersConfiguration,
  trends,
  labelClicked,
  counterWidth,
  counterMaxWidth,
  labelWidth,
  labelMaxWidth,
  totalItems,
  totalUniqueItemsStats,
  uniqueItemsStats,
  uniquePropertyName,
  hideSignals
}) {
  return /*#__PURE__*/React.createElement("span", {
    className: "truncate flex items-center"
  }, counters.filter(counter => counter.metadataKeyValue !== null).map((counter, i) => /*#__PURE__*/React.createElement("div", {
    key: i,
    className: "mr-2"
  }, /*#__PURE__*/React.createElement(CounterDisplay, {
    counter: counter,
    countersConfiguration: countersConfiguration,
    metadata: metadata,
    trends: trends,
    width: counterWidth,
    maxWidth: counterMaxWidth,
    totalItems: totalItems,
    totalUniqueItemsStats: totalUniqueItemsStats,
    uniqueItemsStats: uniqueItemsStats,
    uniquePropertyName: uniquePropertyName,
    hideSignals: hideSignals
  }))), labels.map((label, i) => /*#__PURE__*/React.createElement("div", {
    key: i,
    className: "mr-1"
  }, /*#__PURE__*/React.createElement(MaxLabelDisplay, {
    countersConfiguration: countersConfiguration,
    metadataKey: label,
    labelClicked: labelClicked,
    metadata: metadata,
    width: labelWidth,
    maxWidth: labelMaxWidth
  }))));
}

function DropdownOption$2({
  label,
  value,
  pl = 1
}) {
  return /*#__PURE__*/React.createElement(react.Listbox.Option, {
    style: {
      paddingLeft: pl + 'rem'
    },
    className: ({
      active
    }) => `relative cursor-default select-none py-2 pr-4 ${active ? 'bg-gray-400 dark:bg-[#444154] text-white' : 'text-gray-300 dark:text-[#747189]'}`,
    value: value
  }, ({
    selected
  }) => /*#__PURE__*/React.createElement("span", {
    className: "w-full flex"
  }, /*#__PURE__*/React.createElement("div", {
    className: "flex items-center"
  }, /*#__PURE__*/React.createElement("input", {
    checked: selected,
    onChange: () => {},
    type: "radio",
    className: "w-4 h-4 text-[#4D4DFE] bg-gray-100 border-gray-300 focus:ring-[#4D4DFE] dark:ring-offset-gray-800 focus:ring-2 dark:bg-[#322F46] dark:border-[#322F46]"
  }), /*#__PURE__*/React.createElement("label", {
    className: `text-sm ml-2 font-medium text-gray-900 dark:text-gray-300 block truncate lowercase first-letter:uppercase ${selected ? 'font-medium' : 'font-normal'}`
  }, label))));
}

function KeyValueSingleSelect({
  metadataKeyValue,
  countersConfigurations,
  selectedMetadataKeyValueChange,
  placeholder
}) {
  return /*#__PURE__*/React.createElement(react.Listbox, {
    value: metadataKeyValue,
    onChange: selectedMetadataKeyValueChange,
    by: (a, b) => a?.key === b?.key && a?.value === b?.value
  }, /*#__PURE__*/React.createElement("div", {
    className: "relative"
  }, /*#__PURE__*/React.createElement(react.Listbox.Button, {
    className: "relative rounded-lg bg-gray-200 dark:bg-[#272535] py-2 pl-3 pr-10 text-left focus:outline-none focus-visible:border-indigo-500 focus-visible:ring-2 focus-visible:ring-white focus-visible:ring-opacity-75 focus-visible:ring-offset-2 focus-visible:ring-offset-orange-300 sm:text-sm"
  }, /*#__PURE__*/React.createElement("span", {
    className: "block truncate lowercase first-letter:uppercase text-gray-700 dark:text-white"
  }, metadataKeyValue ? getMetadataKeyValueDisplay(metadataKeyValue) : placeholder), /*#__PURE__*/React.createElement("span", {
    className: "pointer-events-none absolute inset-y-0 right-0 flex items-center pr-2"
  }, /*#__PURE__*/React.createElement(solid.ChevronUpDownIcon, {
    className: "h-5 w-5 text-gray-400",
    "aria-hidden": "true"
  }))), /*#__PURE__*/React.createElement(react.Transition, {
    as: React.Fragment,
    leave: "transition ease-in duration-100",
    leaveFrom: "opacity-100",
    leaveTo: "opacity-0"
  }, /*#__PURE__*/React.createElement(react.Listbox.Options, {
    className: "fixed mt-1 z-10 max-h-60 scrollbar-thin scrollbar-thumb-[#747189] scrollbar-track-[#272533] overflow-y-scroll scrollbar-thumb-rounded-full scrollbar-track-rounded-full rounded-md bg-gray-200 dark:bg-[#272533] py-1 text-base shadow-lg ring-1 ring-gray-500 dark:ring-black ring-opacity-5 focus:outline-none sm:text-sm"
  }, uniqBy(Object.keys(countersConfigurations).sort((a, b) => a < b ? -1 : a > b ? 1 : 0), key => key).map((key, i) => /*#__PURE__*/React.createElement(CascadedOption, {
    countersConfigurations: countersConfigurations,
    optionName: key,
    index: i,
    selected: (metadataKeyValue?.key ?? '') === key && metadataKeyValue?.value !== undefined,
    key: i
  }))))));
}
function CascadedOption({
  countersConfigurations,
  optionName,
  index,
  selected
}) {
  const [opened, setOpened] = React.useState(selected);
  const configData = countersConfigurations[optionName];
  return /*#__PURE__*/React.createElement(React.Fragment, {
    key: index
  }, /*#__PURE__*/React.createElement("div", {
    className: "w-full flex"
  }, /*#__PURE__*/React.createElement(DropdownOption$2, {
    label: optionName,
    value: {
      key: optionName
    },
    key: optionName
  }), configData.items && configData.items.length > 0 && /*#__PURE__*/React.createElement("button", {
    type: "button",
    className: "ml-auto mr-3",
    onClick: () => setOpened(opened => !opened)
  }, opened ? /*#__PURE__*/React.createElement(solid.ChevronUpIcon, {
    className: "h-4 w-4 text-gray-500 dark:text-gray-300 "
  }) : /*#__PURE__*/React.createElement(solid.ChevronDownIcon, {
    className: "h-4 w-4 text-gray-500 dark:text-gray-300 "
  }), /*#__PURE__*/React.createElement("span", {
    className: "sr-only"
  }, "Open or Close metadata"))), opened && uniqBy(configData.items ?? [], group => group.label).map((group, i) => {
    return /*#__PURE__*/React.createElement(DropdownOption$2, {
      label: group.label ?? '',
      value: {
        key: optionName,
        value: group.label
      },
      pl: 2,
      key: optionName + group.label + i.toString()
    });
  }));
}

const totalSumCalculationName = 'Total SUM';
const topValueCalculationName = 'Top value total sum';
const topValuePercentCalculationName = 'Top value %';
const topGroupCalculationName = 'Top group total';
const topGroupPercentCalculationName = 'Top group %';
const percentOfItemsCalculationName = '% of items';
const percentOfAllCalculationName = '% of all items';
const trendCalculationName = 'trend';
const percentOfTotalUniqueItemsCalculationName = '% of all unique items';
const totalUniqueItemsCalculationName = 'Total unique items';
const totalSumCalculationConfiguration = {
  name: totalSumCalculationName,
  hasGroups: false,
  hasMultipleMembers: false,
  type: 'number',
  calculate: totalSumCalculation
};
const topValueCalculationConfiguration = {
  name: topValueCalculationName,
  hasGroups: false,
  hasMultipleMembers: true,
  type: 'number',
  calculate: topValueCalculation
};
const topValuePercentCalculationConfiguration = {
  name: topValuePercentCalculationName,
  hasGroups: false,
  hasMultipleMembers: true,
  type: 'percentage',
  calculate: topValuePercentCalculation
};
const topGroupCalculationConfiguration = {
  name: topGroupCalculationName,
  hasGroups: true,
  hasMultipleMembers: false,
  type: 'number',
  calculate: topGroupCalculation
};
const topGroupPercentCalculationConfiguration = {
  name: topGroupPercentCalculationName,
  hasGroups: true,
  hasMultipleMembers: false,
  type: 'percentage',
  calculate: topGroupPercentCalculation
};
const percentOfItemsCalculationConfiguration = {
  name: percentOfItemsCalculationName,
  hasGroups: false,
  hasMultipleMembers: false,
  type: 'percentage',
  calculate: percentOfItemsCalculation
};
const percentOfAllItemsCalculationConfiguration = {
  name: percentOfAllCalculationName,
  hasGroups: false,
  hasMultipleMembers: false,
  type: 'percentage',
  calculate: percentOfAllItemsCalculation
};
const trendCalculationConfiguration = {
  name: trendCalculationName,
  hasGroups: false,
  hasMultipleMembers: false,
  type: 'trend',
  calculate: trendCalculation
};
const percentOfTotalUniqueItemsCalculationConfiguration = {
  name: percentOfTotalUniqueItemsCalculationName,
  hasGroups: false,
  hasMultipleMembers: false,
  type: 'percentage',
  calculate: percentOfAllUniqueItemsCalculation
};
const totalUniqueItemsCalculationConfiguration = {
  name: totalUniqueItemsCalculationName,
  hasGroups: false,
  hasMultipleMembers: false,
  type: 'number',
  calculate: totalUniqueItemsCalculation
};

function Counters({
  countersConfigurations,
  calculationsConfigurations,
  currentCounters,
  countersChanged,
  addCounterText,
  title
}) {
  const addCounter = newMetadataKeyValue => {
    const allowedCalculations = getCalculationTypes(calculationsConfigurations, newMetadataKeyValue, countersConfigurations);
    countersChanged([...currentCounters, {
      metadataKeyValue: newMetadataKeyValue,
      calculationConfiguration: getDefaultCalculation(allowedCalculations)
    }]);
  };
  const deleteCounter = index => {
    currentCounters.splice(index, 1);
    countersChanged([...currentCounters]);
  };
  const updateCounter = (index, updatedCounter) => {
    currentCounters[index] = updatedCounter;
    countersChanged([...currentCounters]);
  };
  return /*#__PURE__*/React.createElement("div", {
    className: "w-full"
  }, /*#__PURE__*/React.createElement("p", {
    className: "text-xl text-gray-500 dark:text-white mb-1"
  }, title), /*#__PURE__*/React.createElement("div", {
    className: "flex w-full flex-wrap items-center"
  }, currentCounters.map((addedCounter, i) => /*#__PURE__*/React.createElement("div", {
    className: "ml-1 mb-1",
    key: i
  }, /*#__PURE__*/React.createElement(Counter, {
    counterData: addedCounter,
    countersConfigurations: countersConfigurations,
    calculationsConfigurations: calculationsConfigurations,
    counterDeleted: () => deleteCounter(i),
    counterChanged: counter => updateCounter(i, counter)
  }))), /*#__PURE__*/React.createElement("div", {
    className: "ml-1"
  }, /*#__PURE__*/React.createElement(KeyValueSingleSelect, {
    metadataKeyValue: null,
    countersConfigurations: countersConfigurations,
    selectedMetadataKeyValueChange: addCounter,
    placeholder: addCounterText
  }))));
}
function Counter({
  counterData,
  countersConfigurations,
  calculationsConfigurations,
  counterDeleted,
  counterChanged
}) {
  const selectedMetadataKeyValueChangeHandler = newMetadataKeyValue => {
    const allowedCalculations = getCalculationTypes(calculationsConfigurations, newMetadataKeyValue, countersConfigurations);
    counterChanged({
      metadataKeyValue: newMetadataKeyValue,
      calculationConfiguration: allowedCalculations.some(calc => calc.name === counterData.calculationConfiguration.name) ? counterData.calculationConfiguration : getDefaultCalculation(allowedCalculations)
    });
  };
  const counterChangeHandler = newCounter => {
    counterChanged({
      metadataKeyValue: counterData.metadataKeyValue,
      calculationConfiguration: newCounter
    });
  };
  return /*#__PURE__*/React.createElement("div", {
    className: "w-full"
  }, /*#__PURE__*/React.createElement("div", {
    className: "flex bg-gray-200 dark:bg-[#272535] p-1 w-fit"
  }, /*#__PURE__*/React.createElement(KeyValueSingleSelect, {
    metadataKeyValue: counterData.metadataKeyValue,
    countersConfigurations: countersConfigurations,
    selectedMetadataKeyValueChange: selectedMetadataKeyValueChangeHandler,
    placeholder: "Select"
  }), /*#__PURE__*/React.createElement(react.Listbox, {
    value: counterData.calculationConfiguration,
    onChange: counterChangeHandler,
    by: "name"
  }, /*#__PURE__*/React.createElement("div", {
    className: "relative ml-1"
  }, /*#__PURE__*/React.createElement(react.Listbox.Button, {
    className: "relative rounded-lg bg-gray-200 dark:bg-[#272535] py-2 pl-3 pr-10 text-left focus:outline-none focus-visible:border-indigo-500 focus-visible:ring-2 focus-visible:ring-white focus-visible:ring-opacity-75 focus-visible:ring-offset-2 focus-visible:ring-offset-orange-300 sm:text-sm"
  }, /*#__PURE__*/React.createElement("span", {
    className: "block truncate lowercase first-letter:uppercase text-gray-700 dark:text-white"
  }, counterData.calculationConfiguration.name), /*#__PURE__*/React.createElement("span", {
    className: "pointer-events-none absolute inset-y-0 right-0 flex items-center pr-2"
  }, /*#__PURE__*/React.createElement(solid.ChevronUpDownIcon, {
    className: "h-5 w-5 text-gray-400",
    "aria-hidden": "true"
  }))), /*#__PURE__*/React.createElement(react.Transition, {
    as: React.Fragment,
    leave: "transition ease-in duration-100",
    leaveFrom: "opacity-100",
    leaveTo: "opacity-0"
  }, /*#__PURE__*/React.createElement(react.Listbox.Options, {
    className: "fixed mt-1 z-10 max-h-60 scrollbar-thin scrollbar-thumb-[#747189] scrollbar-track-[#272533] overflow-y-scroll scrollbar-thumb-rounded-full scrollbar-track-rounded-full rounded-md bg-gray-200 dark:bg-[#272533] py-1 text-base shadow-lg ring-1 ring-gray-500 dark:ring-black ring-opacity-5 focus:outline-none sm:text-sm"
  }, getCalculationTypes(calculationsConfigurations, counterData.metadataKeyValue, countersConfigurations).sort(function (a, b) {
    const textA = a.name.toLowerCase();
    const textB = b.name.toLowerCase();
    return textA < textB ? -1 : textA > textB ? 1 : 0;
  }).map(counter => /*#__PURE__*/React.createElement(DropdownOption$2, {
    label: counter.name,
    value: counter,
    key: counter.name
  })))))), /*#__PURE__*/React.createElement("button", {
    type: "button",
    className: "ml-1",
    onClick: counterDeleted
  }, /*#__PURE__*/React.createElement(solid.XMarkIcon, {
    className: "h-4 w-4 text-gray-400 dark:text-gray-600 hover:scale-125 transition duration-100 ease-linear"
  }), /*#__PURE__*/React.createElement("span", {
    className: "sr-only"
  }, "Delete counter"))));
}
function getCalculationTypes(calculationTypes, selectedMetadataKeyValue, countersConfiguration) {
  if (!selectedMetadataKeyValue) return calculationTypes.filter(calc => !calc.hasGroups && !calc.hasMultipleMembers);
  const hasGroups = getMetadataKeyValueGroups(selectedMetadataKeyValue, countersConfiguration).length > 0;
  if (hasGroups) return calculationTypes;
  const keyValueHasMultipleMembers = hasMultipleMembers(selectedMetadataKeyValue, countersConfiguration);
  if (keyValueHasMultipleMembers) return calculationTypes.filter(calc => !calc.hasGroups);
  return calculationTypes.filter(calc => !calc.hasGroups && !calc.hasMultipleMembers);
}
function getDefaultCalculation(allowedCalculations) {
  return allowedCalculations.find(calc => calc.name === topGroupPercentCalculationName) ?? allowedCalculations.find(calc => calc.name === topValuePercentCalculationName) ?? allowedCalculations[0];
}

function Labels({
  currentLabels,
  labelsOptions,
  labelsChanged,
  title = 'Labels'
}) {
  const [selectedLabels, setSelectedLabels] = React.useState(currentLabels);
  const handleChange = newLabels => {
    setSelectedLabels(newLabels);
    labelsChanged(newLabels);
  };
  return /*#__PURE__*/React.createElement("div", {
    className: "w-full"
  }, /*#__PURE__*/React.createElement("p", {
    className: "text-xl text-gray-500 dark:text-white mb-1"
  }, title), /*#__PURE__*/React.createElement(react.Listbox, {
    value: selectedLabels,
    onChange: handleChange,
    multiple: true
  }, /*#__PURE__*/React.createElement("div", {
    className: "relative"
  }, /*#__PURE__*/React.createElement(react.Listbox.Button, {
    className: "relative rounded-lg bg-gray-200 dark:bg-[#272535] py-2 pl-3 pr-10 text-left shadow-md focus:outline-none focus-visible:border-indigo-500 focus-visible:ring-2 focus-visible:ring-white focus-visible:ring-opacity-75 focus-visible:ring-offset-2 focus-visible:ring-offset-orange-300 sm:text-sm"
  }, /*#__PURE__*/React.createElement("span", {
    className: "block truncate lowercase first-letter:uppercase dark:text-white text-gray-700"
  }, selectedLabels.length === 0 ? 'Select' : `${selectedLabels.length} labels selected`), /*#__PURE__*/React.createElement("span", {
    className: "pointer-events-none absolute inset-y-0 right-0 flex items-center pr-2"
  }, /*#__PURE__*/React.createElement(solid.ChevronUpDownIcon, {
    className: "h-5 w-5 text-gray-400",
    "aria-hidden": "true"
  }))), /*#__PURE__*/React.createElement(react.Transition, {
    as: React.Fragment,
    leave: "transition ease-in duration-100",
    leaveFrom: "opacity-100",
    leaveTo: "opacity-0"
  }, /*#__PURE__*/React.createElement(react.Listbox.Options, {
    className: "fixed mt-1 z-10 max-h-60 scrollbar-thin scrollbar-thumb-[#747189] scrollbar-track-[#272533] overflow-y-scroll scrollbar-thumb-rounded-full scrollbar-track-rounded-full rounded-md bg-gray-200 dark:bg-[#272533] py-1 text-base shadow-lg ring-1 ring-black ring-opacity-5 focus:outline-none sm:text-sm"
  }, uniqBy(labelsOptions, key => key).map(key => /*#__PURE__*/React.createElement(DropdownOption$1, {
    key: key,
    label: key,
    value: key
  })))))));
}
function DropdownOption$1({
  label,
  value,
  pl = 1
}) {
  return /*#__PURE__*/React.createElement(react.Listbox.Option, {
    style: {
      paddingLeft: pl + 'rem'
    },
    className: ({
      active
    }) => `relative cursor-default select-none py-2 pr-4 ${active ? 'bg-gray-400 dark:bg-[#444154] text-white' : 'text-gray-300 dark:text-[#747189]'}`,
    value: value
  }, ({
    selected
  }) => /*#__PURE__*/React.createElement("span", {
    className: "w-full flex"
  }, /*#__PURE__*/React.createElement("div", {
    className: "flex items-center"
  }, /*#__PURE__*/React.createElement("input", {
    checked: selected,
    onChange: () => {},
    type: "checkbox",
    className: "w-4 h-4 text-[#4D4DFE] bg-gray-100 border-gray-300 focus:ring-[#4D4DFE] dark:ring-offset-gray-800 focus:ring-2 dark:bg-[#322F46] dark:border-[#322F46]"
  }), /*#__PURE__*/React.createElement("label", {
    className: `text-sm ml-2 font-medium text-gray-900 dark:text-gray-300 block truncate lowercase first-letter:uppercase ${selected ? 'font-medium' : 'font-normal'}`
  }, label))));
}

function CustomizeTab({
  currentCounters,
  selectedLabels,
  countersConfigurations,
  calculationsConfigurations,
  labelsOptions,
  countersChanged,
  labelsChanged,
  currentColorsAxis,
  selectedSizeAxis,
  colorsAxisChanged,
  sizeAxisChanged
}) {
  const [position, setPosition] = React.useState(false);
  const ref = React.useRef(null);
  const onClick = () => {
    if (typeof window !== 'undefined') {
      setTimeout(() => {
        if (ref.current) {
          //@ts-ignore
          const popover = ref.current.getBoundingClientRect();
          if (popover.bottom >= window.innerHeight) {
            setPosition(true);
          } else {
            setPosition(false);
          }
        }
      }, 100);
    }
  };
  return /*#__PURE__*/React.createElement("div", {
    className: "max-w-sm"
  }, /*#__PURE__*/React.createElement(react.Popover, {
    className: "relative"
  }, ({
    open
  }) => /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(react.Popover.Button, {
    onClick: onClick,
    className: `
                ${open ? '' : 'text-opacity-90'}
                group inline-flex items-center rounded-md px-3 py-2 text-base font-medium text-gray-500 dark:text-black hover:text-opacity-100 focus:outline-none `
  }, /*#__PURE__*/React.createElement("span", {
    className: "text-[#747189] font-normal"
  }, "Customize"), /*#__PURE__*/React.createElement(solid.ChevronDownIcon, {
    className: `${open ? '' : 'text-opacity-70'}
                  ml-2 h-5 w-5 text-[#747189]  transition duration-150 ease-in-out group-hover:text-opacity-80`,
    "aria-hidden": "true"
  })), /*#__PURE__*/React.createElement(react.Transition, {
    as: React.Fragment,
    enter: "transition ease-out duration-200",
    enterFrom: "opacity-0 translate-y-1",
    enterTo: "opacity-100 translate-y-0",
    leave: "transition ease-in duration-150",
    leaveFrom: "opacity-100 translate-y-0",
    leaveTo: "opacity-0 translate-y-1"
  }, /*#__PURE__*/React.createElement(react.Popover.Panel, {
    ref: ref,
    className: `fixed z-10 ${position ? '-mt-[400px]' : 'mt-3'} w-screen max-w-sm transform lg:max-w-4xl`
  }, /*#__PURE__*/React.createElement("div", {
    className: "overflow-hidden rounded-lg shadow-lg ring-1 ring-gray-500 dark:ring-black ring-opacity-5"
  }, /*#__PURE__*/React.createElement("div", {
    className: "relative bg-white dark:bg-[#1D1C27] p-5 max-h-[85vh] text-gray-500 dark:text-white"
  }, /*#__PURE__*/React.createElement("div", {
    className: "w-full"
  }, /*#__PURE__*/React.createElement("p", {
    className: "text-xl mb-1"
  }, "Size Axis"), /*#__PURE__*/React.createElement(KeyValueSingleSelect, {
    metadataKeyValue: selectedSizeAxis,
    placeholder: "Select",
    countersConfigurations: countersConfigurations,
    selectedMetadataKeyValueChange: sizeAxisChanged
  })), /*#__PURE__*/React.createElement("div", {
    className: "w-full mt-2"
  }, /*#__PURE__*/React.createElement(Counters, {
    countersConfigurations: countersConfigurations,
    calculationsConfigurations: calculationsConfigurations.filter(calc => calc.type === 'percentage'),
    currentCounters: currentColorsAxis,
    countersChanged: colorsAxisChanged,
    addCounterText: "Add axis",
    title: "Color Axis"
  })), /*#__PURE__*/React.createElement("div", {
    className: "w-full mt-2"
  }, /*#__PURE__*/React.createElement(Counters, {
    countersConfigurations: countersConfigurations,
    calculationsConfigurations: calculationsConfigurations,
    currentCounters: currentCounters,
    countersChanged: countersChanged,
    addCounterText: "Add counter",
    title: "Counters"
  })), /*#__PURE__*/React.createElement("div", {
    className: "w-full mt-2"
  }, /*#__PURE__*/React.createElement(Labels, {
    currentLabels: selectedLabels,
    labelsOptions: labelsOptions.sort(),
    labelsChanged: labelsChanged
  })))))))));
}

function Datepicker({
  selectedDate,
  dateChanged,
  placeholder = 'Select Date'
}) {
  const DAYS = ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'];
  const [dayCount, setDayCount] = React.useState([]);
  const [blankDays, setBlankDays] = React.useState([]);
  const [showDatepicker, setShowDatepicker] = React.useState(false);
  const [datepickerHeaderDate, setDatepickerHeaderDate] = React.useState(new Date());
  const [type, setType] = React.useState('date');
  const decrement = () => {
    switch (type) {
      case 'date':
        setDatepickerHeaderDate(prev => dateFns.subMonths(prev, 1));
        break;
      case 'month':
        setDatepickerHeaderDate(prev => dateFns.subYears(prev, 1));
        break;
      case 'year':
        setDatepickerHeaderDate(prev => dateFns.subMonths(prev, 1));
        break;
    }
  };
  const increment = () => {
    switch (type) {
      case 'date':
        setDatepickerHeaderDate(prev => dateFns.addMonths(prev, 1));
        break;
      case 'month':
        setDatepickerHeaderDate(prev => dateFns.addYears(prev, 1));
        break;
      case 'year':
        setDatepickerHeaderDate(prev => dateFns.subMonths(prev, 1));
        break;
    }
  };
  const isToday = date => selectedDate && dateFns.isEqual(new Date(selectedDate.getFullYear(), selectedDate.getMonth(), date), selectedDate);
  const setDateValue = date => () => {
    dateChanged(new Date(datepickerHeaderDate.getFullYear(), datepickerHeaderDate.getMonth(), date));
    setShowDatepicker(false);
  };
  const getDayCount = date => {
    let daysInMonth = dateFns.getDaysInMonth(date);
    // find where to start calendar day of week
    let dayOfWeek = dateFns.getDay(new Date(date.getFullYear(), date.getMonth(), 1));
    let blankDaysArray = [];
    for (let i = 1; i <= dayOfWeek; i++) {
      blankDaysArray.push(i);
    }
    let daysArray = [];
    for (let i = 1; i <= daysInMonth; i++) {
      daysArray.push(i);
    }
    setBlankDays(blankDaysArray);
    setDayCount(daysArray);
  };
  const isSelectedMonth = month => selectedDate && dateFns.isEqual(new Date(selectedDate.getFullYear(), month, selectedDate.getDate()), selectedDate);
  const setMonthValue = month => () => {
    setDatepickerHeaderDate(new Date(datepickerHeaderDate.getFullYear(), month, datepickerHeaderDate.getDate()));
    setType('date');
  };
  const toggleDatepicker = () => setShowDatepicker(prev => !prev);
  const showMonthPicker = () => setType('month');
  const showYearPicker = () => setType('date');
  React.useEffect(() => {
    getDayCount(datepickerHeaderDate);
  }, [datepickerHeaderDate]);
  return /*#__PURE__*/React.createElement("div", {
    className: "mb-5 w-64"
  }, /*#__PURE__*/React.createElement("label", {
    htmlFor: "datepicker",
    className: "font-bold mb-1 text-gray-500 dark:text-white block"
  }, placeholder), /*#__PURE__*/React.createElement("div", {
    className: "relative"
  }, /*#__PURE__*/React.createElement("input", {
    type: "hidden",
    name: "date"
  }), /*#__PURE__*/React.createElement("input", {
    type: "text",
    readOnly: true,
    className: "cursor-pointer w-full pl-4 pr-10 py-3 leading-none rounded-lg shadow-sm focus:outline-none focus:shadow-outline bg-gray-200 dark:bg-[#2C293D] text-gray-700 dark:text-white font-medium",
    placeholder: "Select date",
    value: selectedDate ? dateFns.format(selectedDate, 'dd/MM/yyyy') : '',
    onClick: toggleDatepicker
  }), /*#__PURE__*/React.createElement("div", {
    className: "cursor-pointer absolute top-0 right-0 px-3 py-2",
    onClick: toggleDatepicker
  }, /*#__PURE__*/React.createElement("svg", {
    className: "h-6 w-6 text-gray-600 dark:text-gray-300",
    fill: "none",
    viewBox: "0 0 24 24",
    stroke: "currentColor"
  }, /*#__PURE__*/React.createElement("path", {
    strokeLinecap: "round",
    strokeLinejoin: "round",
    strokeWidth: 2,
    d: "M8 7V3m8 4V3m-9 8h10M5 21h14a2 2 0 002-2V7a2 2 0 00-2-2H5a2 2 0 00-2 2v12a2 2 0 002 2z"
  }))), showDatepicker && /*#__PURE__*/React.createElement("div", {
    className: "bg-gray-200 dark:bg-[#272535] mt-1 rounded-lg shadow p-4 fixed z-10 overflow-auto",
    style: {
      width: '16rem'
    }
  }, /*#__PURE__*/React.createElement("div", {
    className: "flex justify-between items-center mb-2"
  }, /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("button", {
    type: "button",
    className: "transition ease-in-out duration-100 inline-flex cursor-pointer hover:bg-gray-400 hover:text-gray-900 dark:hover:bg-gray-200 dark:hover:text-gray-700 p-1 rounded-full",
    onClick: decrement
  }, /*#__PURE__*/React.createElement("svg", {
    className: "h-6 w-6 text-gray-500 dark:text-gray-300 hover:text-gray-700 inline-flex",
    fill: "none",
    viewBox: "0 0 24 24",
    stroke: "currentColor"
  }, /*#__PURE__*/React.createElement("path", {
    strokeLinecap: "round",
    strokeLinejoin: "round",
    strokeWidth: 2,
    d: "M15 19l-7-7 7-7"
  })))), type === 'date' && /*#__PURE__*/React.createElement("div", {
    onClick: showMonthPicker,
    className: "flex-grow p-1 text-lg font-bold text-gray-400 dark:text-gray-200 cursor-pointer hover:bg-gray-400 dark:hover:bg-gray-200 hover:text-gray-700 rounded-lg"
  }, /*#__PURE__*/React.createElement("p", {
    className: "text-center"
  }, dateFns.format(datepickerHeaderDate, 'MMMM'))), /*#__PURE__*/React.createElement("div", {
    onClick: showYearPicker,
    className: "flex-grow p-1 text-lg font-bold text-gray-400 dark:text-gray-200 cursor-pointer hover:bg-gray-400 dark:hover:bg-gray-200 hover:text-gray-700 rounded-lg"
  }, /*#__PURE__*/React.createElement("p", {
    className: "text-center"
  }, dateFns.format(datepickerHeaderDate, 'yyyy'))), /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("button", {
    type: "button",
    className: "transition ease-in-out duration-100 inline-flex cursor-pointer hover:bg-gray-400 dark:hover:bg-gray-200 hover:text-gray-700 p-1 rounded-full",
    onClick: increment
  }, /*#__PURE__*/React.createElement("svg", {
    className: "h-6 w-6 text-gray-500 dark:text-gray-300 hover:text-gray-700 inline-flex",
    fill: "none",
    viewBox: "0 0 24 24",
    stroke: "currentColor"
  }, /*#__PURE__*/React.createElement("path", {
    strokeLinecap: "round",
    strokeLinejoin: "round",
    strokeWidth: 2,
    d: "M9 5l7 7-7 7"
  }))))), type === 'date' && /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
    className: "flex flex-wrap mb-3 -mx-1"
  }, DAYS.map((day, i) => /*#__PURE__*/React.createElement("div", {
    key: i,
    style: {
      width: '14.26%'
    },
    className: "px-1"
  }, /*#__PURE__*/React.createElement("div", {
    className: "text-gray-400 font-medium text-center text-xs"
  }, day)))), /*#__PURE__*/React.createElement("div", {
    className: "flex flex-wrap -mx-1"
  }, blankDays.map((_, i) => /*#__PURE__*/React.createElement("div", {
    key: i,
    style: {
      width: '14.26%'
    },
    className: "text-center border p-1 border-transparent text-sm"
  })), dayCount.map((d, i) => /*#__PURE__*/React.createElement("div", {
    key: i,
    style: {
      width: '14.26%'
    },
    className: "px-1 mb-1"
  }, /*#__PURE__*/React.createElement("div", {
    onClick: setDateValue(d),
    className: `cursor-pointer text-center text-sm rounded-full leading-loose transition ease-in-out duration-100 ${isToday(d) ? 'bg-blue-500 text-gray-300 dark:text-white' : 'text-gray-400 dark:text-gray-200 hover:bg-blue-200'}`
  }, d))))), type === 'month' && /*#__PURE__*/React.createElement("div", {
    className: "flex flex-wrap -mx-1"
  }, Array(12).fill(null).map((_, i) => /*#__PURE__*/React.createElement("div", {
    key: i,
    onClick: setMonthValue(i),
    style: {
      width: '25%'
    }
  }, /*#__PURE__*/React.createElement("div", {
    className: `cursor-pointer p-5 font-semibold text-center text-sm rounded-lg hover:bg-gray-200 hover:text-gray-700 ${isSelectedMonth(i) ? 'bg-blue-500 text-gray-500 dark:text-white' : 'text-gray-600 dark:text-gray-200 hover:bg-blue-200'}`
  }, dateFns.format(new Date(datepickerHeaderDate.getFullYear(), i, datepickerHeaderDate.getDate()), 'MMM'))))), ' ')));
}

function DatesFilters({
  fromDate,
  toDate,
  fromDateChanged,
  toDateChanged,
  trendPeriods,
  trendPeriodsChanged
}) {
  const [position, setPosition] = React.useState(false);
  const ref = React.useRef(null);
  const onClick = () => {
    if (typeof window !== 'undefined') {
      setTimeout(() => {
        if (ref.current) {
          //@ts-ignore
          const popover = ref.current.getBoundingClientRect();
          if (popover.bottom >= window.innerHeight) {
            setPosition(true);
          } else {
            setPosition(false);
          }
        }
      }, 100);
    }
  };
  return /*#__PURE__*/React.createElement(react.Popover, {
    className: "relative"
  }, ({
    open
  }) => /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement(react.Popover.Button, {
    onClick: onClick,
    className: `
                ${open ? '' : 'text-opacity-90'}
                group inline-flex items-center rounded-md px-3 py-2 text-base font-medium text-gray-500 dark:text-white hover:text-opacity-100 focus:outline-none `
  }, /*#__PURE__*/React.createElement(solid.CalendarDaysIcon, {
    className: "h-7 w-7"
  }), /*#__PURE__*/React.createElement(solid.ChevronDownIcon, {
    className: `${open ? '' : 'text-opacity-70'}
                  ml-2 h-5 w-5 text-[#747189] transition duration-150 ease-in-out group-hover:text-opacity-80`,
    "aria-hidden": "true"
  })), /*#__PURE__*/React.createElement(react.Transition, {
    as: React.Fragment,
    enter: "transition ease-out duration-200",
    enterFrom: "opacity-0 translate-y-1",
    enterTo: "opacity-100 translate-y-0",
    leave: "transition ease-in duration-150",
    leaveFrom: "opacity-100 translate-y-0",
    leaveTo: "opacity-0 translate-y-1"
  }, /*#__PURE__*/React.createElement(react.Popover.Panel, {
    ref: ref,
    className: `fixed z-10 ${position ? '-mt-[300px]' : 'mt-3'} transform max-w-md`
  }, /*#__PURE__*/React.createElement("div", {
    className: "overflow-hidden rounded-lg shadow-lg ring-1 ring-gray-500 dark:ring-black ring-opacity-5"
  }, /*#__PURE__*/React.createElement("div", {
    className: "relative bg-white dark:bg-[#272535] p-5"
  }, /*#__PURE__*/React.createElement("div", {
    className: "w-full"
  }, /*#__PURE__*/React.createElement("div", {
    className: "flex flex-wrap w-full mt-3 justify-between"
  }, /*#__PURE__*/React.createElement("div", {
    className: "flex items-center"
  }, /*#__PURE__*/React.createElement(Datepicker, {
    selectedDate: fromDate,
    dateChanged: fromDateChanged,
    placeholder: "From"
  }), /*#__PURE__*/React.createElement("div", {
    className: "ml-1"
  }, /*#__PURE__*/React.createElement("button", {
    type: "button",
    onClick: () => fromDateChanged(null)
  }, /*#__PURE__*/React.createElement(solid.XMarkIcon, {
    className: "h-4 w-4 text-gray-500 dark:text-gray-200 hover:scale-125 transition duration-100 ease-linear"
  }), /*#__PURE__*/React.createElement("span", {
    className: "sr-only"
  }, "Clear dates filters")))), /*#__PURE__*/React.createElement("div", {
    className: "flex items-center"
  }, /*#__PURE__*/React.createElement(Datepicker, {
    selectedDate: toDate,
    dateChanged: toDateChanged,
    placeholder: "To"
  }), /*#__PURE__*/React.createElement("div", {
    className: "ml-1"
  }, /*#__PURE__*/React.createElement("button", {
    type: "button",
    onClick: () => toDateChanged(null)
  }, /*#__PURE__*/React.createElement(solid.XMarkIcon, {
    className: "h-4 w-4 text-gray-200 hover:scale-125 transition duration-100 ease-linear"
  }), /*#__PURE__*/React.createElement("span", {
    className: "sr-only"
  }, "Clear dates filters")))), trendPeriods !== undefined && trendPeriodsChanged && /*#__PURE__*/React.createElement("div", {
    className: "flex items-center"
  }, /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("label", {
    htmlFor: "small-input",
    className: "block mb-2 text-sm font-medium text-gray-500 dark:text-gray-300"
  }, "Trend periods amount"), /*#__PURE__*/React.createElement("input", {
    type: "number",
    id: "small-input",
    onChange: e => trendPeriodsChanged(Number(e.target.value)),
    value: trendPeriods,
    className: "block p-2 w-1/2 rounded-lg border sm:text-xs bg-gray-300 dark:bg-gray-700 border-gray-300 dark:border-gray-600 placeholder-gray-700 dark:placeholder-gray-400 text-gray-500 dark:text-white focus:ring-blue-500 focus:border-blue-500"
  })))))))))));
}

function ItemActions({
  node,
  searchSimilarClusters,
  splitPhrase,
  mergeClusters,
  translationEnabled
}) {
  const controller = React.useRef(null);
  const [isOpen, setIsOpen] = React.useState(false);
  const [loading, setLoading] = React.useState(false);
  const [action, setAction] = React.useState(null);
  const [similarClusters, setSimilarClusters] = React.useState(null);
  const [selectedClusters, setSelectedClusters] = React.useState(null);
  const [searchText, setSearchText] = React.useState(null);
  const [error, setError] = React.useState(null);
  const [mergeTo, setMergeTo] = React.useState(false);
  React.useEffect(() => {
    if (!node || !searchSimilarClusters) return;
    if (node.type === 'Cluster') {
      setAction('Merge');
      setSearchText('');
    } else if (node.type === 'Phrase') {
      setAction('Split');
    }
    setIsOpen(true);
  }, [node]);
  React.useEffect(() => {
    if (controller.current && !isOpen) controller.current.abort();
    if (!isOpen) {
      setError(null);
      setLoading(false);
      setAction(null);
      setSimilarClusters(null);
      setSelectedClusters(null);
      setSearchText(null);
      controller.current = new AbortController();
    }
  }, [isOpen]);
  React.useEffect(() => {
    if (searchText === null || !node || !searchSimilarClusters) return;
    const fetchData = async controller => {
      try {
        setLoading(true);
        const res = await searchSimilarClusters(searchText === '' ? node.text : searchText, controller);
        setSimilarClusters(res);
      } catch (e) {
        console.error(e);
      } finally {
        setLoading(false);
      }
    };
    const currentController = new AbortController();
    fetchData(currentController).catch(e => {
      console.error(e);
    });
    return () => {
      currentController.abort();
    };
  }, [searchText, node, searchSimilarClusters]);
  function closeModal() {
    setIsOpen(false);
    setSelectedClusters(null);
    setMergeTo(false);
    setSearchText(null);
    setError(null);
    setLoading(false);
    setSimilarClusters(null);
    setAction(null);
  }
  async function invokeAction() {
    if (!splitPhrase || !mergeClusters || !controller.current || !node) return closeModal();
    setError(null);
    setLoading(true);
    if (action === 'Split') {
      const res = await splitPhrase(node.id, controller.current);
      if (res.message) {
        setError(res.message);
      } else {
        closeModal();
      }
    } else {
      const firstSelectedCluster = selectedClusters?.at(0);
      if (selectedClusters && selectedClusters.length > 0 && firstSelectedCluster) {
        let res;
        if (mergeTo) {
          res = await mergeClusters([node.id], firstSelectedCluster, controller.current);
        } else {
          res = await mergeClusters(selectedClusters, node.id, controller.current);
        }
        if (res && res.message) {
          setError(res.message);
        } else {
          closeModal();
        }
      }
    }
    setLoading(false);
  }
  function clusterSelected(clusterId, selected) {
    if (selected) return setSelectedClusters(current => current?.filter(c => c !== clusterId) ?? []);
    if (mergeTo) {
      return setSelectedClusters([clusterId]);
    }
    return setSelectedClusters(selected => {
      return [...(selected ?? []), clusterId];
    });
  }
  return !isOpen ? null : /*#__PURE__*/React.createElement(reactFocusOn.FocusOn, {
    as: "div",
    className: "relative z-10",
    onClickOutside: closeModal
  }, /*#__PURE__*/React.createElement("div", {
    className: "fixed inset-0 overflow-y-auto"
  }, /*#__PURE__*/React.createElement("div", {
    className: "flex min-h-full items-center justify-center p-4 text-center"
  }, /*#__PURE__*/React.createElement("div", {
    className: "w-full max-w-md transform overflow-hidden rounded-2xl bg-white dark:bg-[#272535] p-6 text-left align-middle shadow-xl transition-all"
  }, /*#__PURE__*/React.createElement("h3", {
    className: "text-center text-lg font-medium leading-6 text-gray-900 dark:text-white"
  }, action === 'Merge' ? /*#__PURE__*/React.createElement("div", null, "Merging:") : /*#__PURE__*/React.createElement("span", null, "Split to a new cluster:"), /*#__PURE__*/React.createElement("span", {
    className: "group relative flex w-full"
  }, "\"", /*#__PURE__*/React.createElement("span", {
    className: "truncate"
  }, node?.text), "\"", /*#__PURE__*/React.createElement("span", {
    className: "absolute hidden group-hover:flex -left-5 -translate-y-full px-2 py-1 bg-gray-700 rounded-lg text-center text-white text-sm after:content-[''] after:absolute after:left-1/2 after:top-[100%] after:-translate-x-1/2 after:border-8 after:border-x-transparent after:border-b-transparent after:border-t-gray-700"
  }, node?.text))), error && /*#__PURE__*/React.createElement("p", {
    id: "filled_error_help",
    className: "mt-2 text-xs text-red-600 dark:text-red-400"
  }, /*#__PURE__*/React.createElement("span", {
    className: "font-medium"
  }, "Oh, snapp!"), " ", error), searchText !== null ? /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement("input", {
    type: "text",
    value: searchText,
    onChange: e => setSearchText(e.target.value),
    id: "search",
    className: "bg-gray-50 border border-gray-300 text-gray-900 text-sm rounded-lg focus:ring-blue-500 focus:border-blue-500 block w-full p-2.5 dark:bg-gray-700 dark:border-gray-600 dark:placeholder-gray-400 dark:text-white dark:focus:ring-blue-500 dark:focus:border-blue-500",
    placeholder: "Search for clusters",
    required: true
  })) : null, action === 'Merge' ? /*#__PURE__*/React.createElement("div", {
    className: "flex w-full mt-1 items-center align-middle justify-center"
  }, /*#__PURE__*/React.createElement("span", {
    className: `mr-1 ${mergeTo ? 'text-gray-400 ' : 'text-white'}`
  }, "Merge From"), /*#__PURE__*/React.createElement(react.Switch, {
    checked: mergeTo,
    onChange: () => setMergeTo(state => {
      setSelectedClusters([]);
      return !state;
    }),
    className: `${mergeTo ? 'bg-teal-900' : 'bg-teal-700'}
          relative inline-flex h-[38px] w-[74px] shrink-0 cursor-pointer rounded-full border-2 border-transparent transition-colors duration-200 ease-in-out focus:outline-none focus-visible:ring-2  focus-visible:ring-white focus-visible:ring-opacity-75`
  }, /*#__PURE__*/React.createElement("span", {
    className: "sr-only"
  }, "Use setting"), /*#__PURE__*/React.createElement("span", {
    "aria-hidden": "true",
    className: `${mergeTo ? 'translate-x-9' : 'translate-x-0'}
            pointer-events-none inline-block h-[34px] w-[34px] transform rounded-full bg-white shadow-lg ring-0 transition duration-200 ease-in-out`
  })), /*#__PURE__*/React.createElement("span", {
    className: `ml-1 ${mergeTo ? 'text-white ' : 'text-gray-400'}`
  }, "Merge To")) : null, loading ? /*#__PURE__*/React.createElement("div", {
    className: "grow w-full justify-center items-center flex mt-2"
  }, /*#__PURE__*/React.createElement("div", {
    className: "text-center"
  }, /*#__PURE__*/React.createElement("div", {
    role: "status"
  }, /*#__PURE__*/React.createElement("svg", {
    className: "inline mr-2 w-8 h-8 text-gray-200 animate-spin fill-blue-600",
    viewBox: "0 0 100 101",
    fill: "none",
    xmlns: "http://www.w3.org/2000/svg"
  }, /*#__PURE__*/React.createElement("path", {
    d: "M100 50.5908C100 78.2051 77.6142 100.591 50 100.591C22.3858 100.591 0 78.2051 0 50.5908C0 22.9766 22.3858 0.59082 50 0.59082C77.6142 0.59082 100 22.9766 100 50.5908ZM9.08144 50.5908C9.08144 73.1895 27.4013 91.5094 50 91.5094C72.5987 91.5094 90.9186 73.1895 90.9186 50.5908C90.9186 27.9921 72.5987 9.67226 50 9.67226C27.4013 9.67226 9.08144 27.9921 9.08144 50.5908Z",
    fill: "currentColor"
  }), /*#__PURE__*/React.createElement("path", {
    d: "M93.9676 39.0409C96.393 38.4038 97.8624 35.9116 97.0079 33.5539C95.2932 28.8227 92.871 24.3692 89.8167 20.348C85.8452 15.1192 80.8826 10.7238 75.2124 7.41289C69.5422 4.10194 63.2754 1.94025 56.7698 1.05124C51.7666 0.367541 46.6976 0.446843 41.7345 1.27873C39.2613 1.69328 37.813 4.19778 38.4501 6.62326C39.0873 9.04874 41.5694 10.4717 44.0505 10.1071C47.8511 9.54855 51.7191 9.52689 55.5402 10.0491C60.8642 10.7766 65.9928 12.5457 70.6331 15.2552C75.2735 17.9648 79.3347 21.5619 82.5849 25.841C84.9175 28.9121 86.7997 32.2913 88.1811 35.8758C89.083 38.2158 91.5421 39.6781 93.9676 39.0409Z",
    fill: "currentFill"
  })), /*#__PURE__*/React.createElement("span", {
    className: "sr-only"
  }, "Loading...")))) : similarClusters && /*#__PURE__*/React.createElement("div", {
    className: "overflow-y-auto max-h-64 mt-2"
  }, /*#__PURE__*/React.createElement(ClustersList, {
    clusters: similarClusters.filter(c => c.id.toString() !== node?.id),
    selected: selectedClusters,
    clusterSelected: clusterSelected,
    translationEnabled: translationEnabled
  })), /*#__PURE__*/React.createElement("div", {
    className: "mt-4 flex justify-between"
  }, /*#__PURE__*/React.createElement("button", {
    type: "button",
    className: "text-center w-1/3 rounded-md border border-transparent bg-green-100 px-4 py-2 text-sm font-medium text-green-900 hover:bg-green-200 focus:outline-none focus-visible:ring-2 focus-visible:ring-green-500 focus-visible:ring-offset-2",
    onClick: invokeAction,
    disabled: loading || action === 'Merge' && !selectedClusters
  }, action), /*#__PURE__*/React.createElement("button", {
    type: "button",
    className: "text-center w-1/3 rounded-md border border-transparent bg-blue-100 px-4 py-2 text-sm font-medium text-blue-900 hover:bg-blue-200 focus:outline-none focus-visible:ring-2 focus-visible:ring-blue-500 focus-visible:ring-offset-2",
    onClick: closeModal
  }, "Cancel"))))));
}
function ClustersList({
  clusters,
  selected,
  clusterSelected,
  translationEnabled
}) {
  return /*#__PURE__*/React.createElement("div", {
    className: "w-full pr-1 py-2"
  }, /*#__PURE__*/React.createElement("div", {
    className: "mx-auto w-full max-w-md"
  }, /*#__PURE__*/React.createElement(react.RadioGroup, null, /*#__PURE__*/React.createElement(react.RadioGroup.Label, {
    className: "sr-only"
  }, "Select Destination Cluster"), /*#__PURE__*/React.createElement("div", {
    className: "space-y-2"
  }, clusters.map(cluster => {
    const checked = selected?.some(c => c === cluster.id) ?? false;
    return /*#__PURE__*/React.createElement(react.RadioGroup.Option, {
      key: cluster.id,
      value: cluster,
      onClick: () => clusterSelected(cluster.id, checked),
      className: `
                  ${checked ? 'bg-sky-900 bg-opacity-75 text-white' : 'bg-white'}
                    relative flex cursor-pointer rounded-lg px-5 py-4 shadow-md focus:outline-none`
    }, /*#__PURE__*/React.createElement("div", {
      className: "flex w-full items-center justify-between"
    }, /*#__PURE__*/React.createElement("div", {
      className: "flex items-center"
    }, /*#__PURE__*/React.createElement("div", {
      className: "text-sm"
    }, /*#__PURE__*/React.createElement(react.RadioGroup.Label, {
      as: "p",
      className: `font-medium  ${checked ? 'text-white' : 'text-gray-900'}`
    }, translationEnabled && cluster.translation ? cluster.translation : cluster.text))), checked && /*#__PURE__*/React.createElement("div", {
      className: "shrink-0 text-white"
    }, /*#__PURE__*/React.createElement(CheckIcon, {
      className: "h-6 w-6"
    }))));
  })))));
}
function CheckIcon(props) {
  return /*#__PURE__*/React.createElement("svg", {
    viewBox: "0 0 24 24",
    fill: "none",
    ...props
  }, /*#__PURE__*/React.createElement("circle", {
    cx: 12,
    cy: 12,
    r: 12,
    fill: "#fff",
    opacity: "0.2"
  }), /*#__PURE__*/React.createElement("path", {
    d: "M7 13l3 3 7-7",
    stroke: "#fff",
    strokeWidth: 1.5,
    strokeLinecap: "round",
    strokeLinejoin: "round"
  }));
}

const defaultCountersConfigurations = {
  signals: {
    display: {
      color: 'white',
      icon: /*#__PURE__*/React.createElement("span", {
        className: "flex flex-col",
        style: {
          height: '1em'
        }
      }, /*#__PURE__*/React.createElement(outline.HandThumbUpIcon, null), /*#__PURE__*/React.createElement(outline.HandThumbDownIcon, null))
    },
    items: [{
      label: 'positive',
      display: {
        color: 'green',
        icon: /*#__PURE__*/React.createElement(outline.FaceSmileIcon, null)
      },
      members: [{
        metadataName: 'emotion',
        values: ['happiness']
      }, {
        metadataName: 'sentiment',
        values: ['POS']
      }],
      isGroup: true
    }, {
      label: 'negative',
      display: {
        color: 'red',
        icon: /*#__PURE__*/React.createElement(outline.FaceFrownIcon, null)
      },
      members: [{
        metadataName: 'emotion',
        values: ['anger', 'sadness']
      }, {
        metadataName: 'sentiment',
        values: ['NEG']
      }],
      isGroup: true
    }]
  },
  person: {
    display: {
      color: 'white',
      icon: /*#__PURE__*/React.createElement(outline.UserIcon, null)
    }
  },
  topic: {
    display: {
      color: 'white',
      icon: /*#__PURE__*/React.createElement(solid.HashtagIcon, null)
    }
  },
  keyword: {
    display: {
      color: 'white',
      icon: /*#__PURE__*/React.createElement(outline.KeyIcon, null)
    }
  },
  geo: {
    display: {
      color: 'white',
      icon: /*#__PURE__*/React.createElement(outline.GlobeAltIcon, null)
    }
  },
  org: {
    display: {
      color: 'white',
      icon: /*#__PURE__*/React.createElement(outline.BuildingOffice2Icon, null)
    }
  },
  location: {
    display: {
      color: 'white',
      icon: /*#__PURE__*/React.createElement(outline.MapPinIcon, null)
    }
  },
  groups: {
    display: {
      color: 'white',
      icon: /*#__PURE__*/React.createElement(outline.UsersIcon, null)
    }
  },
  art: {
    display: {
      color: 'white',
      icon: /*#__PURE__*/React.createElement(solid.FilmIcon, null)
    }
  },
  law: {
    display: {
      color: 'white',
      icon: /*#__PURE__*/React.createElement(outline.ScaleIcon, null)
    }
  },
  event: {
    display: {
      color: 'white',
      icon: /*#__PURE__*/React.createElement(outline.CalendarIcon, null)
    }
  },
  sentiment: {
    display: {
      color: 'white',
      icon: /*#__PURE__*/React.createElement("span", {
        className: "flex flex-col",
        style: {
          height: '1em'
        }
      }, /*#__PURE__*/React.createElement(outline.HandThumbUpIcon, null), /*#__PURE__*/React.createElement(outline.HandThumbDownIcon, null))
    },
    items: [{
      label: 'POS',
      display: {
        color: 'green',
        icon: /*#__PURE__*/React.createElement(outline.HandThumbUpIcon, null)
      }
    }, {
      label: 'NEG',
      display: {
        color: 'red',
        icon: /*#__PURE__*/React.createElement(outline.HandThumbDownIcon, null)
      }
    }]
  },
  emotion: {
    display: {
      color: 'white',
      icon: /*#__PURE__*/React.createElement(outline.HeartIcon, null)
    },
    items: [{
      label: 'positive',
      display: {
        color: 'green',
        icon: /*#__PURE__*/React.createElement(outline.FaceSmileIcon, null)
      },
      members: [{
        values: ['happiness']
      }],
      isGroup: true
    }, {
      label: 'negative',
      display: {
        color: 'red',
        icon: /*#__PURE__*/React.createElement(outline.FaceFrownIcon, null)
      },
      members: [{
        values: ['anger', 'sadness']
      }],
      isGroup: true
    }, {
      label: 'happiness',
      display: {
        color: 'green',
        icon: /*#__PURE__*/React.createElement(outline.FaceSmileIcon, null)
      }
    }, {
      label: 'surprise',
      display: {
        color: 'green',
        icon: /*#__PURE__*/React.createElement(outline.FaceSmileIcon, null)
      }
    }, {
      label: 'anger',
      display: {
        color: 'red',
        icon: /*#__PURE__*/React.createElement(outline.FaceFrownIcon, null)
      }
    }, {
      label: 'sadness',
      display: {
        color: 'red',
        icon: /*#__PURE__*/React.createElement(outline.FaceFrownIcon, null)
      }
    }]
  },
  score: {
    display: {
      color: 'white',
      icon: /*#__PURE__*/React.createElement("span", {
        className: "flex flex-col",
        style: {
          height: '1em'
        }
      }, /*#__PURE__*/React.createElement(outline.HandThumbUpIcon, null), /*#__PURE__*/React.createElement(outline.HandThumbDownIcon, null))
    },
    items: [{
      label: '2',
      display: {
        color: 'green',
        icon: /*#__PURE__*/React.createElement(outline.HandThumbUpIcon, null)
      }
    }, {
      label: '1',
      display: {
        color: 'green',
        icon: /*#__PURE__*/React.createElement(outline.HandThumbUpIcon, null)
      }
    }, {
      label: '-1',
      display: {
        color: 'red',
        icon: /*#__PURE__*/React.createElement(outline.HandThumbDownIcon, null)
      }
    }, {
      label: '-2',
      display: {
        color: 'red',
        icon: /*#__PURE__*/React.createElement(outline.HandThumbDownIcon, null)
      }
    }, {
      label: 'positive',
      display: {
        color: 'green',
        icon: /*#__PURE__*/React.createElement(outline.HandThumbUpIcon, null)
      },
      members: [{
        values: ['1', '2']
      }],
      isGroup: true
    }, {
      label: 'positive',
      display: {
        color: 'red',
        icon: /*#__PURE__*/React.createElement(outline.HandThumbDownIcon, null)
      },
      members: [{
        values: ['-1', '-2']
      }],
      isGroup: true
    }]
  },
  city: {
    display: {
      color: 'white',
      icon: /*#__PURE__*/React.createElement(solid.GlobeAmericasIcon, null)
    }
  },
  [CUSTOM_METADATA_KEY]: {
    display: {
      color: 'white',
      icon: /*#__PURE__*/React.createElement(outline.ChatBubbleOvalLeftEllipsisIcon, null)
    }
  }
};

const defaultCalculations = [totalSumCalculationConfiguration, topValueCalculationConfiguration, topValuePercentCalculationConfiguration, topGroupCalculationConfiguration, topGroupPercentCalculationConfiguration, percentOfItemsCalculationConfiguration, percentOfAllItemsCalculationConfiguration, trendCalculationConfiguration, percentOfTotalUniqueItemsCalculationConfiguration, totalUniqueItemsCalculationConfiguration];

function ColorsAxis({
  width,
  colorsConfig
}) {
  return /*#__PURE__*/React.createElement("div", {
    style: {
      width: width,
      opacity: 0.9
    },
    className: "h-full fixed flex flex-col"
  }, colorsConfig.map((colorConfig, i) => /*#__PURE__*/React.createElement("div", {
    key: i,
    className: "w-full",
    style: {
      background: colorConfig,
      height: '4px',
      marginTop: i === 0 ? 'auto' : '1px'
    }
  })));
}

const BAR_PADDING = 0.1;
const BarChart = ({
  dataNodes,
  height,
  width,
  nodeClicked,
  barColor,
  fontFamily,
  textColor,
  counters,
  countersConfiguration,
  labels,
  labelClicked,
  sizeAxis,
  colorAxis,
  translate,
  totalItems,
  nodeActionsClicked,
  totalUniqueItemsStats,
  uniquePropertyName
}) => {
  const [actionsVisible, setActionsVisible] = React.useState(null);
  const barsHeight = dataNodes.length * 40;
  const [groups, setGroups] = React.useState([]);
  React.useEffect(() => {
    // Y axis is for groups since the barplot is horizontal
    setGroups(dataNodes.sort((a, b) => sizeAxis?.key === CUSTOM_METADATA_KEY ? b.amount - a.amount : totalSumCalculation(sizeAxis, b.metadata, b.trends, countersConfiguration).result - totalSumCalculation(sizeAxis, a.metadata, b.trends, countersConfiguration).result).map(d => d.text ?? ''));
  }, [dataNodes, sizeAxis]);
  const yScale = React.useMemo(() => {
    return d3.scaleBand().domain(groups).range([0, barsHeight]).padding(BAR_PADDING);
  }, [height, groups]);
  const max = React.useMemo(() => d3.extent(dataNodes.map(d => sizeAxis?.key === CUSTOM_METADATA_KEY ? d.amount : totalSumCalculation(sizeAxis, d.metadata, d.trends, countersConfiguration).result))[1] ?? 20, [dataNodes, sizeAxis]);
  // X axis
  const xScale = React.useMemo(() => {
    return d3.scaleLinear().domain([0, max || 10]).range([0, width]);
  }, [width, max]);
  // Build the shapes
  const allShapes = dataNodes.map((d, i) => {
    const x = 24;
    const y = yScale(d.text ?? '') ?? 0;
    const result = sizeAxis?.key === CUSTOM_METADATA_KEY ? d.amount : totalSumCalculation(sizeAxis, d.metadata, d.trends, countersConfiguration).result;
    const xWidth = xScale(result === max ? result : Math.min(max - 1, result + max * 0.1)) - 24;
    const barWidth = xWidth > 0 ? xWidth : 0;
    const barHeight = 36;
    const opacity = 1;
    const fill = barColor;
    const fillOpacity = 1;
    const rx = 1;
    const colorsConfig = getBackgroundColorLayers(colorAxis, d.metadata, d.trends, countersConfiguration, totalItems, totalUniqueItemsStats, d.metadata_stats, uniquePropertyName);
    return /*#__PURE__*/React.createElement("g", {
      key: i
    }, /*#__PURE__*/React.createElement("rect", {
      x: x,
      y: y,
      width: barWidth,
      height: barHeight,
      opacity: opacity,
      fill: fill,
      fillOpacity: fillOpacity,
      rx: rx
    }), /*#__PURE__*/React.createElement("foreignObject", {
      x: x,
      y: y,
      width: width,
      height: barHeight,
      opacity: opacity,
      fill: fill,
      fillOpacity: fillOpacity,
      rx: rx
    }, /*#__PURE__*/React.createElement("div", {
      className: "h-full w-full",
      onMouseEnter: () => {
        nodeActionsClicked(d);
        setActionsVisible(i);
      },
      onMouseLeave: () => {
        setActionsVisible(null);
      }
    }, /*#__PURE__*/React.createElement(ColorsAxis, {
      width: barWidth,
      colorsConfig: colorsConfig
    }), /*#__PURE__*/React.createElement("div", {
      className: "flex h-full w-full items-center ml-1 relative text-gray-500",
      style: {
        fontFamily: fontFamily,
        fontWeight: 300,
        fontStyle: 'normal',
        color: textColor,
        fontSize: '14px'
      }
    }, /*#__PURE__*/React.createElement("span", {
      className: "flex items-center max-w-[30%] truncate mr-2"
    }, counters.filter(counter => counter.metadataKeyValue !== null).map((counter, i) => /*#__PURE__*/React.createElement("div", {
      key: i,
      className: "ml-1"
    }, /*#__PURE__*/React.createElement(CounterDisplay, {
      counter: counter,
      countersConfiguration: countersConfiguration,
      metadata: d.metadata,
      trends: d.trends,
      width: "6ch",
      totalItems: totalItems,
      totalUniqueItemsStats: totalUniqueItemsStats,
      uniquePropertyName: uniquePropertyName,
      uniqueItemsStats: d.metadata_stats
    })))), /*#__PURE__*/React.createElement("span", {
      className: "truncate w-3/6 hover:text-gray-400 dark:hover:text-gray-300 hover:cursor-pointer",
      onClick: () => nodeClicked(d),
      dir: "auto"
    }, translate && d.item_translated_text !== undefined && d.item_translated_text !== null && d.item_translated_text !== '' ? d.item_translated_text : d.item_original_text), /*#__PURE__*/React.createElement("span", {
      className: "truncate flex items-center w-2/6 ml-auto"
    }, labels.map((label, i) => /*#__PURE__*/React.createElement("div", {
      key: i
    }, /*#__PURE__*/React.createElement(MaxLabelDisplay, {
      countersConfiguration: countersConfiguration,
      metadataKey: label,
      labelClicked: labelClicked,
      metadata: d.metadata,
      width: "15ch"
    })))), /*#__PURE__*/React.createElement("div", {
      "data-for": "global-actions",
      "data-tip": true,
      "data-event": "click",
      className: `self-end mr-1 hover:cursor-pointer hover:text-white ${actionsVisible === i ? 'visible' : 'invisible'}`
    }, /*#__PURE__*/React.createElement(outline.EllipsisHorizontalIcon, {
      className: "h-4 w-4"
    }))))));
  });
  return /*#__PURE__*/React.createElement("svg", {
    width: width,
    height: barsHeight
  }, allShapes);
};

const ItemsListDisplay = ({
  items,
  bgColor,
  textColor,
  counters,
  labels,
  countersConfiguration,
  labelClicked,
  translate,
  chatLogEnabled,
  totalItems
}) => {
  return /*#__PURE__*/React.createElement("div", {
    className: "overflow-x-auto text-black bg-[#F7F7F7] dark:bg-[#272535] dark:text-white",
    style: {
      backgroundColor: bgColor,
      color: textColor
    }
  }, /*#__PURE__*/React.createElement("table", {
    className: "table-auto w-full h-full text-left"
  }, /*#__PURE__*/React.createElement("thead", {
    className: "border-b-2 border-b-slate-500"
  }, /*#__PURE__*/React.createElement("tr", null, /*#__PURE__*/React.createElement("th", {
    className: "p-1"
  }, "Items"), /*#__PURE__*/React.createElement("th", {
    className: "p-1"
  }, "Time"), counters.filter(counter => counter.metadataKeyValue?.key !== CUSTOM_METADATA_KEY).map((counter, i) => /*#__PURE__*/React.createElement("th", {
    className: "p-1 lowercase first-letter:uppercase",
    key: i
  }, counter.metadataKeyValue ? getMetadataKeyValueDisplay(counter.metadataKeyValue) : '')), labels.map((label, i) => /*#__PURE__*/React.createElement("th", {
    className: "p-1 lowercase first-letter:uppercase",
    key: i
  }, label)))), /*#__PURE__*/React.createElement("tbody", null, items.map((item, i) => {
    const item_date = parseDate(item.item_timestamp);
    const onClickRow = () => {
      if (chatLogEnabled) {
        try {
          window.postMessage({
            event: 'OPEN_CHAT_LOG',
            payload: {
              item
            }
          }, '*');
        } catch (e) {}
      }
    };
    return /*#__PURE__*/React.createElement("tr", {
      key: i,
      role: chatLogEnabled ? 'button' : undefined,
      onClick: chatLogEnabled ? onClickRow : undefined,
      className: chatLogEnabled ? 'hover:bg-slate-500 dark:hover:bg-slate-800 cursor-pointer' : ''
    }, /*#__PURE__*/React.createElement("td", {
      className: "max-w-[60ch] truncate p-1"
    }, translate && item.item_translated_text !== undefined && item.item_translated_text !== null && item.item_translated_text !== '' ? item.item_translated_text : item.original_text), /*#__PURE__*/React.createElement("td", {
      className: "p-1",
      "data-for": "global",
      "data-tip": item_date,
      style: {
        width: '1em',
        height: '1em'
      }
    }, item_date.split(' ').at(0)), counters.filter(counter => counter.metadataKeyValue?.key !== CUSTOM_METADATA_KEY).map((counter, i) => /*#__PURE__*/React.createElement("td", {
      key: i,
      className: "p-1"
    }, /*#__PURE__*/React.createElement(CounterDisplay, {
      counter: counter,
      countersConfiguration: countersConfiguration,
      metadata: item.metadata,
      trends: [],
      totalItems: totalItems
    }))), labels.map((label, i) => /*#__PURE__*/React.createElement("td", {
      key: i
    }, /*#__PURE__*/React.createElement(MaxLabelDisplay, {
      metadataKey: label,
      labelClicked: labelClicked,
      countersConfiguration: countersConfiguration,
      metadata: item.metadata
    }))));
  }))));
};
function parseDate(date) {
  if (!date) return '';
  return dateFns.format(dateFns.parse(date, 'yyyy-MM-dd HH:mm:ss.SSSSSS', new Date()), 'dd/MM/yyyy HH:mm');
}

const BIG_COLOR_DEFAULT = '#322F46';
const SMALL_COLOR_DEFAULT = '#2C293D';
const Treemap = ({
  dataNodes,
  nodeClicked,
  height,
  width,
  bigColor,
  smallColor,
  countFontSize = 14,
  fontFamily,
  textColor,
  borderWidth = 0,
  borderColor,
  labels,
  counters,
  countersConfiguration,
  labelClicked,
  sizeAxis,
  colorAxis,
  nodeActionsClicked,
  translate,
  totalItems,
  totalUniqueItemsStats,
  uniquePropertyName,
  hideLeafHeader,
  hideActionsMenu,
  hideSignals
}) => {
  const mainNode = React.useMemo(() => {
    return {
      id: '',
      children: dataNodes.map(c => {
        return {
          ...c,
          amount: sizeAxis?.key === CUSTOM_METADATA_KEY ? c.amount : totalSumCalculation(sizeAxis, c.metadata, c.trends, countersConfiguration).result,
          children: []
        };
      }),
      amount: 0,
      metadata: {},
      trends: [],
      type: '',
      properties: {}
    };
  }, [dataNodes, sizeAxis]);
  const treeHierarchy = React.useMemo(() => {
    const elementsValues = mainNode.children.map(item => item.amount);
    const maxElementValue = Math.max(...elementsValues);
    const elementsSum = elementsValues.reduce((a, b) => a + b, 0);
    const growFactor = maxElementValue > elementsSum * 0.9 ? 0.1 / (1 - maxElementValue / elementsSum) : 1;
    return d3.hierarchy(mainNode).sum(d => {
      return maxElementValue === d.amount ? d.amount : d.amount * growFactor;
    }).sort((n1, n2) => n2.data.amount - n1.data.amount);
  }, [mainNode]);
  const root = React.useMemo(() => {
    const treeGenerator = d3.treemap().size([width, height]).padding(0);
    return treeGenerator(treeHierarchy);
  }, [treeHierarchy, width, height]);
  const colors = React.useMemo(() => {
    const len = root.leaves().length;
    if (!bigColor || !smallColor || !(chromaJs.valid(bigColor) && chromaJs.valid(smallColor))) return chromaJs.scale([BIG_COLOR_DEFAULT, SMALL_COLOR_DEFAULT]).domain([0, len]);
    return chromaJs.scale([bigColor, smallColor]).domain([0, len]);
  }, [root, bigColor, smallColor]);
  const [actionsVisible, setActionsVisible] = React.useState(null);
  const allShapes = root.leaves().map((leaf, index) => {
    const height = leaf.y1 - leaf.y0;
    const width = leaf.x1 - leaf.x0;
    const fontSize = calculateFontSize(height, width);
    const lineHeight = fontSize * 1.1 + 5;
    const paddingY = 10;
    const lines = Math.max(1, Math.floor((height - paddingY + 20) / lineHeight) - 3); // -2 is for the d.data.value and <br />, 20 is for the value absolute top padding
    const colorsConfig = getBackgroundColorLayers(colorAxis, leaf.data.metadata, leaf.data.trends, countersConfiguration, totalItems, totalUniqueItemsStats, leaf.data.metadata_stats, uniquePropertyName);
    return /*#__PURE__*/React.createElement("g", {
      key: index
    }, /*#__PURE__*/React.createElement("rect", {
      x: leaf.x0,
      y: leaf.y0,
      width: width,
      height: height,
      stroke: "transparent",
      fill: colors(index).hex()
    }), /*#__PURE__*/React.createElement("foreignObject", {
      x: leaf.x0,
      y: leaf.y0,
      width: width,
      height: height
    }, /*#__PURE__*/React.createElement("div", {
      className: "h-full w-full",
      onMouseEnter: () => {
        nodeActionsClicked(leaf.data);
        setActionsVisible(index);
      },
      onMouseLeave: () => {
        setActionsVisible(null);
      }
    }, /*#__PURE__*/React.createElement(ColorsAxis, {
      width: width,
      colorsConfig: colorsConfig
    }), /*#__PURE__*/React.createElement("div", {
      className: "flex flex-col h-full w-full p-1 relative border-slate-500 dark:border-[#272535] text-black dark:text-white",
      style: {
        fontFamily: fontFamily,
        fontWeight: 300,
        fontStyle: 'normal',
        color: textColor,
        borderRightWidth: `${borderWidth}px`,
        borderLeftWidth: `${borderWidth}px`,
        borderBottomWidth: `${borderWidth}px`,
        borderColor: borderColor
      }
    }, hideLeafHeader ? null : /*#__PURE__*/React.createElement("div", {
      className: "flex",
      style: {
        fontSize: `${countFontSize}px`
      }
    }, /*#__PURE__*/React.createElement(CountersLabelsDisplay, {
      counters: counters,
      labels: labels,
      metadata: leaf.data.metadata,
      trends: leaf.data.trends,
      countersConfiguration: countersConfiguration,
      labelClicked: labelClicked,
      totalItems: totalItems,
      totalUniqueItemsStats: totalUniqueItemsStats,
      uniquePropertyName: uniquePropertyName,
      uniqueItemsStats: leaf.data.metadata_stats,
      hideSignals: hideSignals
    })), /*#__PURE__*/React.createElement("span", {
      className: "items-center flex justify-center h-full hover:cursor-pointer",
      onClick: () => nodeClicked(leaf.data),
      style: {
        fontSize: `${fontSize}px`
      }
    }, /*#__PURE__*/React.createElement("span", {
      "data-element": "rect-text",
      className: "overflow-hidden text-center",
      dir: "auto",
      style: {
        lineHeight: `${lineHeight}px`,
        wordBreak: 'break-word',
        WebkitLineClamp: lines,
        WebkitTouchCallout: 'none',
        display: '-webkit-box',
        WebkitBoxOrient: 'vertical'
      }
    }, translate && leaf.data.item_translated_text !== undefined && leaf.data.item_translated_text !== null && leaf.data.item_translated_text !== '' ? leaf.data.item_translated_text : leaf.data.item_original_text)), hideActionsMenu ? null : /*#__PURE__*/React.createElement("div", {
      "data-for": "global-actions",
      "data-tip": true,
      "data-event": "click",
      className: `self-end hover:cursor-pointer hover:text-white ${actionsVisible === index ? 'visible' : 'invisible'}`
    }, /*#__PURE__*/React.createElement(outline.EllipsisHorizontalIcon, {
      className: "h-4 w-4"
    }))))));
  });
  return /*#__PURE__*/React.createElement("svg", {
    width: width,
    height: height
  }, allShapes);
};

// Please do not use types off of a default export module or else Storybook Docs will suffer.
// see: https://github.com/storybookjs/storybook/issues/9556
/**
 * One Ai Analytics Component
 */
const OneAiAnalytics = ({
  dataNodes = {
    totalItems: 0,
    nodes: []
  },
  currentNode,
  currentPage = 0,
  totalPagesAmount = 0,
  goBackClicked = () => {},
  nodeClicked = () => {},
  nextPageClicked = () => {},
  prevPageClicked = () => {},
  itemsDisplay = ItemsListDisplay,
  chatLogEnabled = false,
  darkMode = true,
  background,
  treemapBigColor = darkMode ? '#322F46' : '#F7F7F7',
  treemapSmallColor = darkMode ? '#2C293D' : '#F7F7F7',
  treemapCountFontSize = 14,
  fontFamily = "'Poppins', sans-serif",
  textColor = darkMode ? 'white' : '#111111',
  treemapBorderWidth = 1,
  navbarColor,
  treemapBorderColor = darkMode ? '#272535' : 'white',
  barColor = darkMode ? '#322F46' : '#F7F7F7',
  loading,
  error = null,
  nodesPath = [],
  dateRangeChanged = () => {},
  labelsFilters,
  labelClicked = () => {},
  labelFilterDeleted = () => {},
  trendPeriods,
  trendPeriodsChanged,
  splitPhrase,
  mergeClusters,
  searchSimilarClusters,
  translationEnabled = true,
  customizeEnabled = true,
  filterOnlySkills = false,
  datePickerEnabled = true,
  startDate = null,
  endDate = null,
  toggleHide = () => {},
  propertiesFilters = {},
  setPropertiesFilters = () => {},
  metaOptions,
  currentMetaOption,
  metaOptionsChanged = () => {},
  refresh = () => {},
  uniqueMetaKey: uniquePropertyName,
  hideNavBar = false,
  hideToolBar = false,
  hideLeafHeader = false,
  hideActionsMenu = false,
  hideNavFilters = false,
  hideSignals = false,
  hideNavigationMenu = false,
  tooltipOffsetLeft = 0,
  tooltipOffsetTop = 0
}) => {
  const [display, setDisplay] = React.useState('Treemap');
  const {
    width,
    height,
    ref
  } = reactResizeDetector.useResizeDetector();
  const [metaData, setMetaData] = React.useState({});
  const [nodes, setNodes] = React.useState([]);
  const [labels, setLabels] = React.useState([]);
  const [counters, setCounters] = React.useState([]);
  const [sizeAxis, setSizeAxis] = React.useState(null);
  const [colorAxis, setColorAxis] = React.useState([]);
  const [countersConfigurations, setCountersConfigurations] = React.useState({});
  const [currentNodeActions, setCurrentNodeActions] = React.useState(null);
  const [translate, setTranslate] = React.useState(false);
  const [fromDate, setFromDate] = React.useState(startDate ? new Date(startDate) : null);
  const [toDate, setToDate] = React.useState(endDate ? new Date(endDate) : null);
  const loadedNodes = React.useRef([]);
  const currentCollection = React.useRef(null);
  const [currentHoveredNode, setCurrentHoveredNode] = React.useState(null);
  React.useEffect(() => {
    if (!nodesPath || nodesPath.length === 0 || currentCollection.current === nodesPath[0].text) return;
    currentCollection.current = nodesPath[0].text;
    setLabels(JSON.parse(localStorage.getItem(getCurrentStorageKey(labelsStorageKey, currentCollection.current)) ?? '[]'));
    setCounters(getInitialCounterTypes(defaultCalculations, [{
      metadataKeyValue: {
        key: CUSTOM_METADATA_KEY
      },
      calculationName: percentOfTotalUniqueItemsCalculationName
    }, {
      metadataKeyValue: {
        key: CUSTOM_METADATA_KEY
      },
      calculationName: totalUniqueItemsCalculationName
    }, {
      metadataKeyValue: {
        key: 'signals'
      },
      calculationName: topGroupPercentCalculationName
    }], currentCollection.current, countersStorageKey));
    const storedSizeAxis = localStorage.getItem(getCurrentStorageKey(sizeAxisStorageKey, currentCollection.current));
    setSizeAxis(storedSizeAxis ? JSON.parse(storedSizeAxis) : {
      key: CUSTOM_METADATA_KEY
    });
    setColorAxis(getInitialCounterTypes(defaultCalculations, [{
      metadataKeyValue: {
        key: 'signals',
        value: 'positive'
      },
      calculationName: percentOfItemsCalculationName
    }, {
      metadataKeyValue: {
        key: 'signals',
        value: 'negative'
      },
      calculationName: percentOfItemsCalculationName
    }], currentCollection.current, colorAxisStorageKey));
  }, [nodesPath]);
  React.useEffect(() => {
    setMetaData(currentMetadata => {
      const newMetadata = currentNode ? loadedNodes.current.some(loadedNode => loadedNode.type === currentNode.type && loadedNode.id === getNodeId(currentNode)) ? currentMetadata : mergeMetadata(currentMetadata, currentNode?.data.metadata ?? {}) : nodes.filter(node => !loadedNodes.current.some(loadedNode => loadedNode.id === node.id && loadedNode.type === node.type)).reduce((finalMetadata, currentNode) => mergeMetadata(finalMetadata, currentNode.metadata), currentMetadata);
      loadedNodes.current.push({
        type: currentNode?.type ?? COLLECTION_TYPE,
        id: currentNode ? getNodeId(currentNode) : COLLECTION_TYPE
      });
      if (!currentNode) loadedNodes.current.push(...nodes.map(node => {
        return {
          type: node.type,
          id: node.id
        };
      }));
      return newMetadata;
    });
  }, [currentNode, nodes]);
  React.useEffect(() => {
    setNodes(dataNodes.nodes.map(d => {
      const itemsCount = getNodeItemsCount(d);
      const nodeText = getNodeText(d);
      const {
        originalText,
        translatedText
      } = getNodeOriginalAndTranslatedText(d);
      return {
        id: getNodeId(d),
        amount: itemsCount,
        text: nodeText,
        item_original_text: originalText ?? nodeText,
        item_translated_text: translatedText,
        metadata: {
          [CUSTOM_METADATA_KEY]: [{
            value: CUSTOM_METADATA_KEY,
            count: itemsCount
          }],
          ...d.data.metadata
        },
        trends: getNodeTrends(d),
        type: d.type,
        properties: d.data.properties,
        metadata_stats: d.data.metadata_stats
      };
    }));
  }, [dataNodes]);
  React.useEffect(() => {
    const newCountersConfigurations = {};
    Object.keys(metaData).concat(Object.keys(defaultCountersConfigurations)).forEach(key => {
      const defaultConfig = defaultCountersConfigurations[key];
      const valuesConfigured = defaultConfig?.items?.map(group => group.label) ?? [];
      const counterConfiguration = {
        label: (defaultConfig?.label ?? key).toLowerCase(),
        display: defaultConfig?.display,
        members: defaultConfig?.members ?? [{
          metadataName: key
        }],
        isGroup: defaultConfig?.isGroup ?? false,
        items: key === CUSTOM_METADATA_KEY ? undefined : (defaultConfig?.items?.map(group => {
          return {
            label: group.label,
            display: group.display,
            isGroup: group.isGroup,
            members: group.members?.map(member => {
              return {
                metadataName: member.metadataName ?? key,
                values: member.values
              };
            }) ?? [{
              metadataName: key,
              values: [group.label ?? '']
            }]
          };
        }) ?? []).concat(metaData[key]?.filter(meta => !valuesConfigured.includes(meta.value)).map(meta => {
          return {
            label: meta.value,
            members: [{
              metadataName: key,
              values: [meta.value]
            }],
            display: undefined,
            isGroup: false
          };
        }) ?? [])
      };
      newCountersConfigurations[key] = counterConfiguration;
    });
    setCountersConfigurations(newCountersConfigurations);
  }, [metaData]);
  React.useEffect(() => {
    const storedCounters = counters.map(counter => {
      return {
        metadataKeyValue: counter.metadataKeyValue,
        calculationName: counter.calculationConfiguration.name
      };
    });
    const storedColorAxis = colorAxis.map(counter => {
      return {
        metadataKeyValue: counter.metadataKeyValue,
        calculationName: counter.calculationConfiguration.name
      };
    });
    if (currentCollection.current) {
      localStorage.setItem(getCurrentStorageKey(countersStorageKey, currentCollection.current), JSON.stringify(storedCounters));
      localStorage.setItem(getCurrentStorageKey(labelsStorageKey, currentCollection.current), JSON.stringify(labels));
      localStorage.setItem(getCurrentStorageKey(colorAxisStorageKey, currentCollection.current), JSON.stringify(storedColorAxis));
      localStorage.setItem(getCurrentStorageKey(sizeAxisStorageKey, currentCollection.current), JSON.stringify(sizeAxis));
    }
  }, [counters, labels, sizeAxis, colorAxis]);
  React.useEffect(() => {
    dateRangeChanged(fromDate, toDate);
  }, [fromDate, toDate]);
  React.useEffect(() => {
    ReactTooltip.hide();
    ReactTooltip.rebuild();
  });
  return /*#__PURE__*/React.createElement("div", {
    className: `oneai-analytics-namespace h-full w-full overflow-hidden ${darkMode ? 'dark' : ''}`
  }, /*#__PURE__*/React.createElement("div", {
    id: "headlessui-portal-root",
    className: `h-full w-full flex flex-col overflow-hidden bg-[#f3e5e5] dark:bg-[#161414] p-1 ${darkMode ? 'dark' : ''}`,
    style: {
      background
    }
  }, /*#__PURE__*/React.createElement(ReactTooltip, {
    id: "global",
    overridePosition: ({
      left,
      top
    }) => {
      return {
        left: left - tooltipOffsetLeft,
        top: top - tooltipOffsetTop
      };
    }
  }), /*#__PURE__*/React.createElement(ReactTooltip, {
    id: "global-actions",
    place: "top",
    effect: "solid",
    overridePosition: ({
      left,
      top
    }) => {
      return {
        left: left - tooltipOffsetLeft,
        top: top - tooltipOffsetTop
      };
    },
    clickable: true,
    className: "!p-1"
  }, /*#__PURE__*/React.createElement("div", {
    className: "flex flex-col w-full h-full items-baseline"
  }, /*#__PURE__*/React.createElement("button", {
    type: "button",
    onClick: () => {
      ReactTooltip.hide();
      setCurrentNodeActions(currentHoveredNode);
    },
    className: "text-gray-900 w-full bg-white border border-gray-300 focus:outline-none hover:bg-gray-100 focus:ring-4 focus:ring-gray-200 font-medium rounded-lg text-sm px-2 py-2 mr-1 mb-1 dark:bg-gray-800 dark:text-white dark:border-gray-600 dark:hover:bg-gray-700 dark:hover:border-gray-600 dark:focus:ring-gray-700"
  }, currentHoveredNode?.type === 'Cluster' ? 'Merge' : 'Split'), /*#__PURE__*/React.createElement("button", {
    type: "button",
    onClick: () => {
      ReactTooltip.hide();
      ReactTooltip.rebuild();
      toggleHide(currentHoveredNode, currentHoveredNode?.properties?.['hide'] === 'true' ? 'false' : 'true');
    },
    className: "text-gray-900 w-full bg-white border border-gray-300 focus:outline-none hover:bg-gray-100 focus:ring-4 focus:ring-gray-200 font-medium rounded-lg text-sm px-2 py-2 mr-1 mb-1 dark:bg-gray-800 dark:text-white dark:border-gray-600 dark:hover:bg-gray-700 dark:hover:border-gray-600 dark:focus:ring-gray-700"
  }, currentHoveredNode?.properties?.['hide'] === 'true' ? 'Show' : 'Hide'))), /*#__PURE__*/React.createElement(ItemActions, {
    node: currentNodeActions,
    splitPhrase: splitPhrase,
    mergeClusters: mergeClusters,
    searchSimilarClusters: searchSimilarClusters,
    translationEnabled: translate
  }), hideToolBar ? null : /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
    className: "w-full rounded-t-md border-b border-[#322F46] bg-white dark:bg-[#272535]",
    style: {
      height: '65px',
      background: navbarColor,
      fontFamily: fontFamily
    }
  }, /*#__PURE__*/React.createElement("div", {
    className: "flex flex-row items-center py-6 ml-[24px] h-full"
  }, /*#__PURE__*/React.createElement("div", {
    className: "flex flex-row w-5/12 justify-start items-center"
  }, /*#__PURE__*/React.createElement("div", {
    className: "h-full flex"
  }, /*#__PURE__*/React.createElement("svg", {
    className: getVisualizationLogoClasses(display === 'Treemap'),
    onClick: () => setDisplay('Treemap'),
    width: "24",
    height: "24",
    viewBox: "0 0 24 24",
    fill: "none",
    stroke: "currentColor",
    xmlns: "http://www.w3.org/2000/svg"
  }, /*#__PURE__*/React.createElement("path", {
    d: "M3.5 5.89477C3.5 5.06635 4.17157 4.39478 5 4.39478H19C19.8284 4.39478 20.5 5.06635 20.5 5.89478V11.9211H3.5V5.89477Z"
  }), /*#__PURE__*/React.createElement("path", {
    d: "M3.5 11.9736H13.8684V19.4999H5C4.17157 19.4999 3.5 18.8284 3.5 17.9999V11.9736Z"
  }), /*#__PURE__*/React.createElement("path", {
    d: "M13.9211 11.9736H20.5001V17.9999C20.5001 18.8284 19.8285 19.4999 19.0001 19.4999H13.9211V11.9736Z"
  }), /*#__PURE__*/React.createElement("path", {
    d: "M8.68433 4.36841V11.4737"
  })), /*#__PURE__*/React.createElement("svg", {
    className: getVisualizationLogoClasses(display === 'BarChart'),
    onClick: () => setDisplay('BarChart'),
    width: "24",
    height: "24",
    viewBox: "0 0 24 24",
    stroke: "currentColor",
    fill: "none",
    xmlns: "http://www.w3.org/2000/svg"
  }, /*#__PURE__*/React.createElement("path", {
    d: "M3.75 20.25V3.75"
  }), /*#__PURE__*/React.createElement("path", {
    d: "M3.75 9.75H15.75C15.75 11.5074 15.75 12.4926 15.75 14.25H3.75"
  }), /*#__PURE__*/React.createElement("path", {
    d: "M20.25 5.25H3.75V9.75H20.25V5.25Z"
  }), /*#__PURE__*/React.createElement("path", {
    d: "M12.75 14.25V18.75H3.75"
  }))), datePickerEnabled ? /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement(DatesFilters, {
    fromDate: fromDate,
    fromDateChanged: setFromDate,
    toDate: toDate,
    toDateChanged: setToDate,
    trendPeriods: trendPeriods,
    trendPeriodsChanged: trendPeriodsChanged
  })) : null, hideNavFilters ? null : /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement(LabelsFiltersSelect, {
    filterOnlySkills: filterOnlySkills,
    selectedLabels: labelsFilters ?? [],
    countersConfigurations: countersConfigurations,
    labelFilterDeleted: labelFilterDeleted,
    selectedMetadataKeyValueChange: metadataKeyValue => labelClicked(metadataKeyValue.key, metadataKeyValue.value)
  })), customizeEnabled ? /*#__PURE__*/React.createElement("div", null, /*#__PURE__*/React.createElement(CustomizeTab, {
    currentCounters: counters,
    selectedLabels: labels,
    countersConfigurations: countersConfigurations,
    labelsOptions: Object.keys(metaData).filter(key => key !== CUSTOM_METADATA_KEY),
    calculationsConfigurations: defaultCalculations,
    countersChanged: setCounters,
    labelsChanged: setLabels,
    selectedSizeAxis: sizeAxis,
    sizeAxisChanged: setSizeAxis,
    currentColorsAxis: colorAxis,
    colorsAxisChanged: setColorAxis
  })) : null), /*#__PURE__*/React.createElement("div", {
    className: "flex flex-row w-full justify-end items-center"
  }, /*#__PURE__*/React.createElement(outline.ArrowPathIcon, {
    onClick: refresh,
    "data-for": "global",
    "data-tip": "Refresh data",
    className: "h-6 w-6 p-1 mr-1 hover:cursor-pointer focus:outline-none text-[#747189] dark:hover:text-white"
  }), propertiesFilters['hide'] === 'true' ? /*#__PURE__*/React.createElement(outline.EyeIcon, {
    onClick: () => setPropertiesFilters({
      hide: 'false'
    }),
    "data-for": "global",
    "data-tip": "Show hidden nodes",
    className: "h-6 w-6 p-1 mr-1 hover:cursor-pointer focus:outline-none text-[#747189] dark:hover:text-white"
  }) : /*#__PURE__*/React.createElement(outline.EyeSlashIcon, {
    onClick: () => setPropertiesFilters({
      hide: 'true'
    }),
    "data-for": "global",
    "data-tip": "Hide hidden nodes",
    className: "h-6 w-6 p-1 mr-1 hover:cursor-pointer focus:outline-none bg-[#EFEFEF] dark:text-white dark:bg-[#322F46]"
  }), translationEnabled ? /*#__PURE__*/React.createElement(outline.LanguageIcon, {
    onClick: () => setTranslate(translate => !translate),
    "data-for": "global",
    "data-tip": translate ? 'Disable translation' : 'Enable translation',
    className: `h-6 w-6 p-1 mr-1 hover:cursor-pointer focus:outline-none ${translate ? 'bg-[#EFEFEF] dark:text-white dark:bg-[#322F46]' : 'text-[#747189] hover:cursor-pointer dark:hover:text-white'}`
  }) : null)))), /*#__PURE__*/React.createElement("div", {
    className: "w-full grow flex flex-col relative overflow-hidden bg-white dark:bg-[#272535]",
    style: {
      background: navbarColor
    }
  }, hideNavBar ? /*#__PURE__*/React.createElement("button", {
    type: "button",
    onClick: () => goBackClicked(1),
    disabled: currentNode === null,
    className: `rounded-lg absolute z-[50000] left-2 top-2 inline-flex ${currentNode ? 'hover:bg-[#EFEFEF] dark:hover:bg-slate-700' : 'hover:cursor-default'}`
  }, currentNode ? /*#__PURE__*/React.createElement("svg", {
    className: "h-[1em] w-[1em] self-center text-[#111111] dark:text-white",
    width: "24",
    height: "24",
    viewBox: "0 0 24 24",
    strokeWidth: "2",
    stroke: "currentColor",
    fill: "none",
    strokeLinecap: "round",
    strokeLinejoin: "round"
  }, ' ', /*#__PURE__*/React.createElement("path", {
    stroke: "none",
    d: "M0 0h24v24H0z"
  }), ' ', /*#__PURE__*/React.createElement("line", {
    x1: "5",
    y1: "12",
    x2: "19",
    y2: "12"
  }), ' ', /*#__PURE__*/React.createElement("line", {
    x1: "5",
    y1: "12",
    x2: "9",
    y2: "16"
  }), ' ', /*#__PURE__*/React.createElement("line", {
    x1: "5",
    y1: "12",
    x2: "9",
    y2: "8"
  })) : null, /*#__PURE__*/React.createElement("span", {
    className: "sr-only"
  }, "Go back")) : /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
    className: "w-full",
    style: {
      height: '65px',
      fontFamily: fontFamily
    }
  }, /*#__PURE__*/React.createElement("div", {
    className: "flex flex-row items-center ml-[24px] h-full"
  }, /*#__PURE__*/React.createElement("div", {
    className: "flex flex-row justify-start mr-4 items-center w-full",
    style: {
      fontFamily: fontFamily,
      fontWeight: 300,
      fontStyle: 'normal',
      fontSize: '14px'
    }
  }, /*#__PURE__*/React.createElement("button", {
    type: "button",
    onClick: () => goBackClicked(1),
    disabled: currentNode === null,
    className: `rounded-lg inline-flex ${currentNode ? 'hover:bg-[#EFEFEF] dark:hover:bg-slate-700' : 'hover:cursor-default'}`
  }, currentNode ? /*#__PURE__*/React.createElement("svg", {
    className: "h-[1em] w-[1em] self-center text-[#111111] dark:text-white",
    width: "24",
    height: "24",
    viewBox: "0 0 24 24",
    strokeWidth: "2",
    stroke: "currentColor",
    fill: "none",
    strokeLinecap: "round",
    strokeLinejoin: "round"
  }, ' ', /*#__PURE__*/React.createElement("path", {
    stroke: "none",
    d: "M0 0h24v24H0z"
  }), ' ', /*#__PURE__*/React.createElement("line", {
    x1: "5",
    y1: "12",
    x2: "19",
    y2: "12"
  }), ' ', /*#__PURE__*/React.createElement("line", {
    x1: "5",
    y1: "12",
    x2: "9",
    y2: "16"
  }), ' ', /*#__PURE__*/React.createElement("line", {
    x1: "5",
    y1: "12",
    x2: "9",
    y2: "8"
  })) : /*#__PURE__*/React.createElement(solid.HomeIcon, {
    className: "h-[0.9em] w-[1em] text-[#111111] dark:text-white"
  }), /*#__PURE__*/React.createElement("span", {
    className: "sr-only"
  }, "Go back")), /*#__PURE__*/React.createElement("div", {
    className: "ml-1 text-[#111111] dark:text-gray-300 truncate flex items-center"
  }, nodesPath.map((node, i) => /*#__PURE__*/React.createElement("div", {
    key: i,
    className: "flex"
  }, /*#__PURE__*/React.createElement("div", {
    className: "max-w-[50ch] truncate"
  }, /*#__PURE__*/React.createElement("span", {
    className: "cursor-pointer hover:text-gray-600 dark:hover:text-gray-50",
    onClick: () => goBackClicked(i === 0 ? nodesPath.length : nodesPath.length - 1 - i),
    dir: "auto"
  }, translate && node.translated ? node.translated : node.text)), nodesPath.length - 1 !== i && /*#__PURE__*/React.createElement("span", {
    className: "ml-1 mr-1"
  }, "/"))), metaOptions && nodesPath.length === 1 ? /*#__PURE__*/React.createElement("span", {
    className: "ml-1 flex text-[#111111] dark:text-gray-300"
  }, /*#__PURE__*/React.createElement("span", {
    className: "mr-1"
  }, "/ "), /*#__PURE__*/React.createElement(SingleSelect$1, {
    hideNavigationMenu: hideNavigationMenu,
    options: metaOptions,
    selectedLabel: currentMetaOption ?? 'text',
    onSelect: metaOptionsChanged
  })) : null, (labelsFilters?.length ?? 0) > 0 && labelsFilters?.filter(label => label.value).map((keyValue, i) => /*#__PURE__*/React.createElement("span", {
    key: i,
    className: "flex items-center"
  }, /*#__PURE__*/React.createElement("span", {
    className: "text-gray-500 ml-1"
  }, "/ "), /*#__PURE__*/React.createElement(LabelDisplay, {
    metadataKey: keyValue.key,
    value: keyValue.value ?? '',
    countersConfiguration: countersConfigurations,
    labelClicked: () => {},
    maxWidth: "20ch",
    color: "#747189"
  }), /*#__PURE__*/React.createElement("button", {
    type: "button",
    onClick: () => labelFilterDeleted(i)
  }, /*#__PURE__*/React.createElement(solid.XMarkIcon, {
    className: "h-4 w-4 text-gray-400 hover:scale-125 transition duration-100 ease-linear"
  }), /*#__PURE__*/React.createElement("span", {
    className: "sr-only"
  }, "Delete label filter")))), totalPagesAmount > 1 && currentPage > 0 && /*#__PURE__*/React.createElement("span", {
    className: "ml-1 text-gray-500"
  }, "/ ", currentPage + 1))), /*#__PURE__*/React.createElement("div", null, !loading && /*#__PURE__*/React.createElement(CountersLabelsDisplay, {
    counters: counters,
    labels: labels,
    metadata: nodes.reduce((finalMetadata, currentNode) => mergeMetadata(finalMetadata, currentNode.metadata, dataNodes.totalItems), {}),
    trends: nodes.reduce((finalMetadata, currentNode) => mergeTrends(finalMetadata, currentNode.trends), []),
    countersConfiguration: countersConfigurations,
    labelClicked: labelClicked,
    totalItems: dataNodes.totalItems,
    totalUniqueItemsStats: dataNodes.uniqueItemsStats,
    uniqueItemsStats: dataNodes.uniqueItemsStats,
    uniquePropertyName: uniquePropertyName,
    hideSignals: hideSignals
  }))))), /*#__PURE__*/React.createElement("div", {
    className: "w-full h-full flex flex-col overflow-x-hidden"
  }, loading && /*#__PURE__*/React.createElement("div", {
    className: "grow w-full justify-center items-center flex"
  }, /*#__PURE__*/React.createElement("div", {
    className: "text-center"
  }, /*#__PURE__*/React.createElement("div", {
    role: "status"
  }, /*#__PURE__*/React.createElement("svg", {
    className: "inline mr-2 w-8 h-8 text-gray-200 animate-spin fill-blue-600",
    viewBox: "0 0 100 101",
    fill: "none",
    xmlns: "http://www.w3.org/2000/svg"
  }, /*#__PURE__*/React.createElement("path", {
    d: "M100 50.5908C100 78.2051 77.6142 100.591 50 100.591C22.3858 100.591 0 78.2051 0 50.5908C0 22.9766 22.3858 0.59082 50 0.59082C77.6142 0.59082 100 22.9766 100 50.5908ZM9.08144 50.5908C9.08144 73.1895 27.4013 91.5094 50 91.5094C72.5987 91.5094 90.9186 73.1895 90.9186 50.5908C90.9186 27.9921 72.5987 9.67226 50 9.67226C27.4013 9.67226 9.08144 27.9921 9.08144 50.5908Z",
    fill: "currentColor"
  }), /*#__PURE__*/React.createElement("path", {
    d: "M93.9676 39.0409C96.393 38.4038 97.8624 35.9116 97.0079 33.5539C95.2932 28.8227 92.871 24.3692 89.8167 20.348C85.8452 15.1192 80.8826 10.7238 75.2124 7.41289C69.5422 4.10194 63.2754 1.94025 56.7698 1.05124C51.7666 0.367541 46.6976 0.446843 41.7345 1.27873C39.2613 1.69328 37.813 4.19778 38.4501 6.62326C39.0873 9.04874 41.5694 10.4717 44.0505 10.1071C47.8511 9.54855 51.7191 9.52689 55.5402 10.0491C60.8642 10.7766 65.9928 12.5457 70.6331 15.2552C75.2735 17.9648 79.3347 21.5619 82.5849 25.841C84.9175 28.9121 86.7997 32.2913 88.1811 35.8758C89.083 38.2158 91.5421 39.6781 93.9676 39.0409Z",
    fill: "currentFill"
  })), /*#__PURE__*/React.createElement("span", {
    className: "sr-only"
  }, "Loading...")))), /*#__PURE__*/React.createElement("div", {
    className: `flex flex-row flex-grow ${loading ? 'invisible pointer-events-none w-0 h-0' : 'w-full h-full'}`
  }, currentPage > 0 && /*#__PURE__*/React.createElement("div", {
    className: "h-full flex items-center justify-center hover:cursor-pointer dark:bg-[#322F46]",
    onClick: prevPageClicked,
    style: {
      width: '3%',
      backgroundColor: treemapBigColor
    }
  }, /*#__PURE__*/React.createElement("button", {
    type: "button",
    className: "text-slate-500 hover:text-slate-700 font-medium rounded-lg text-sm text-center inline-flex items-center"
  }, /*#__PURE__*/React.createElement("svg", {
    "aria-hidden": "true",
    className: "w-5 h-5 rotate-180",
    fill: "currentColor",
    viewBox: "0 0 20 20",
    xmlns: "http://www.w3.org/2000/svg"
  }, /*#__PURE__*/React.createElement("path", {
    fillRule: "evenodd",
    d: "M10.293 3.293a1 1 0 011.414 0l6 6a1 1 0 010 1.414l-6 6a1 1 0 01-1.414-1.414L14.586 11H3a1 1 0 110-2h11.586l-4.293-4.293a1 1 0 010-1.414z",
    clipRule: "evenodd"
  })), /*#__PURE__*/React.createElement("span", {
    className: "sr-only"
  }, "Prev"))), /*#__PURE__*/React.createElement("div", {
    ref: ref,
    className: "h-full w-full overflow-y-auto no-scrollbar overflow-x-hidden"
  }, error !== null || dataNodes.totalItems === 0 ? /*#__PURE__*/React.createElement("p", {
    className: "h-full w-full ml-[24px] dark:text-white"
  }, error, dataNodes.totalItems === 0 ? (labelsFilters?.length ?? 0) > 0 ? /*#__PURE__*/React.createElement("span", {
    className: "flex flex-col"
  }, /*#__PURE__*/React.createElement("span", null, "No items match your filter:"), labelsFilters?.map(labelFilter => /*#__PURE__*/React.createElement("span", null, labelFilter.key, labelFilter.value ? `=${labelFilter.value}` : ''))) : ' Collection is empty' : null) : currentNode && currentNode.type === 'Phrase' ? itemsDisplay({
    items: dataNodes.nodes.map(dataNode => dataNode.data),
    chatLogEnabled: chatLogEnabled,
    bgColor: navbarColor,
    textColor: textColor,
    counters: counters,
    labels: labels,
    labelClicked: labelClicked,
    countersConfiguration: countersConfigurations,
    translate: translate,
    totalItems: dataNodes.totalItems
  }) : display === 'Treemap' ? /*#__PURE__*/React.createElement(Treemap, {
    dataNodes: nodes,
    height: height ?? 0,
    width: width ?? 0,
    nodeClicked: node => {
      ReactTooltip.hide();
      nodeClicked({
        type: !currentNode ? 'Cluster' : currentNode.type === 'Cluster' ? 'Phrase' : 'Item',
        id: node.id
      });
    },
    labels: labels,
    counters: counters,
    bigColor: treemapBigColor,
    smallColor: treemapSmallColor,
    countFontSize: treemapCountFontSize,
    fontFamily: fontFamily,
    textColor: textColor,
    borderWidth: treemapBorderWidth,
    borderColor: treemapBorderColor,
    countersConfiguration: countersConfigurations,
    labelClicked: labelClicked,
    sizeAxis: sizeAxis,
    colorAxis: colorAxis,
    nodeActionsClicked: node => {
      setCurrentHoveredNode({
        type: !currentNode ? 'Cluster' : currentNode.type === 'Cluster' ? 'Phrase' : 'Item',
        id: node.id,
        text: node.text ?? '',
        properties: node.properties
      });
    },
    translate: translate,
    totalItems: dataNodes.totalItems,
    totalUniqueItemsStats: dataNodes.uniqueItemsStats,
    uniquePropertyName: uniquePropertyName,
    hideLeafHeader: hideLeafHeader,
    hideActionsMenu: hideActionsMenu,
    hideSignals: hideSignals
  }) : /*#__PURE__*/React.createElement(BarChart, {
    dataNodes: nodes,
    height: height ?? 0,
    width: width ?? 0,
    nodeClicked: node => {
      ReactTooltip.hide();
      nodeClicked({
        type: !currentNode ? 'Cluster' : currentNode.type === 'Cluster' ? 'Phrase' : 'Item',
        id: node.id
      });
    },
    fontFamily: fontFamily,
    textColor: textColor,
    barColor: barColor,
    labels: labels,
    counters: counters,
    countersConfiguration: countersConfigurations,
    labelClicked: labelClicked,
    sizeAxis: sizeAxis,
    colorAxis: colorAxis,
    nodeActionsClicked: node => {
      setCurrentHoveredNode({
        type: !currentNode ? 'Cluster' : currentNode.type === 'Cluster' ? 'Phrase' : 'Item',
        id: node.id,
        text: node.text ?? '',
        properties: node.properties
      });
    },
    translate: translate,
    totalItems: dataNodes.totalItems,
    totalUniqueItemsStats: dataNodes.uniqueItemsStats,
    uniquePropertyName: uniquePropertyName
  })), currentPage < totalPagesAmount - 1 && /*#__PURE__*/React.createElement("div", {
    className: "h-full flex items-center justify-center hover:cursor-pointer",
    onClick: nextPageClicked,
    style: {
      width: '3%',
      backgroundColor: treemapSmallColor
    }
  }, /*#__PURE__*/React.createElement("button", {
    type: "button",
    className: "text-slate-500 hover:text-slate-700 font-medium rounded-lg text-sm text-center inline-flex items-center"
  }, /*#__PURE__*/React.createElement("svg", {
    "aria-hidden": "true",
    className: "w-5 h-5",
    fill: "currentColor",
    viewBox: "0 0 20 20",
    xmlns: "http://www.w3.org/2000/svg"
  }, /*#__PURE__*/React.createElement("path", {
    fillRule: "evenodd",
    d: "M10.293 3.293a1 1 0 011.414 0l6 6a1 1 0 010 1.414l-6 6a1 1 0 01-1.414-1.414L14.586 11H3a1 1 0 110-2h11.586l-4.293-4.293a1 1 0 010-1.414z",
    clipRule: "evenodd"
  })), /*#__PURE__*/React.createElement("span", {
    className: "sr-only"
  }, "Next"))))))));
};
function getVisualizationLogoClasses(active) {
  return `h-7 w-7 mr-1 p-1 rounded-md ${active ? 'bg-[#EFEFEF] dark:text-white dark:bg-[#322F46]' : 'text-[#747189] hover:cursor-pointer dark:hover:text-white'}`;
}
function mergeMetadata(metadata1, metadata2, totalItems) {
  const newMetadata = {};
  Array.from(new Set([...Object.keys(metadata1), ...Object.keys(metadata2)])).forEach(key => {
    if (totalItems === undefined || key !== CUSTOM_METADATA_KEY) newMetadata[key] = [...(metadata1[key] ?? []), ...(metadata2[key] ?? [])];
  });
  if (totalItems === undefined) return newMetadata;
  newMetadata[CUSTOM_METADATA_KEY] = [{
    value: CUSTOM_METADATA_KEY,
    count: totalItems
  }];
  return newMetadata;
}
function mergeTrends(trends1, trends2) {
  if (!trends1 || !trends2) return [];
  const newTrends = [];
  const sourceTrends = trends1.length >= trends2.length ? trends1 : trends2;
  sourceTrends.forEach((_, i) => {
    newTrends.push({
      days: sourceTrends[i].days,
      period_start_date: sourceTrends[i].period_start_date,
      period_end_date: sourceTrends[i].period_end_date,
      items_count: (trends1.at(i)?.items_count ?? 0) + (trends2.at(i)?.items_count ?? 0),
      phrases_count: (trends1.at(i)?.phrases_count ?? 0) + (trends2.at(i)?.phrases_count ?? 0),
      metadata: mergeMetadata(trends1.at(i)?.metadata ?? {}, trends2.at(i)?.metadata ?? {})
    });
  });
  return newTrends;
}
function getInitialCounterTypes(calculationConfiguration, defaultCounters, collection, storageKey) {
  const storedCounters = JSON.parse(localStorage.getItem(getCurrentStorageKey(storageKey, collection)) ?? '[]');
  const counters = storedCounters.length > 0 ? storedCounters : defaultCounters;
  return counters.map(counter => {
    return {
      calculationConfiguration: calculationConfiguration.find(calc => calc.name === counter.calculationName),
      metadataKeyValue: counter.metadataKeyValue
    };
  }).filter(calc => calc.calculationConfiguration !== undefined);
}
function getCurrentStorageKey(prefix, collection) {
  return `${prefix}-${collection}`;
}

const cache = /*#__PURE__*/new Map();
const nodeToPageCache = /*#__PURE__*/new Map();
// Please do not use types off of a default export module or else Storybook Docs will suffer.
// see: https://github.com/storybookjs/storybook/issues/9556
/**
 * One AI Analytics api wrapper component
 */
const OneAIAnalyticsApiWrapper = ({
  domain = 'prod',
  apiKey = '',
  collection = '',
  collectionDisplayName = collection,
  refreshToken = '',
  uniqueMetaKey: uniquePropertyName,
  datePickerEnabled = true,
  customizeEnabled = true,
  filterOnlySkills = false,
  startDate = undefined,
  endDate = undefined,
  hideNavBar = false,
  hideToolBar = false,
  hideLeafHeader = false,
  hideActionsMenu = false,
  hideNavFilters = false,
  hideSignals = false,
  hideNavigationMenu = false,
  ...rest
}) => {
  domain = resolveDomain(domain);
  const [loading, setLoading] = React.useState(true);
  const [error, setError] = React.useState(null);
  const [currentNodes, setCurrentNodes] = React.useState({
    totalItems: 0,
    nodes: []
  });
  const [clickedNodes, setClickedNodes] = React.useState([]);
  const [currentPage, setCurrentPage] = React.useState(0);
  const [totalPages, setTotalPages] = React.useState(0);
  const [dateRange, setDateRange] = React.useState([startDate ? new Date(startDate) : null, endDate ? new Date(endDate) : null]);
  const [labelsFilters, setLabelsFilters] = React.useState([]);
  const [localRefreshToken, setLocalRefreshToken] = React.useState(refreshToken);
  const [trendPeriods, setTrendPeriods] = React.useState(0);
  const [propertiesFilters, setPropertiesFilters] = React.useState({
    hide: 'true'
  });
  const [metaOptions, setMetaOptions] = React.useState([]);
  const [currentMetaGroup, setCurrentMetaGroup] = React.useState('text');
  const [metaGroupClicked, setMetaGroupClicked] = React.useState(null);
  const previousValues = React.useRef({
    domain: null,
    apiKey: null,
    collection: null,
    refreshToken: null,
    localRefreshToken: null,
    clickedNodes: null,
    currentPage: null,
    lastMetaGroup: 'text',
    startDate: '',
    endDate: ''
  });
  const fetchMetaClusters = async controller => {
    const metaClusters = await fetchMetaClustersApi(controller, domain, collection, apiKey, 0, dateRange[0], dateRange[1], labelsFilters, trendPeriods, propertiesFilters);
    const newNodes = metaClusters?.data?.map(c => {
      return {
        type: 'Meta',
        data: c
      };
    });
    const mappedNodes = newNodes?.map(n => n.data.meta_key) ?? [];
    setMetaOptions(mappedNodes);
  };
  React.useEffect(() => {
    if (previousValues.current.domain !== domain || previousValues.current.apiKey !== apiKey || previousValues.current.collection !== collection || previousValues.current.refreshToken !== refreshToken || previousValues.current.lastMetaGroup !== currentMetaGroup || previousValues.current.startDate !== startDate || previousValues.current.endDate !== endDate) {
      setCurrentNodes({
        totalItems: 0,
        nodes: []
      });
      setDateRange([startDate ? new Date(startDate) : null, endDate ? new Date(endDate) : null]);
      setClickedNodes([]);
      setCurrentPage(0);
      cache.clear();
      previousValues.current = {
        ...previousValues.current,
        domain,
        apiKey,
        collection,
        refreshToken,
        clickedNodes: null,
        currentPage: null,
        lastMetaGroup: currentMetaGroup,
        startDate,
        endDate
      };
    }
  }, [domain, apiKey, collection, refreshToken, currentMetaGroup, startDate, endDate]);
  React.useEffect(() => {
    const fetchData = async controller => {
      setLoading(true);
      const currentClicked = clickedNodes.at(-1);
      if (currentMetaGroup !== 'text' && !currentClicked) {
        const cached = getNodesFromCache(COLLECTION_TYPE, collection + '_meta', currentPage);
        if (cached) {
          setCurrentNodes({
            totalItems: cached.totalItems,
            nodes: cached.nodes
          });
          setTotalPages(cached.totalPages);
        } else {
          const metaClusters = await fetchMetaClustersApi(controller, domain, collection, apiKey, 0, dateRange[0], dateRange[1], labelsFilters, trendPeriods, propertiesFilters, currentMetaGroup);
          if (metaClusters.error) {
            if (metaClusters.error.includes('AbortError')) {
              return;
            }
            setError(metaClusters.error);
            return setLoading(false);
          }
          setError(null);
          const newNodes = metaClusters.data.sort((m1, m2) => m2.items_count - m1.items_count).map(c => {
            return {
              type: 'Meta',
              data: c
            };
          });
          const totalPages = Math.ceil(metaClusters.data.length / PAGE_SIZE);
          const totalItems = metaClusters.data.reduce((prev, current) => prev + current.items_count, 0);
          const slicedNodes = newNodes.slice(currentPage * PAGE_SIZE, (currentPage + 1) * PAGE_SIZE);
          if (clickedNodes.at(-1) == currentClicked) {
            setCurrentNodes({
              totalItems: totalItems,
              nodes: slicedNodes
            });
            setTotalPages(totalPages);
          }
          setNodesDataInCache('Collection', collection + '_meta', currentPage, slicedNodes, totalPages, totalItems);
        }
      } else if (!currentClicked) {
        const cached = getNodesFromCache(COLLECTION_TYPE, collection, currentPage);
        if (cached) {
          setTotalPages(cached.totalPages);
          setCurrentNodes({
            totalItems: cached.totalItems,
            uniqueItemsStats: cached.uniqueItemsStats,
            nodes: cached.nodes
          });
        } else {
          const clusters = await fetchClusters(controller, domain, collection, apiKey, currentPage, dateRange[0], dateRange[1], [...labelsFilters, ...(metaGroupClicked ? [metaGroupClicked] : [])], trendPeriods, propertiesFilters, uniquePropertyName ? [uniquePropertyName] : []);
          if (clusters.error) {
            if (clusters.error.includes('AbortError')) {
              return;
            }
            setError(clusters.error);
            return setLoading(false);
          }
          setError(null);
          const newNodes = clusters.data.map(c => {
            return {
              type: 'Cluster',
              data: c
            };
          });
          if (clickedNodes.at(-1) == currentClicked) {
            setCurrentNodes({
              totalItems: clusters.totalItems,
              uniqueItemsStats: clusters.uniqueItemsStats,
              nodes: newNodes
            });
            setTotalPages(clusters.totalPages);
          }
          setNodesDataInCache('Collection', collection, currentPage, newNodes, clusters.totalPages, clusters.totalItems, clusters.uniqueItemsStats);
        }
      } else if (currentClicked.type === 'Cluster') {
        const clusterId = currentClicked.data.cluster_id.toString();
        const cached = getNodesFromCache(currentClicked.type, clusterId, currentPage);
        if (cached) {
          setTotalPages(cached.totalPages);
          setCurrentNodes({
            totalItems: cached.totalItems,
            uniqueItemsStats: cached.uniqueItemsStats,
            nodes: cached.nodes
          });
        } else {
          const phrases = await fetchPhrases(controller, domain, collection, clusterId, apiKey, currentPage, dateRange[0], dateRange[1], [...labelsFilters, ...(metaGroupClicked ? [metaGroupClicked] : [])], trendPeriods, propertiesFilters, uniquePropertyName ? [uniquePropertyName] : []);
          if (phrases.error) {
            if (phrases.error.includes('AbortError')) {
              return;
            }
            setError(phrases.error);
            return setLoading(false);
          }
          setError(null);
          const newNodes = phrases.data.map(p => {
            return {
              type: 'Phrase',
              data: p
            };
          });
          if (clickedNodes.at(-1) == currentClicked) {
            setCurrentNodes({
              totalItems: phrases.totalItems,
              uniqueItemsStats: phrases.uniqueItemsStats,
              nodes: newNodes
            });
            setTotalPages(phrases.totalPages);
          }
          setNodesDataInCache(currentClicked.type, clusterId, currentPage, newNodes, phrases.totalPages, phrases.totalItems, phrases.uniqueItemsStats);
        }
      } else if (currentClicked.type === 'Phrase') {
        const phraseId = currentClicked.data.phrase_id.toString();
        const cached = getNodesFromCache(currentClicked.type, phraseId, currentPage);
        if (cached) {
          setTotalPages(cached.totalPages);
          setCurrentNodes({
            totalItems: cached.totalItems,
            nodes: cached.nodes
          });
        } else {
          const items = await fetchItems(controller, domain, collection, phraseId, apiKey, currentPage, dateRange[0], dateRange[1], [...labelsFilters, ...(metaGroupClicked ? [metaGroupClicked] : [])], trendPeriods, propertiesFilters);
          if (items.error) {
            if (items.error.includes('AbortError')) {
              return;
            }
            setError(items.error);
            return setLoading(false);
          }
          setError(null);
          const newNodes = items.data.map(i => {
            return {
              type: 'Item',
              data: i
            };
          });
          if (clickedNodes.at(-1) == currentClicked) {
            setCurrentNodes({
              totalItems: items.totalItems,
              nodes: newNodes
            });
            setTotalPages(items.totalPages);
          }
          setNodesDataInCache(currentClicked.type, phraseId, currentPage, newNodes, items.totalPages, items.totalItems);
        }
      }
      setLoading(false);
    };
    const controller = new AbortController();
    if (previousValues.current.domain !== domain || previousValues.current.apiKey !== apiKey || previousValues.current.collection !== collection || previousValues.current.clickedNodes != clickedNodes || previousValues.current.currentPage !== currentPage || previousValues.current.localRefreshToken !== localRefreshToken || previousValues.current.lastMetaGroup !== currentMetaGroup) {
      cache.clear();
      fetchMetaClusters(controller).catch(e => {
        console.error(e);
      });
      fetchData(controller).catch(e => {
        console.error(e);
        setLoading(false);
      });
      previousValues.current = {
        ...previousValues.current,
        domain,
        collection,
        apiKey,
        localRefreshToken,
        clickedNodes,
        currentPage,
        lastMetaGroup: currentMetaGroup
      };
    }
    return () => {
      controller.abort();
    };
  }, [domain, collection, apiKey, localRefreshToken, clickedNodes, currentPage, currentMetaGroup]);
  const nodeClicked = node => {
    const currentNodeDetails = getNodeDetails(clickedNodes.at(-1), collection);
    if (currentMetaGroup !== 'text') {
      return setLocalRefreshToken(current => {
        const [key, value] = node.id.split('$$');
        setMetaGroupClicked({
          key,
          value
        });
        setCurrentMetaGroup('text');
        return current.length > 2 ? '1' : current + '1';
      });
    }
    setNodePageNumberInCache(currentNodeDetails.type, currentNodeDetails.id, currentPage);
    let clickedNode = null;
    if (node.type === 'Cluster') {
      clickedNode = currentNodes.nodes.find(n => n.data.cluster_id.toString() === node.id) ?? null;
    } else if (node.type === 'Phrase') {
      clickedNode = currentNodes.nodes.find(n => n.data.phrase_id.toString() === node.id) ?? null;
    }
    if (clickedNode) {
      const nodeCachedPage = getNodePageNumberFromCache(clickedNode.type, getNodeId(clickedNode));
      setCurrentPage(nodeCachedPage);
      setClickedNodes(currentClickedCluster => [...currentClickedCluster, clickedNode]);
    }
  };
  const goBack = (skip = 1) => {
    if (skip === 0) {
      if (currentMetaGroup !== 'text') {
        setMetaGroupClicked(null);
        setCurrentMetaGroup('text');
      }
      return;
    }
    setClickedNodes(clickedClusters => {
      const originalClickedAmount = clickedClusters.length;
      for (let i = 0; i < skip; i++) {
        clickedClusters.pop();
      }
      const currentNodeDetails = getNodeDetails(clickedClusters.at(-1), collection);
      const nodeCachedPage = getNodePageNumberFromCache(currentNodeDetails.type, currentNodeDetails.id);
      setCurrentPage(nodeCachedPage);
      const extraBackNodes = originalClickedAmount - skip;
      if (extraBackNodes < 0) {
        setLocalRefreshToken(current => {
          if (extraBackNodes === -1) {
            setCurrentMetaGroup(metaGroupClicked?.key ?? 'text');
            setMetaGroupClicked(null);
          } else {
            setMetaGroupClicked(null);
            setCurrentMetaGroup('text');
          }
          return current.length > 2 ? '1' : current + '1';
        });
      }
      return [...clickedClusters];
    });
  };
  return currentNodes ? /*#__PURE__*/React.createElement(OneAiAnalytics, {
    dataNodes: currentNodes ?? {
      totalItems: 0,
      nodes: []
    },
    currentNode: clickedNodes.at(-1),
    nodeClicked: nodeClicked,
    goBackClicked: goBack,
    currentPage: currentPage,
    totalPagesAmount: totalPages,
    nextPageClicked: () => setCurrentPage(page => page + 1),
    prevPageClicked: () => setCurrentPage(page => page - 1),
    loading: loading,
    error: error,
    nodesPath: [{
      text: collectionDisplayName
    }, ...(metaGroupClicked ? [{
      text: metaGroupClicked.key
    }, {
      text: metaGroupClicked.value ?? ''
    }] : []), ...clickedNodes.map(node => {
      const {
        originalText,
        translatedText
      } = getNodeOriginalAndTranslatedText(node);
      return {
        text: originalText ?? getNodeText(node),
        translated: translatedText
      };
    })],
    dateRangeChanged: (from, to) => setLocalRefreshToken(current => {
      setDateRange([from, to]);
      return current + '1';
    }),
    labelsFilters: labelsFilters,
    setLabelsFilters: setLabelsFilters,
    labelClicked: (key, value) => {
      if (!labelsFilters.some(keyValue => keyValue.key.toLowerCase() === key.toLowerCase() && keyValue.value?.toLowerCase() === value.toLowerCase())) setLocalRefreshToken(current => {
        setLabelsFilters(labels => [...labels, {
          key,
          value
        }]);
        return current.length > 2 ? '1' : current + '1';
      });
    },
    labelFilterDeleted: i => setLocalRefreshToken(current => {
      setLabelsFilters(filters => {
        filters.splice(i, 1);
        return [...filters];
      });
      return current.length > 2 ? '1' : current + '1';
    }),
    trendPeriods: trendPeriods,
    trendPeriodsChanged: newTrendPeriod => setLocalRefreshToken(current => {
      setTrendPeriods(newTrendPeriod);
      return current + '1';
    }),
    searchSimilarClusters: (text, controller) => searchSimilarClusters(controller, domain, collection, apiKey, text),
    splitPhrase: (phraseId, controller) => splitPhrase(controller, domain, collection, apiKey, phraseId, setLocalRefreshToken),
    mergeClusters: (source, destination, controller) => mergeClusters(controller, domain, collection, apiKey, source, destination, setLocalRefreshToken),
    toggleHide: (node, hide) => toggleHide(domain, collection, apiKey, node, hide, clickedNodes.at(-1), setLocalRefreshToken),
    propertiesFilters: propertiesFilters,
    setPropertiesFilters: properties => setLocalRefreshToken(current => {
      setPropertiesFilters(properties);
      return current.length > 2 ? '1' : current + '1';
    }),
    metaOptions: ['text', ...uniqBy(metaOptions, m => m.toLowerCase())],
    currentMetaOption: currentMetaGroup,
    metaOptionsChanged: metaOptions => setCurrentMetaGroup(metaOptions),
    refresh: () => setLocalRefreshToken(current => {
      return current.length > 2 ? '1' : current + '1';
    }),
    customizeEnabled: customizeEnabled,
    datePickerEnabled: datePickerEnabled,
    startDate: startDate,
    endDate: endDate,
    uniqueMetaKey: uniquePropertyName,
    filterOnlySkills: filterOnlySkills,
    hideNavBar: hideNavBar,
    hideToolBar: hideToolBar,
    hideLeafHeader: hideLeafHeader,
    hideActionsMenu: hideActionsMenu,
    hideNavFilters: hideNavFilters,
    hideSignals: hideSignals,
    hideNavigationMenu: hideNavigationMenu,
    ...rest
  }) : null;
};
async function fetchClusters(controller, domain, collection, apiKey, page, from, to, labelsFilters, trendPeriods, propertiesFilters, uniqueMetaData) {
  return await fetchApi(controller, `${domain}/clustering/v1/collections/${collection}/clusters`, apiKey, 'clusters', page, from, to, labelsFilters, trendPeriods, propertiesFilters, uniqueMetaData.length > 0 ? '&include-metadata-stats=true&include-metadata-stats-names=' + uniqueMetaData.join(',') : '');
}
async function fetchPhrases(controller, domain, collection, clusterId, apiKey, page, from, to, labelsFilters, trendPeriods, propertiesFilters, uniqueMetaData) {
  return await fetchApi(controller, `${domain}/clustering/v1/collections/${collection}/clusters/${clusterId}/phrases`, apiKey, 'phrases', page, from, to, labelsFilters, trendPeriods, propertiesFilters, uniqueMetaData.length > 0 ? '&include-metadata-stats=true&include-metadata-stats-names=' + uniqueMetaData.join(',') : '');
}
async function fetchItems(controller, domain, collection, phraseId, apiKey, page, from, to, labelsFilters, trendPeriods, propertiesFilters) {
  return await fetchApi(controller, `${domain}/clustering/v1/collections/${collection}/phrases/${phraseId}/items`, apiKey, 'items', page, from, to, labelsFilters, trendPeriods, propertiesFilters);
}
async function fetchMetaClustersApi(controller, domain, collection, apiKey, page, from, to, labelsFilters, trendPeriods, propertiesFilters, metaGroup) {
  return await fetchApi(controller, `${domain}/clustering/v1/collections/${collection}/metadata${metaGroup ? `/${metaGroup}` : ''}`, apiKey, 'content', page, from, to, labelsFilters, trendPeriods, propertiesFilters, `&group-by-meta-value=true${metaGroup ? '&include-metadata=true' : ''}`, 1000);
}
async function fetchApi(controller, url, apiKey, accessor, page, from, to, labelsFilters, trendPeriods, propertiesFilters, extraParams, limit) {
  const labelsFiltersString = labelsFilters.map(metadataKeyValue => metadataKeyValue.key && metadataKeyValue.value ? `'${metadataKeyValue.key}' eq '${metadataKeyValue.value}'` : '').filter(str => str !== '');
  const propertiesFiltersString = Object.keys(propertiesFilters).map(key => {
    const value = propertiesFilters[key];
    if (value) {
      return `'${key}' neq '${value}'`;
    }
    return '';
  });
  try {
    const res = await fetch(encodeURI(`${url}?page=${page}&limit=${limit !== undefined ? limit : PAGE_SIZE}&translate=true` + (from ? `&from-date=${dateFns.format(from, 'yyyy-MM-dd')}` : '') + (to ? `&to-date=${dateFns.format(to, 'yyyy-MM-dd') + 'T23:59:59.999Z'}` : '') + (labelsFiltersString.length > 0 ? `&item-metadata=${labelsFiltersString.join(' and ')}` : '') + (trendPeriods > 1 ? `&include-trends=true&trend-periods-limit=${trendPeriods}` : '') + (propertiesFiltersString.length > 0 ? `&properties-query=${propertiesFiltersString.join(' and ')}` : '') + (extraParams ? extraParams : '')), {
      method: 'GET',
      headers: {
        'Content-Type': 'application/json',
        'api-key': apiKey
      },
      signal: controller.signal
    });
    if (res.status !== 200 || !res.ok) return {
      totalPages: 0,
      totalItems: 0,
      data: [],
      error: 'Error fetching data, status: ' + res.status + ': ' + (await res.text())
    };
    const json = await res.json();
    return {
      totalPages: json['total_pages'],
      totalItems: json['total_items'],
      uniqueItemsStats: json['total_metadata_stats'],
      data: json[accessor],
      error: null
    };
  } catch (e) {
    const error = String(e);
    console.error('error occurred ->', error);
    return {
      totalPages: 0,
      totalItems: 0,
      data: [],
      error: `Error fetching data, ${error}`
    };
  }
}
function getNodesFromCache(parentType, parentId, page) {
  const cached = cache.get(assembleCacheId(parentType, parentId, page));
  if (cached && getSecondsDiff(cached.time, new Date()) < 90) {
    return cached;
  }
  return null;
}
function setNodesDataInCache(parentType, parentId, page, nodes, totalPages, totalItems, uniqueItemsStats) {
  cache.set(assembleCacheId(parentType, parentId, page), {
    nodes: nodes,
    totalPages: totalPages,
    time: new Date(),
    totalItems: totalItems,
    uniqueItemsStats: uniqueItemsStats
  });
}
function getNodePageNumberFromCache(nodeType, nodeId) {
  const cached = nodeToPageCache.get(assembleCacheId(nodeType, nodeId));
  if (cached) {
    return cached;
  }
  return 0;
}
function setNodePageNumberInCache(nodeType, nodeId, page) {
  nodeToPageCache.set(assembleCacheId(nodeType, nodeId), page);
}
function assembleCacheId(type, id, page = 0) {
  return `${type}-${id}-${page}`;
}
async function searchSimilarClusters(controller, domain, collection, apiKey, text) {
  try {
    const res = await fetch(`${domain}/clustering/v1/collections/${collection}/clusters/find?text=${text}&translate=true`, {
      method: 'GET',
      headers: {
        'Content-Type': 'application/json',
        'api-key': apiKey
      },
      signal: controller.signal
    });
    if (res.status !== 200 || !res.ok) return [];
    const clusters = await res.json();
    if (!clusters || clusters.length === 0) return [];
    return clusters.sort((c1, c2) => c2.similarity - c1.similarity).map(c => {
      return {
        id: c.cluster_id,
        text: c.cluster_text,
        translation: c.item_translated_text
      };
    });
  } catch (e) {
    console.error('error occurred ->', e);
    return [];
  }
}
async function splitPhrase(controller, domain, collection, apiKey, phraseId, setLocalRefreshToken) {
  try {
    const res = await fetch(`${domain}/clustering/v1/collections/${collection}/phrases/${phraseId}/split`, {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
        'api-key': apiKey
      },
      signal: controller.signal
    });
    if (res.status !== 200 || !res.ok) return {
      status: 'error',
      message: 'unknown'
    };
    const json = await res.json();
    if (!json.message) {
      setLocalRefreshToken(current => {
        if (current.length > 5) {
          return '';
        }
        return current + '1';
      });
    }
    return json;
  } catch (e) {
    console.error('error occurred ->', e);
    return {
      status: 'error',
      message: String(e)
    };
  }
}
async function mergeClusters(controller, domain, collection, apiKey, source, destination, setLocalRefreshToken) {
  try {
    const res = await fetch(`${domain}/clustering/v1/collections/${collection}/merge`, {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
        Accept: 'application/json',
        'api-key': apiKey
      },
      body: JSON.stringify({
        source_clusters: source,
        destination_cluster: destination
      }),
      signal: controller.signal
    });
    if (res.status !== 200 || !res.ok) return {
      status: 'error',
      message: 'unknown'
    };
    const json = await res.json();
    if (!json.message) {
      setLocalRefreshToken(current => {
        if (current.length > 5) {
          return '';
        }
        return current + '1';
      });
    }
    return json;
  } catch (e) {
    console.error('error occurred ->', e);
    return {
      status: 'error',
      message: String(e)
    };
  }
}
async function toggleHide(domain, collection, apiKey, node, hide, currentClickedNode, setLocalRefreshToken) {
  if (!node) return;
  try {
    const clusterId = node.type === 'Cluster' ? node.id : getNodeId(currentClickedNode);
    const res = await fetch(`${domain}/clustering/v1/collections/${collection}/clusters/${clusterId}${node.type === 'Phrase' ? `/phrases/${node.id}` : ''}/settings`, {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
        Accept: 'application/json',
        'api-key': apiKey
      },
      body: JSON.stringify({
        properties: {
          hide: hide
        }
      })
    });
    if (res.status !== 200 || !res.ok) return;
    setLocalRefreshToken(current => {
      if (current.length > 5) {
        return '';
      }
      return current + '1';
    });
  } catch (e) {
    console.error('error occurred ->', e);
  }
}

// Please do not use types off of a default export module or else Storybook Docs will suffer.
// see: https://github.com/storybookjs/storybook/issues/9556
/**
 * One AI Analytics static data wrapper component
 */
const OneAIAnalyticsStaticDataWrapper = ({
  exampleNodes,
  collection = '',
  ...rest
}) => {
  const [currentNodes, setCurrentNodes] = React.useState(exampleNodes);
  const [clickedNodes, setClickedNodes] = React.useState([]);
  const [currentPage, setCurrentPage] = React.useState(0);
  const currentPages = React.useMemo(() => chunks(currentNodes, PAGE_SIZE), [currentNodes]);
  React.useEffect(() => {
    setClickedNodes([]);
    setCurrentPage(0);
    setCurrentNodes(exampleNodes);
  }, [exampleNodes]);
  const nodeClicked = node => {
    setCurrentNodes(currentClusters => {
      const clickedNode = currentPages.at(currentPage)?.at(Number(node.id));
      if (clickedNode) {
        setClickedNodes(currentClickedCluster => [...currentClickedCluster, clickedNode]);
        setCurrentPage(0);
      }
      return clickedNode?.children ?? clickedNode?.items?.map(item => {
        return {
          id: item,
          items_count: 1,
          text: item,
          type: 'Item'
        };
      }) ?? currentClusters;
    });
  };
  const goBack = (skip = 0) => {
    setClickedNodes(clickedClusters => {
      for (let i = 0; i < skip; i++) {
        clickedClusters.pop();
      }
      setCurrentPage(0);
      setCurrentNodes(clickedClusters.at(-1)?.children ?? exampleNodes);
      return [...clickedClusters];
    });
  };
  const currentClickedNode = React.useMemo(() => {
    const currentClicked = clickedNodes.at(-1);
    if (currentClicked) {
      return {
        type: currentClicked.type,
        data: getNodeData(currentClicked, 0)
      };
    }
    return;
  }, [clickedNodes]);
  return /*#__PURE__*/React.createElement(OneAiAnalytics, {
    dataNodes: {
      totalItems: exampleNodes.reduce((acc, node) => acc + node.items_count, 0),
      nodes: currentPages.at(currentPage)?.map((node, index) => {
        return {
          type: node.type,
          data: getNodeData(node, index)
        };
      }) ?? []
    },
    currentNode: currentClickedNode,
    nodeClicked: nodeClicked,
    goBackClicked: goBack,
    currentPage: currentPage,
    totalPagesAmount: currentPages.length,
    nextPageClicked: () => setCurrentPage(page => page + 1),
    prevPageClicked: () => setCurrentPage(page => page - 1),
    nodesPath: [{
      text: collection
    }, ...clickedNodes.map(node => ({
      text: node.text
    }))],
    ...rest
  });
};
function getNodeData(node, index) {
  if (node.type === 'Cluster') {
    return {
      cluster_id: index,
      cluster_phrase: node.text,
      collection: '',
      items_count: node.items_count,
      phrases_count: node.children?.length ?? 0,
      metadata: {}
    };
  }
  if (node.type === 'Phrase') {
    return {
      phrase_id: index,
      metadata: {},
      items_count: node.items_count,
      text: node.text
    };
  }
  if (node.type === 'Item') {
    return {
      id: index,
      original_text: node.text,
      metadata: {}
    };
  }
  return;
}

const IGNORE_ID = 'False';
const CUSTOM_VALUE_ID = 'CUSTOM_VALUE';
const COLUMN_TYPES_OPTIONS = [{
  id: IGNORE_ID,
  label: 'ignore'
}, {
  id: 'input',
  label: 'text'
}, {
  id: 'input_translated',
  label: 'translation'
}, {
  id: 'timestamp',
  label: 'date & time',
  validation: value => moment(value, moment.ISO_8601, true).isValid()
}, {
  id: CUSTOM_VALUE_ID,
  label: 'custom value'
}];

function SingleSelect({
  options,
  selectedLabelId,
  onSelect
}) {
  const selected = options.find(o => o.id === (selectedLabelId ?? IGNORE_ID));
  return /*#__PURE__*/React.createElement(react.Listbox, null, /*#__PURE__*/React.createElement("div", {
    className: "relative bg-[#36334B] rounded-lg min-w-[100px]"
  }, /*#__PURE__*/React.createElement(react.Listbox.Button, {
    className: "relative text-left focus:outline-none text-sm flex items-center px-2 py-2 justify-between w-full"
  }, /*#__PURE__*/React.createElement("span", {
    className: "text-[#111111] dark:text-gray-300 text-sm mr-1"
  }, selected?.label ?? 'Unknown'), /*#__PURE__*/React.createElement("span", null, /*#__PURE__*/React.createElement(outline.ChevronDownIcon, {
    className: "h-4 w-4 text-gray-200",
    "aria-hidden": "true"
  }))), /*#__PURE__*/React.createElement(react.Transition, {
    as: React.Fragment,
    leave: "transition ease-in duration-100",
    leaveFrom: "opacity-100",
    leaveTo: "opacity-0"
  }, /*#__PURE__*/React.createElement(react.Listbox.Options, {
    className: "absolute mt-1 z-10 p-1 max-h-60 scrollbar-thin scrollbar-thumb-[#747189] scrollbar-track-[#272533] overflow-y-scroll scrollbar-thumb-rounded-full scrollbar-track-rounded-full rounded-md bg-gray-600 dark:bg-[#272533] py-1 text-base shadow-lg ring-1 ring-gray-500 dark:ring-black ring-opacity-5 focus:outline-none sm:text-sm"
  }, options.map((key, i) => /*#__PURE__*/React.createElement(DropdownOption, {
    label: key.label,
    value: key.id,
    selected: selected?.id === key.id,
    labelClicked: onSelect,
    key: i
  }))))));
}
function DropdownOption({
  label,
  value,
  selected,
  labelClicked
}) {
  return /*#__PURE__*/React.createElement(react.Listbox.Option, {
    className: ({
      active
    }) => `relative cursor-default select-none py-2 pr-4 ${active ? 'bg-gray-400 dark:bg-[#444154] text-white' : 'text-gray-300 dark:text-[#747189]'}`,
    onClick: () => labelClicked(value),
    value: value
  }, /*#__PURE__*/React.createElement("span", {
    className: "w-full flex"
  }, /*#__PURE__*/React.createElement("div", {
    className: "flex items-center"
  }, /*#__PURE__*/React.createElement("input", {
    checked: selected,
    onChange: () => {},
    type: "radio",
    className: "w-4 h-4 text-[#4D4DFE] bg-gray-100 border-gray-300 focus:ring-[#4D4DFE] dark:ring-offset-gray-800 focus:ring-2 dark:bg-[#322F46] dark:border-[#322F46]"
  }), /*#__PURE__*/React.createElement("label", {
    className: `text-sm ml-2 font-medium text-gray-900 dark:text-gray-300 block truncate lowercase first-letter:uppercase ${selected ? 'font-medium' : 'font-normal'}`
  }, label))));
}

const loader = {
  v: '4.8.0',
  meta: {
    g: 'LottieFiles AE 1.0.0',
    a: 'Marlon Muller',
    k: '',
    d: 'loading_cut',
    tc: ''
  },
  fr: 60,
  ip: 0,
  op: 194,
  w: 800,
  h: 800,
  nm: 'Final Comp',
  ddd: 0,
  assets: [],
  layers: [{
    ddd: 0,
    ind: 1,
    ty: 4,
    nm: 'RETANGLE 9',
    sr: 1,
    ks: {
      o: {
        a: 0,
        k: 100,
        ix: 11
      },
      r: {
        a: 1,
        k: [{
          i: {
            x: [0.175],
            y: [1]
          },
          o: {
            x: [0.734],
            y: [0]
          },
          t: 46.018,
          s: [0]
        }, {
          t: 106.078125,
          s: [180]
        }],
        ix: 10
      },
      p: {
        s: true,
        x: {
          a: 0,
          k: 575,
          ix: 3
        },
        y: {
          a: 1,
          k: [{
            i: {
              x: [0.104],
              y: [1]
            },
            o: {
              x: [1],
              y: [0]
            },
            t: 28,
            s: [166.2]
          }, {
            i: {
              x: [0.127],
              y: [1]
            },
            o: {
              x: [1],
              y: [0]
            },
            t: 108.08,
            s: [632.4]
          }, {
            t: 191,
            s: [166.2]
          }],
          ix: 4
        }
      },
      a: {
        a: 0,
        k: [43, -3, 0],
        ix: 1
      },
      s: {
        a: 0,
        k: [70, 70, 100],
        ix: 6
      }
    },
    ao: 0,
    shapes: [{
      ty: 'gr',
      it: [{
        ty: 'rc',
        d: 1,
        s: {
          a: 0,
          k: [334, 334],
          ix: 2
        },
        p: {
          a: 0,
          k: [0, 0],
          ix: 3
        },
        r: {
          a: 0,
          k: 290,
          ix: 4
        },
        nm: 'Rectangle Path 1',
        mn: 'ADBE Vector Shape - Rect',
        hd: false
      }, {
        ty: 'fl',
        c: {
          a: 0,
          k: [0, 1, 1, 1],
          ix: 4
        },
        o: {
          a: 0,
          k: 100,
          ix: 5
        },
        r: 1,
        bm: 0,
        nm: 'Fill 1',
        mn: 'ADBE Vector Graphic - Fill',
        hd: false
      }, {
        ty: 'tr',
        p: {
          a: 0,
          k: [43, -3],
          ix: 2
        },
        a: {
          a: 0,
          k: [0, 0],
          ix: 1
        },
        s: {
          a: 0,
          k: [41.722, 41.722],
          ix: 3
        },
        r: {
          a: 0,
          k: 0,
          ix: 6
        },
        o: {
          a: 0,
          k: 100,
          ix: 7
        },
        sk: {
          a: 0,
          k: 0,
          ix: 4
        },
        sa: {
          a: 0,
          k: 0,
          ix: 5
        },
        nm: 'Transform'
      }],
      nm: 'Rectangle 1',
      np: 2,
      cix: 2,
      bm: 0,
      ix: 1,
      mn: 'ADBE Vector Group',
      hd: false
    }],
    ip: 155,
    op: 194,
    st: 28,
    bm: 0
  }, {
    ddd: 0,
    ind: 2,
    ty: 4,
    nm: 'RETANGLE 8',
    sr: 1,
    ks: {
      o: {
        a: 0,
        k: 100,
        ix: 11
      },
      r: {
        a: 1,
        k: [{
          i: {
            x: [0.175],
            y: [1]
          },
          o: {
            x: [0.734],
            y: [0]
          },
          t: 46.018,
          s: [0]
        }, {
          i: {
            x: [0.557],
            y: [1]
          },
          o: {
            x: [1],
            y: [0]
          },
          t: 106.078,
          s: [180]
        }, {
          t: 169,
          s: [0]
        }],
        ix: 10
      },
      p: {
        s: true,
        x: {
          a: 0,
          k: 575,
          ix: 3
        },
        y: {
          a: 1,
          k: [{
            i: {
              x: [0.104],
              y: [1]
            },
            o: {
              x: [1],
              y: [0]
            },
            t: 28,
            s: [166.2]
          }, {
            i: {
              x: [0.157],
              y: [1]
            },
            o: {
              x: [1],
              y: [0]
            },
            t: 108.08,
            s: [632.4]
          }, {
            t: 188.16015625,
            s: [166.2]
          }],
          ix: 4
        }
      },
      a: {
        a: 0,
        k: [43, -3, 0],
        ix: 1
      },
      s: {
        a: 0,
        k: [70, 70, 100],
        ix: 6
      }
    },
    ao: 0,
    shapes: [{
      ty: 'gr',
      it: [{
        ty: 'rc',
        d: 1,
        s: {
          a: 0,
          k: [334, 334],
          ix: 2
        },
        p: {
          a: 0,
          k: [0, 0],
          ix: 3
        },
        r: {
          a: 1,
          k: [{
            i: {
              x: [0.833],
              y: [1]
            },
            o: {
              x: [0.167],
              y: [0]
            },
            t: 53,
            s: [369]
          }, {
            i: {
              x: [0.833],
              y: [1]
            },
            o: {
              x: [0.333],
              y: [0]
            },
            t: 81,
            s: [1]
          }, {
            i: {
              x: [0.667],
              y: [1]
            },
            o: {
              x: [0.333],
              y: [0]
            },
            t: 134,
            s: [1]
          }, {
            t: 165,
            s: [369]
          }],
          ix: 4
        },
        nm: 'Rectangle Path 1',
        mn: 'ADBE Vector Shape - Rect',
        hd: false
      }, {
        ty: 'fl',
        c: {
          a: 0,
          k: [0.30196078431372547, 0.30196078431372547, 1, 1],
          ix: 4
        },
        o: {
          a: 0,
          k: 100,
          ix: 5
        },
        r: 1,
        bm: 0,
        nm: 'Fill 1',
        mn: 'ADBE Vector Graphic - Fill',
        hd: false
      }, {
        ty: 'tr',
        p: {
          a: 0,
          k: [43, -3],
          ix: 2
        },
        a: {
          a: 0,
          k: [0, 0],
          ix: 1
        },
        s: {
          a: 0,
          k: [41.722, 41.722],
          ix: 3
        },
        r: {
          a: 0,
          k: 0,
          ix: 6
        },
        o: {
          a: 0,
          k: 100,
          ix: 7
        },
        sk: {
          a: 0,
          k: 0,
          ix: 4
        },
        sa: {
          a: 0,
          k: 0,
          ix: 5
        },
        nm: 'Transform'
      }],
      nm: 'Rectangle 1',
      np: 2,
      cix: 2,
      bm: 0,
      ix: 1,
      mn: 'ADBE Vector Group',
      hd: false
    }],
    ip: 72,
    op: 155,
    st: 28,
    bm: 0
  }, {
    ddd: 0,
    ind: 3,
    ty: 4,
    nm: 'RETANGLE 7',
    sr: 1,
    ks: {
      o: {
        a: 0,
        k: 100,
        ix: 11
      },
      r: {
        a: 0,
        k: 0,
        ix: 10
      },
      p: {
        s: true,
        x: {
          a: 0,
          k: 575,
          ix: 3
        },
        y: {
          a: 1,
          k: [{
            i: {
              x: [0.104],
              y: [1]
            },
            o: {
              x: [1],
              y: [0]
            },
            t: 28,
            s: [166.2]
          }, {
            i: {
              x: [0.157],
              y: [1]
            },
            o: {
              x: [1],
              y: [0]
            },
            t: 108.08,
            s: [632.4]
          }, {
            t: 188.16015625,
            s: [166.2]
          }],
          ix: 4
        }
      },
      a: {
        a: 0,
        k: [43, -3, 0],
        ix: 1
      },
      s: {
        a: 0,
        k: [70, 70, 100],
        ix: 6
      }
    },
    ao: 0,
    shapes: [{
      ty: 'gr',
      it: [{
        ty: 'rc',
        d: 1,
        s: {
          a: 0,
          k: [334, 334],
          ix: 2
        },
        p: {
          a: 0,
          k: [0, 0],
          ix: 3
        },
        r: {
          a: 0,
          k: 300,
          ix: 4
        },
        nm: 'Rectangle Path 1',
        mn: 'ADBE Vector Shape - Rect',
        hd: false
      }, {
        ty: 'fl',
        c: {
          a: 0,
          k: [0, 1, 1, 1],
          ix: 4
        },
        o: {
          a: 0,
          k: 100,
          ix: 5
        },
        r: 1,
        bm: 0,
        nm: 'Fill 1',
        mn: 'ADBE Vector Graphic - Fill',
        hd: false
      }, {
        ty: 'tr',
        p: {
          a: 0,
          k: [43, -3],
          ix: 2
        },
        a: {
          a: 0,
          k: [0, 0],
          ix: 1
        },
        s: {
          a: 0,
          k: [41.722, 41.722],
          ix: 3
        },
        r: {
          a: 0,
          k: 0,
          ix: 6
        },
        o: {
          a: 0,
          k: 100,
          ix: 7
        },
        sk: {
          a: 0,
          k: 0,
          ix: 4
        },
        sa: {
          a: 0,
          k: 0,
          ix: 5
        },
        nm: 'Transform'
      }],
      nm: 'Rectangle 1',
      np: 2,
      cix: 2,
      bm: 0,
      ix: 1,
      mn: 'ADBE Vector Group',
      hd: false
    }],
    ip: 0,
    op: 72,
    st: 28,
    bm: 0
  }, {
    ddd: 0,
    ind: 4,
    ty: 4,
    nm: 'RETANGLE 6',
    sr: 1,
    ks: {
      o: {
        a: 0,
        k: 100,
        ix: 11
      },
      r: {
        a: 1,
        k: [{
          i: {
            x: [0.175],
            y: [1]
          },
          o: {
            x: [0.734],
            y: [0]
          },
          t: 32.018,
          s: [0]
        }, {
          t: 92.078125,
          s: [180]
        }],
        ix: 10
      },
      p: {
        s: true,
        x: {
          a: 0,
          k: 225,
          ix: 3
        },
        y: {
          a: 1,
          k: [{
            i: {
              x: [0.104],
              y: [1]
            },
            o: {
              x: [1],
              y: [0]
            },
            t: 14,
            s: [166.2]
          }, {
            i: {
              x: [0.127],
              y: [1]
            },
            o: {
              x: [1],
              y: [0]
            },
            t: 94.08,
            s: [632.4]
          }, {
            t: 177,
            s: [166.2]
          }],
          ix: 4
        }
      },
      a: {
        a: 0,
        k: [43, -3, 0],
        ix: 1
      },
      s: {
        a: 0,
        k: [70, 70, 100],
        ix: 6
      }
    },
    ao: 0,
    shapes: [{
      ty: 'gr',
      it: [{
        ty: 'rc',
        d: 1,
        s: {
          a: 0,
          k: [334, 334],
          ix: 2
        },
        p: {
          a: 0,
          k: [0, 0],
          ix: 3
        },
        r: {
          a: 0,
          k: 290,
          ix: 4
        },
        nm: 'Rectangle Path 1',
        mn: 'ADBE Vector Shape - Rect',
        hd: false
      }, {
        ty: 'fl',
        c: {
          a: 0,
          k: [0, 1, 1, 1],
          ix: 4
        },
        o: {
          a: 0,
          k: 100,
          ix: 5
        },
        r: 1,
        bm: 0,
        nm: 'Fill 1',
        mn: 'ADBE Vector Graphic - Fill',
        hd: false
      }, {
        ty: 'tr',
        p: {
          a: 0,
          k: [43, -3],
          ix: 2
        },
        a: {
          a: 0,
          k: [0, 0],
          ix: 1
        },
        s: {
          a: 0,
          k: [41.722, 41.722],
          ix: 3
        },
        r: {
          a: 0,
          k: 0,
          ix: 6
        },
        o: {
          a: 0,
          k: 100,
          ix: 7
        },
        sk: {
          a: 0,
          k: 0,
          ix: 4
        },
        sa: {
          a: 0,
          k: 0,
          ix: 5
        },
        nm: 'Transform'
      }],
      nm: 'Rectangle 1',
      np: 2,
      cix: 2,
      bm: 0,
      ix: 1,
      mn: 'ADBE Vector Group',
      hd: false
    }],
    ip: 141,
    op: 198,
    st: 14,
    bm: 0
  }, {
    ddd: 0,
    ind: 5,
    ty: 4,
    nm: 'RETANGLE 5',
    sr: 1,
    ks: {
      o: {
        a: 0,
        k: 100,
        ix: 11
      },
      r: {
        a: 1,
        k: [{
          i: {
            x: [0.175],
            y: [1]
          },
          o: {
            x: [0.734],
            y: [0]
          },
          t: 32.018,
          s: [0]
        }, {
          i: {
            x: [0.557],
            y: [1]
          },
          o: {
            x: [1],
            y: [0]
          },
          t: 92.078,
          s: [180]
        }, {
          t: 155,
          s: [0]
        }],
        ix: 10
      },
      p: {
        s: true,
        x: {
          a: 0,
          k: 225,
          ix: 3
        },
        y: {
          a: 1,
          k: [{
            i: {
              x: [0.104],
              y: [1]
            },
            o: {
              x: [1],
              y: [0]
            },
            t: 14,
            s: [166.2]
          }, {
            i: {
              x: [0.157],
              y: [1]
            },
            o: {
              x: [1],
              y: [0]
            },
            t: 94.08,
            s: [632.4]
          }, {
            t: 174.16015625,
            s: [166.2]
          }],
          ix: 4
        }
      },
      a: {
        a: 0,
        k: [43, -3, 0],
        ix: 1
      },
      s: {
        a: 0,
        k: [70, 70, 100],
        ix: 6
      }
    },
    ao: 0,
    shapes: [{
      ty: 'gr',
      it: [{
        ty: 'rc',
        d: 1,
        s: {
          a: 0,
          k: [334, 334],
          ix: 2
        },
        p: {
          a: 0,
          k: [0, 0],
          ix: 3
        },
        r: {
          a: 1,
          k: [{
            i: {
              x: [0.833],
              y: [1]
            },
            o: {
              x: [0.167],
              y: [0]
            },
            t: 39,
            s: [369]
          }, {
            i: {
              x: [0.833],
              y: [1]
            },
            o: {
              x: [0.333],
              y: [0]
            },
            t: 67,
            s: [1]
          }, {
            i: {
              x: [0.667],
              y: [1]
            },
            o: {
              x: [0.333],
              y: [0]
            },
            t: 120,
            s: [1]
          }, {
            t: 151,
            s: [369]
          }],
          ix: 4
        },
        nm: 'Rectangle Path 1',
        mn: 'ADBE Vector Shape - Rect',
        hd: false
      }, {
        ty: 'fl',
        c: {
          a: 0,
          k: [1, 0.9098039215686274, 0, 1],
          ix: 4
        },
        o: {
          a: 0,
          k: 100,
          ix: 5
        },
        r: 1,
        bm: 0,
        nm: 'Fill 1',
        mn: 'ADBE Vector Graphic - Fill',
        hd: false
      }, {
        ty: 'tr',
        p: {
          a: 0,
          k: [43, -3],
          ix: 2
        },
        a: {
          a: 0,
          k: [0, 0],
          ix: 1
        },
        s: {
          a: 0,
          k: [41.722, 41.722],
          ix: 3
        },
        r: {
          a: 0,
          k: 0,
          ix: 6
        },
        o: {
          a: 0,
          k: 100,
          ix: 7
        },
        sk: {
          a: 0,
          k: 0,
          ix: 4
        },
        sa: {
          a: 0,
          k: 0,
          ix: 5
        },
        nm: 'Transform'
      }],
      nm: 'Rectangle 1',
      np: 2,
      cix: 2,
      bm: 0,
      ix: 1,
      mn: 'ADBE Vector Group',
      hd: false
    }],
    ip: 58,
    op: 141,
    st: 14,
    bm: 0
  }, {
    ddd: 0,
    ind: 6,
    ty: 4,
    nm: 'RETANGLE 4',
    sr: 1,
    ks: {
      o: {
        a: 0,
        k: 100,
        ix: 11
      },
      r: {
        a: 0,
        k: 0,
        ix: 10
      },
      p: {
        s: true,
        x: {
          a: 0,
          k: 225,
          ix: 3
        },
        y: {
          a: 1,
          k: [{
            i: {
              x: [0.104],
              y: [1]
            },
            o: {
              x: [1],
              y: [0]
            },
            t: 14,
            s: [166.2]
          }, {
            i: {
              x: [0.157],
              y: [1]
            },
            o: {
              x: [1],
              y: [0]
            },
            t: 94.08,
            s: [632.4]
          }, {
            t: 174.16015625,
            s: [166.2]
          }],
          ix: 4
        }
      },
      a: {
        a: 0,
        k: [43, -3, 0],
        ix: 1
      },
      s: {
        a: 0,
        k: [70, 70, 100],
        ix: 6
      }
    },
    ao: 0,
    shapes: [{
      ty: 'gr',
      it: [{
        ty: 'rc',
        d: 1,
        s: {
          a: 0,
          k: [334, 334],
          ix: 2
        },
        p: {
          a: 0,
          k: [0, 0],
          ix: 3
        },
        r: {
          a: 0,
          k: 300,
          ix: 4
        },
        nm: 'Rectangle Path 1',
        mn: 'ADBE Vector Shape - Rect',
        hd: false
      }, {
        ty: 'fl',
        c: {
          a: 0,
          k: [0, 1, 1, 1],
          ix: 4
        },
        o: {
          a: 0,
          k: 100,
          ix: 5
        },
        r: 1,
        bm: 0,
        nm: 'Fill 1',
        mn: 'ADBE Vector Graphic - Fill',
        hd: false
      }, {
        ty: 'tr',
        p: {
          a: 0,
          k: [43, -3],
          ix: 2
        },
        a: {
          a: 0,
          k: [0, 0],
          ix: 1
        },
        s: {
          a: 0,
          k: [41.722, 41.722],
          ix: 3
        },
        r: {
          a: 0,
          k: 0,
          ix: 6
        },
        o: {
          a: 0,
          k: 100,
          ix: 7
        },
        sk: {
          a: 0,
          k: 0,
          ix: 4
        },
        sa: {
          a: 0,
          k: 0,
          ix: 5
        },
        nm: 'Transform'
      }],
      nm: 'Rectangle 1',
      np: 2,
      cix: 2,
      bm: 0,
      ix: 1,
      mn: 'ADBE Vector Group',
      hd: false
    }],
    ip: 0,
    op: 58,
    st: 14,
    bm: 0
  }, {
    ddd: 0,
    ind: 7,
    ty: 4,
    nm: 'RETANGLE 3',
    sr: 1,
    ks: {
      o: {
        a: 0,
        k: 100,
        ix: 11
      },
      r: {
        a: 1,
        k: [{
          i: {
            x: [0.175],
            y: [1]
          },
          o: {
            x: [0.734],
            y: [0]
          },
          t: 18.018,
          s: [0]
        }, {
          t: 78.078125,
          s: [180]
        }],
        ix: 10
      },
      p: {
        s: true,
        x: {
          a: 0,
          k: 400,
          ix: 3
        },
        y: {
          a: 1,
          k: [{
            i: {
              x: [0.104],
              y: [1]
            },
            o: {
              x: [1],
              y: [0]
            },
            t: 0,
            s: [166.2]
          }, {
            i: {
              x: [0.127],
              y: [1]
            },
            o: {
              x: [1],
              y: [0]
            },
            t: 80.08,
            s: [632.4]
          }, {
            t: 163,
            s: [166.2]
          }],
          ix: 4
        }
      },
      a: {
        a: 0,
        k: [43, -3, 0],
        ix: 1
      },
      s: {
        a: 0,
        k: [70, 70, 100],
        ix: 6
      }
    },
    ao: 0,
    shapes: [{
      ty: 'gr',
      it: [{
        ty: 'rc',
        d: 1,
        s: {
          a: 0,
          k: [334, 334],
          ix: 2
        },
        p: {
          a: 0,
          k: [0, 0],
          ix: 3
        },
        r: {
          a: 0,
          k: 290,
          ix: 4
        },
        nm: 'Rectangle Path 1',
        mn: 'ADBE Vector Shape - Rect',
        hd: false
      }, {
        ty: 'fl',
        c: {
          a: 0,
          k: [0, 1, 1, 1],
          ix: 4
        },
        o: {
          a: 0,
          k: 100,
          ix: 5
        },
        r: 1,
        bm: 0,
        nm: 'Fill 1',
        mn: 'ADBE Vector Graphic - Fill',
        hd: false
      }, {
        ty: 'tr',
        p: {
          a: 0,
          k: [43, -3],
          ix: 2
        },
        a: {
          a: 0,
          k: [0, 0],
          ix: 1
        },
        s: {
          a: 0,
          k: [41.722, 41.722],
          ix: 3
        },
        r: {
          a: 0,
          k: 0,
          ix: 6
        },
        o: {
          a: 0,
          k: 100,
          ix: 7
        },
        sk: {
          a: 0,
          k: 0,
          ix: 4
        },
        sa: {
          a: 0,
          k: 0,
          ix: 5
        },
        nm: 'Transform'
      }],
      nm: 'Rectangle 1',
      np: 2,
      cix: 2,
      bm: 0,
      ix: 1,
      mn: 'ADBE Vector Group',
      hd: false
    }],
    ip: 127,
    op: 197,
    st: 0,
    bm: 0
  }, {
    ddd: 0,
    ind: 8,
    ty: 4,
    nm: 'RETANGLE 2',
    sr: 1,
    ks: {
      o: {
        a: 0,
        k: 100,
        ix: 11
      },
      r: {
        a: 1,
        k: [{
          i: {
            x: [0.175],
            y: [1]
          },
          o: {
            x: [0.734],
            y: [0]
          },
          t: 18.018,
          s: [0]
        }, {
          i: {
            x: [0.557],
            y: [1]
          },
          o: {
            x: [1],
            y: [0]
          },
          t: 78.078,
          s: [180]
        }, {
          t: 141,
          s: [0]
        }],
        ix: 10
      },
      p: {
        s: true,
        x: {
          a: 0,
          k: 400,
          ix: 3
        },
        y: {
          a: 1,
          k: [{
            i: {
              x: [0.104],
              y: [1]
            },
            o: {
              x: [1],
              y: [0]
            },
            t: 0,
            s: [166.2]
          }, {
            i: {
              x: [0.157],
              y: [1]
            },
            o: {
              x: [1],
              y: [0]
            },
            t: 80.08,
            s: [632.4]
          }, {
            t: 160.16015625,
            s: [166.2]
          }],
          ix: 4
        }
      },
      a: {
        a: 0,
        k: [43, -3, 0],
        ix: 1
      },
      s: {
        a: 0,
        k: [70, 70, 100],
        ix: 6
      }
    },
    ao: 0,
    shapes: [{
      ty: 'gr',
      it: [{
        ty: 'rc',
        d: 1,
        s: {
          a: 0,
          k: [334, 334],
          ix: 2
        },
        p: {
          a: 0,
          k: [0, 0],
          ix: 3
        },
        r: {
          a: 1,
          k: [{
            i: {
              x: [0.833],
              y: [1]
            },
            o: {
              x: [0.167],
              y: [0]
            },
            t: 25,
            s: [369]
          }, {
            i: {
              x: [0.833],
              y: [1]
            },
            o: {
              x: [0.333],
              y: [0]
            },
            t: 53,
            s: [1]
          }, {
            i: {
              x: [0.667],
              y: [1]
            },
            o: {
              x: [0.333],
              y: [0]
            },
            t: 106,
            s: [1]
          }, {
            t: 137,
            s: [369]
          }],
          ix: 4
        },
        nm: 'Rectangle Path 1',
        mn: 'ADBE Vector Shape - Rect',
        hd: false
      }, {
        ty: 'fl',
        c: {
          a: 0,
          k: [0.9490196078431372, 0.23921568627450981, 0.9137254901960784, 1],
          ix: 4
        },
        o: {
          a: 0,
          k: 100,
          ix: 5
        },
        r: 1,
        bm: 0,
        nm: 'Fill 1',
        mn: 'ADBE Vector Graphic - Fill',
        hd: false
      }, {
        ty: 'tr',
        p: {
          a: 0,
          k: [43, -3],
          ix: 2
        },
        a: {
          a: 0,
          k: [0, 0],
          ix: 1
        },
        s: {
          a: 0,
          k: [41.722, 41.722],
          ix: 3
        },
        r: {
          a: 0,
          k: 0,
          ix: 6
        },
        o: {
          a: 0,
          k: 100,
          ix: 7
        },
        sk: {
          a: 0,
          k: 0,
          ix: 4
        },
        sa: {
          a: 0,
          k: 0,
          ix: 5
        },
        nm: 'Transform'
      }],
      nm: 'Rectangle 1',
      np: 2,
      cix: 2,
      bm: 0,
      ix: 1,
      mn: 'ADBE Vector Group',
      hd: false
    }],
    ip: 44,
    op: 127,
    st: 0,
    bm: 0
  }, {
    ddd: 0,
    ind: 9,
    ty: 4,
    nm: 'RETANGLE',
    sr: 1,
    ks: {
      o: {
        a: 0,
        k: 100,
        ix: 11
      },
      r: {
        a: 0,
        k: 0,
        ix: 10
      },
      p: {
        s: true,
        x: {
          a: 0,
          k: 400,
          ix: 3
        },
        y: {
          a: 1,
          k: [{
            i: {
              x: [0.104],
              y: [1]
            },
            o: {
              x: [1],
              y: [0]
            },
            t: 0,
            s: [166.2]
          }, {
            i: {
              x: [0.157],
              y: [1]
            },
            o: {
              x: [1],
              y: [0]
            },
            t: 80.08,
            s: [632.4]
          }, {
            t: 160.16015625,
            s: [166.2]
          }],
          ix: 4
        }
      },
      a: {
        a: 0,
        k: [43, -3, 0],
        ix: 1
      },
      s: {
        a: 0,
        k: [70, 70, 100],
        ix: 6
      }
    },
    ao: 0,
    shapes: [{
      ty: 'gr',
      it: [{
        ty: 'rc',
        d: 1,
        s: {
          a: 0,
          k: [334, 334],
          ix: 2
        },
        p: {
          a: 0,
          k: [0, 0],
          ix: 3
        },
        r: {
          a: 0,
          k: 300,
          ix: 4
        },
        nm: 'Rectangle Path 1',
        mn: 'ADBE Vector Shape - Rect',
        hd: false
      }, {
        ty: 'fl',
        c: {
          a: 0,
          k: [0, 1, 1, 1],
          ix: 4
        },
        o: {
          a: 0,
          k: 100,
          ix: 5
        },
        r: 1,
        bm: 0,
        nm: 'Fill 1',
        mn: 'ADBE Vector Graphic - Fill',
        hd: false
      }, {
        ty: 'tr',
        p: {
          a: 0,
          k: [43, -3],
          ix: 2
        },
        a: {
          a: 0,
          k: [0, 0],
          ix: 1
        },
        s: {
          a: 0,
          k: [41.722, 41.722],
          ix: 3
        },
        r: {
          a: 0,
          k: 0,
          ix: 6
        },
        o: {
          a: 0,
          k: 100,
          ix: 7
        },
        sk: {
          a: 0,
          k: 0,
          ix: 4
        },
        sa: {
          a: 0,
          k: 0,
          ix: 5
        },
        nm: 'Transform'
      }],
      nm: 'Rectangle 1',
      np: 2,
      cix: 2,
      bm: 0,
      ix: 1,
      mn: 'ADBE Vector Group',
      hd: false
    }],
    ip: 0,
    op: 44,
    st: 0,
    bm: 0
  }],
  markers: [{
    tm: 44,
    cm: '1',
    dr: 0
  }, {
    tm: 129,
    cm: '2',
    dr: 0
  }]
};

const OneAiLoader = ({
  height = '220px',
  width = '180px'
}) => {
  const ref = React.useRef(null);
  const [lottie, setLottie] = React.useState(null);
  React.useEffect(() => {
    import('lottie-web').then(Lottie => setLottie(Lottie.default));
  }, []);
  React.useEffect(() => {
    if (lottie && ref.current) {
      const animation = lottie.loadAnimation({
        container: ref.current,
        renderer: 'svg',
        loop: true,
        autoplay: true,
        animationData: loader
      });
      return () => animation.destroy();
    }
    return;
  }, [lottie]);
  return /*#__PURE__*/React.createElement("div", {
    ref: ref,
    role: "alert",
    "aria-busy": "true",
    style: {
      height,
      width
    }
  });
};

const allowedExtensions = ['csv'];
const OneAiUpload = ({
  domain = 'prod',
  apiKey = '',
  collection = '',
  darkMode = true,
  steps = '',
  input_skill,
  resetAfterUpload = true,
  expected_languages = '',
  override_language_detection = false,
  createCollection = false,
  collectionDomain = 'survey',
  isPublic = false,
  goToCollection
}) => {
  domain = resolveDomain(domain);
  const [data, setData] = React.useState([]);
  const [error, setError] = React.useState(null);
  const [file, setFile] = React.useState(null);
  const [columnsConfigurations, setColumnsConfigurations] = React.useState([]);
  const [loading, setLoading] = React.useState(false);
  const [uploaded, setUploaded] = React.useState(false);
  const [csvHasHeaders, setCsvHasHeaders] = React.useState(true);
  const [numberOfRowsToSkip, setNumberOfRowsToSkip] = React.useState(0);
  const [numberOfRowsToDisplay, setNumberOfRowsToDisplay] = React.useState(100);
  const [maxRows, setMaxRows] = React.useState(null);
  const [taskId, setTaskId] = React.useState(null);
  const [uploadStatus, setUploadStatus] = React.useState(null);
  const [totalNumberOfRows, setTotalNumberOfRows] = React.useState(0);
  const [collectionItemsAmount, setCollectionItemsAmount] = React.useState(null);
  const currentParser = React.useRef(null);
  const initiated = React.useRef(false);
  React.useEffect(() => {
    if (!taskId || !uploaded) return;
    const interval = setInterval(async () => {
      const res = await fetch(encodeURI(`${domain}/api/v0/pipeline/async/tasks/${taskId}`), {
        method: 'GET',
        headers: {
          'Content-Type': 'application/json',
          'api-key': apiKey ?? ''
        }
      });
      const data = await res.json();
      if (data.status === 'COMPLETED') {
        setUploaded(true);
        setUploadStatus('completed');
        if (resetAfterUpload) {
          handleReset();
        }
        clearInterval(interval);
      } else if (data.status === 'FAILED') {
        setUploadStatus('failed');
        clearInterval(interval);
      } else {
        setUploadStatus('in progress');
      }
    }, 1000);
    return () => clearInterval(interval);
  }, [taskId, uploaded]);
  React.useEffect(() => {
    if (initiated.current) return;
    initiated.current = true;
    const fetchData = async abortController => {
      const res = await fetch(encodeURI(`${domain}/clustering/v1/collections/${collection}/clusters?limit=1`), {
        method: 'GET',
        headers: {
          'Content-Type': 'application/json',
          'api-key': apiKey ?? ''
        },
        signal: abortController.signal
      });
      const data = await res.json();
      setCollectionItemsAmount(() => {
        setLoading(false);
        return data.total_items;
      });
    };
    const controller = new AbortController();
    if (domain.length === 0 || apiKey.length === 0 || collection.length === 0) return;
    fetchData(controller);
    return () => controller.abort();
  }, [domain, apiKey, collection]);
  const handleFileChange = e => {
    setError(null);
    setUploaded(false);
    try {
      currentParser.current?.abort();
    } catch (e) {
      console.error(e);
    }
    const inputFile = e.target.files?.item(0);
    if (!inputFile) return;
    // Check if user has entered the file
    // Check the file extensions, if it not
    // included in the allowed extensions
    // we show the error
    const fileExtension = inputFile.type.split('/')[1];
    if (!allowedExtensions.includes(fileExtension)) {
      setError('Please input a csv file');
      return;
    }
    // If input type is correct set the state
    setFile(inputFile);
    handleParse(inputFile);
  };
  const handleParse = file => {
    // If user clicks the parse button without
    // a file we show a error
    if (!file) return setError('Enter a valid file');
    let finished = false;
    Papa.parse(file, {
      worker: true,
      encoding: 'utf-8',
      chunkSize: 512,
      step: function ({
        data,
        errors
      }, parser) {
        if (errors.length > 0) {
          console.log('errors', errors[0]);
        }
        currentParser.current = parser;
        if (!finished) {
          setData(prev => {
            if (prev.length === 0) {
              setColumnsConfigurations(data.map(() => ({
                id: IGNORE_ID
              })));
            }
            if (prev.length > 200) {
              finished = true;
            }
            return [...prev, data];
          });
        }
        setTotalNumberOfRows(prev => prev + 1);
      },
      complete: function (results, file) {
        console.log('parsing complete read', results, file);
      }
    });
  };
  const handleReset = () => {
    setData([]);
    setFile(null);
    setColumnsConfigurations([]);
    setCsvHasHeaders(true);
    setNumberOfRowsToSkip(0);
    setMaxRows(null);
  };
  const handleUpload = async () => {
    if (!file) return;
    setLoading(true);
    if (createCollection) {
      try {
        const res = await fetch(encodeURI(`${domain}/clustering/v1/collections/${collection}/create`), {
          method: 'POST',
          headers: {
            'Content-Type': 'application/json',
            'api-key': apiKey ?? ''
          },
          body: JSON.stringify({
            access: {
              ...(isPublic && {
                all: {
                  query: true,
                  list_clusters: true,
                  list_items: true,
                  add_items: false,
                  edit_clusters: false,
                  discoverable: true
                }
              })
            },
            domain: collectionDomain
          })
        });
        if (!res.ok) {
          const error = await res.json();
          console.error('error', error);
          setLoading(false);
          setError("Couldn't create collection");
          return;
        }
      } catch (e) {
        console.error(e);
        setLoading(false);
        setError("Couldn't create collection");
        return;
      }
    }
    const appendSteps = steps.length > 0 ? steps.replaceAll('[', '').replaceAll(']', '').replaceAll(' ', '').split(',') : [];
    const expectedLanguages = expected_languages.length > 0 ? expected_languages.replaceAll('[', '').replaceAll(']', '').replaceAll(' ', '').split(',') : [];
    try {
      const pipelineJson = {
        content_type: 'text/csv',
        multilingual: {
          enabled: true,
          ...(expectedLanguages.length > 0 && {
            expected_languages: expectedLanguages
          }),
          ...(override_language_detection && {
            override_language_detection
          })
        },
        steps: [...(appendSteps.length > 0 ? appendSteps.map(step => ({
          skill: step
        })) : []), {
          skill: 'clustering',
          params: {
            collection,
            ...(input_skill && {
              input_skill
            })
          }
        }],
        csv_params: {
          columns: columnsConfigurations.map(cc => cc.id === IGNORE_ID ? false : cc.id === CUSTOM_VALUE_ID ? cc.customText : cc.id),
          skip_rows: (csvHasHeaders ? 1 : 0) + numberOfRowsToSkip,
          max_rows: maxRows !== null ? maxRows : totalNumberOfRows
        }
      };
      const fetchFormData = new FormData();
      fetchFormData.append('file', file);
      const uploadRes = await fetch(encodeURI(`${domain}/api/v0/pipeline/async/file?pipeline=${JSON.stringify(pipelineJson)}`), {
        method: 'POST',
        headers: {
          'Content-Type': 'text/csv',
          'api-key': apiKey ?? '',
          'Content-Disposition': `attachment; filename=${encodeURI(file.name)}`
        },
        body: file
      });
      if (!uploadRes.ok) {
        const error = await uploadRes.json();
        console.error('error', error);
        setLoading(false);
        setError("Couldn't upload file");
        return;
      }
      const statusData = await uploadRes.json();
      console.log('file uploaded', statusData);
      setUploadStatus(statusData['status']);
      setTaskId(statusData['task_id']);
    } catch (e) {
      console.error(e);
      setLoading(false);
      setError("Couldn't upload file");
      return;
    }
    setError(null);
    if (resetAfterUpload) {
      handleReset();
    }
    setUploaded(true);
    setLoading(false);
  };
  React.useEffect(() => {
    ReactTooltip.hide();
    ReactTooltip.rebuild();
  });
  const uploadDisabled = columnsConfigurations.filter(c => c.id === 'input').length !== 1;
  const handleDragOver = e => {
    e.preventDefault();
  };
  const handleDragAndDrop = event => {
    if (uploaded || file) {
      return;
    }
    const target = event.dataTransfer;
    event.preventDefault();
    if (target && target.files.length > 0) {
      handleFileChange({
        target: {
          files: target.files
        }
      });
    }
  };
  return /*#__PURE__*/React.createElement("div", {
    onDrop: handleDragAndDrop,
    onDragOver: handleDragOver,
    className: `oneai-analytics-namespace h-full w-full p-2 ${darkMode ? 'dark' : ''}`
  }, /*#__PURE__*/React.createElement("div", {
    className: `h-full w-full overflow-hidden bg-[#272535] flex flex-col items-center text-white ${darkMode ? 'dark' : ''}`
  }, /*#__PURE__*/React.createElement(ReactTooltip, {
    id: "global"
  }), error ? /*#__PURE__*/React.createElement("div", {
    className: "w-full p-2 relative h-2/6"
  }, /*#__PURE__*/React.createElement("div", {
    className: "absolute top-0 right-0 p-2"
  }, /*#__PURE__*/React.createElement("svg", {
    onClick: () => setError(null),
    className: "w-6 h-6 text-white cursor-pointer",
    xmlns: "http://www.w3.org/2000/svg",
    fill: "none",
    viewBox: "0 0 24 24",
    stroke: "currentColor"
  }, /*#__PURE__*/React.createElement("path", {
    strokeLinecap: "round",
    strokeLinejoin: "round",
    strokeWidth: 2,
    d: "M6 18L18 6M6 6l12 12"
  }))), /*#__PURE__*/React.createElement("div", {
    className: "w-full h-full flex flex-col items-center justify-center"
  }, /*#__PURE__*/React.createElement("svg", {
    className: "w-20 h-20 text-red-500",
    xmlns: "http://www.w3.org/2000/svg",
    fill: "none",
    viewBox: "0 0 24 24",
    stroke: "currentColor"
  }, /*#__PURE__*/React.createElement("path", {
    strokeLinecap: "round",
    strokeLinejoin: "round",
    strokeWidth: 2,
    d: "M6 18L18 6M6 6l12 12"
  })), /*#__PURE__*/React.createElement("div", {
    className: "text-2xl font-bold mt-2"
  }, error))) : /*#__PURE__*/React.createElement(React.Fragment, null), uploaded ? /*#__PURE__*/React.createElement("div", {
    className: "w-full p-2 h-full"
  }, /*#__PURE__*/React.createElement("div", {
    className: "w-full h-full flex flex-col items-center justify-center"
  }, uploadStatus === 'completed' ? /*#__PURE__*/React.createElement("svg", {
    className: "w-20 h-20 text-green-500",
    xmlns: "http://www.w3.org/2000/svg",
    fill: "none",
    viewBox: "0 0 24 24",
    stroke: "currentColor"
  }, /*#__PURE__*/React.createElement("path", {
    strokeLinecap: "round",
    strokeLinejoin: "round",
    strokeWidth: 2,
    d: "M5 13l4 4L19 7"
  })) : null, uploadStatus === 'completed' ? /*#__PURE__*/React.createElement("h1", {
    className: "text-2xl font-bold mt-4"
  }, "Upload Complete") : /*#__PURE__*/React.createElement("div", {
    className: "grid grid-flow-row-dense justify-center items-center"
  }, /*#__PURE__*/React.createElement(OneAiLoader, null), /*#__PURE__*/React.createElement("p", {
    className: "font-medium text-center text-lg"
  }, "Processing...")), uploadStatus === 'completed' ? /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("p", {
    className: "text-lg mt-2"
  }, data.length > 0 ? (maxRows ?? totalNumberOfRows) + ' items' : 'Data', ' ', "has been uploaded to '", collection, "'")) : null, /*#__PURE__*/React.createElement("div", {
    className: "flex flex-col w-full justify-center items-center"
  }, /*#__PURE__*/React.createElement("button", {
    className: "bg-[#4D4DFE] text-white text-lg font-normal py-2 px-4 rounded-md mt-4",
    onClick: () => {
      setUploaded(false);
      setUploadStatus(null);
      setTaskId(null);
    }
  }, "Upload another file"), goToCollection && uploadStatus !== 'in progress' && uploadStatus !== null ? /*#__PURE__*/React.createElement("button", {
    className: "bg-[#4D4DFE] text-white text-lg font-normal py-2 px-4 rounded-md mt-4",
    onClick: () => goToCollection()
  }, "Go back to collection") : null))) : data.length > 0 ? /*#__PURE__*/React.createElement("div", {
    className: `w-full p-2 ${uploaded || error ? 'h-4/6' : 'h-full'}`
  }, loading ? /*#__PURE__*/React.createElement("div", {
    className: "absolute top-0 left-0 w-full h-full bg-black opacity-50 z-10"
  }, /*#__PURE__*/React.createElement("div", {
    className: "absolute top-1/2 left-1/2 transform -translate-x-1/2 -translate-y-1/2 flex flex-col items-center text-center"
  }, /*#__PURE__*/React.createElement("svg", {
    className: "animate-spin -ml-1 mr-3 h-5 w-5 text-white",
    xmlns: "http://www.w3.org/2000/svg",
    fill: "none",
    viewBox: "0 0 24 24"
  }, /*#__PURE__*/React.createElement("circle", {
    className: "opacity-25",
    cx: "12",
    cy: "12",
    r: "10",
    stroke: "currentColor",
    strokeWidth: "4"
  }), /*#__PURE__*/React.createElement("path", {
    className: "opacity-75",
    fill: "currentColor",
    d: "M4 12a8 8 0 018-8v1a7 7 0 00-7 7h1z"
  })), /*#__PURE__*/React.createElement("span", {
    className: "text-white"
  }, "Loading..."))) : null, /*#__PURE__*/React.createElement("div", {
    className: 'h-full w-full flex flex-col relative ' + (loading ? 'pointer-events-none' : '')
  }, /*#__PURE__*/React.createElement("div", {
    className: "h-auto w-full flex justify-between"
  }, /*#__PURE__*/React.createElement("span", {
    className: "text-gray-200 pl-2"
  }, "Select columns for upload to", ' ', /*#__PURE__*/React.createElement("span", {
    className: "text-white"
  }, "' ", collection, " '")), /*#__PURE__*/React.createElement("div", {
    className: "flex items-center"
  }, /*#__PURE__*/React.createElement("span", {
    className: "text-gray-200 pr-2"
  }, "Preview rows"), /*#__PURE__*/React.createElement("input", {
    className: "w-16 h-8 text-center text-gray-700 dark:text-gray-400 bg-gray-200 dark:bg-gray-700 rounded-lg focus:outline-none focus:ring-2 focus:ring-gray-600 focus:border-transparent",
    type: "number",
    value: numberOfRowsToDisplay,
    onChange: e => setNumberOfRowsToDisplay(Number(e.target.value))
  }))), /*#__PURE__*/React.createElement("div", {
    className: "relative overflow-auto max-h-full block shadow-md sm:rounded-lg grow w-full scrollbar-upload"
  }, /*#__PURE__*/React.createElement("table", {
    className: "w-full text-sm text-left text-gray-500 dark:text-gray-400"
  }, /*#__PURE__*/React.createElement("thead", {
    className: "text-xs text-gray-700 uppercase dark:text-gray-400 sticky top-0 bg-[#2B293B]"
  }, /*#__PURE__*/React.createElement("tr", null, data[0].map((header, i) => /*#__PURE__*/React.createElement("th", {
    key: `${header} - ${i}`,
    scope: "col",
    className: "px-2 py-3 w-max"
  }, /*#__PURE__*/React.createElement("div", {
    className: "w-full flex flex-col items-start"
  }, /*#__PURE__*/React.createElement("div", {
    className: "w-max"
  }, /*#__PURE__*/React.createElement(SingleSelect, {
    options: COLUMN_TYPES_OPTIONS,
    selectedLabelId: columnsConfigurations[i].id ?? null,
    onSelect: selectedLabelId => {
      setColumnsConfigurations(prev => {
        const newColumnsConfigurations = [...prev];
        newColumnsConfigurations[i] = {
          id: selectedLabelId,
          ...(selectedLabelId === CUSTOM_VALUE_ID && {
            customText: csvHasHeaders ? header : `Column ${i + 1}`
          })
        };
        return newColumnsConfigurations;
      });
    }
  }), /*#__PURE__*/React.createElement("div", {
    className: "w-max h-6"
  }, columnsConfigurations[i].id === CUSTOM_VALUE_ID ? /*#__PURE__*/React.createElement("input", {
    type: "text",
    className: "bg-gray-50 border border-gray-300 text-gray-900 text-sm rounded-lg block dark:bg-gray-700 dark:border-gray-600 dark:placeholder-gray-400 dark:text-white",
    placeholder: columnsConfigurations[i].customText,
    onChange: e => columnsConfigurations[i].customText = e.target.value.length > 0 ? e.target.value : csvHasHeaders ? header : `Column ${i + 1}`
  }) : null)), /*#__PURE__*/React.createElement("span", {
    className: "w-max pl-1"
  }, csvHasHeaders && header ? header : `Column ${i + 1}`)))))), /*#__PURE__*/React.createElement("tbody", {
    className: "h-96 overflow-y-auto"
  }, data.slice(csvHasHeaders ? 1 : 0, numberOfRowsToDisplay).map((row, i) => /*#__PURE__*/React.createElement("tr", {
    key: i,
    className: "border-b border-gray-200 dark:border-gray-700"
  }, row.map((cell, i) => /*#__PURE__*/React.createElement("td", {
    key: i,
    className: "px-2 py-3 max-w-[200px] truncate"
  }, cell))))))), /*#__PURE__*/React.createElement("div", {
    className: "absolute bottom-4 w-full"
  }, /*#__PURE__*/React.createElement("div", {
    className: "flex justify-between mb-2 p-4 backdrop-blur-[2px]"
  }, /*#__PURE__*/React.createElement("div", {
    className: "flex"
  }, /*#__PURE__*/React.createElement("div", {
    className: "flex items-center mr-4"
  }, /*#__PURE__*/React.createElement("label", {
    htmlFor: "checkbox",
    className: "ml-2 text-sm font-medium text-gray-900 dark:text-gray-300"
  }, "CSV has headers"), /*#__PURE__*/React.createElement("input", {
    checked: csvHasHeaders,
    onChange: e => setCsvHasHeaders(e.target.checked),
    id: "checkbox",
    type: "checkbox",
    className: "w-4 h-4 text-indigo-600 bg-gray-100 border-gray-300 rounded focus:ring-indigo-500 dark:focus:ring-indigo-600 dark:ring-offset-gray-800 focus:ring-2 dark:bg-gray-700 dark:border-gray-600"
  })), /*#__PURE__*/React.createElement("div", {
    className: "flex items-center"
  }, /*#__PURE__*/React.createElement("label", {
    htmlFor: "checkbox",
    "data-for": "global",
    "data-tip": "Limit number of rows to upload",
    className: "ml-2 text-sm font-medium text-gray-900 dark:text-gray-300"
  }, "Upload rows"), /*#__PURE__*/React.createElement("input", {
    value: maxRows ?? totalNumberOfRows,
    onChange: e => setMaxRows(Number(e.target.value || totalNumberOfRows)),
    type: "number",
    className: "w-16 h-8 text-center text-gray-700 dark:text-gray-400 bg-gray-200 dark:bg-gray-700 rounded-lg focus:outline-none focus:ring-2 focus:ring-gray-600 focus:border-transparent"
  })), /*#__PURE__*/React.createElement("div", {
    className: "flex items-center"
  }, /*#__PURE__*/React.createElement("label", {
    htmlFor: "checkbox",
    className: "ml-2 text-sm font-medium text-gray-900 dark:text-gray-300"
  }, "Skip rows"), /*#__PURE__*/React.createElement("input", {
    value: numberOfRowsToSkip,
    onChange: e => setNumberOfRowsToSkip(Number(e.target.value)),
    type: "number",
    className: "w-16 h-8 text-center text-gray-700 dark:text-gray-400 bg-gray-200 dark:bg-gray-700 rounded-lg focus:outline-none focus:ring-2 focus:ring-gray-600 focus:border-transparent"
  }))), /*#__PURE__*/React.createElement("div", {
    className: "flex"
  }, /*#__PURE__*/React.createElement("button", {
    type: "button",
    onClick: handleReset,
    className: "text-gray-400 font-medium px-5 text-center hover:text-gray-300 mr-12"
  }, "Cancel"), /*#__PURE__*/React.createElement("span", {
    "data-for": "global",
    "data-tip": uploadDisabled ? 'You must select one column for text' : 'Upload items'
  }, /*#__PURE__*/React.createElement("button", {
    type: "button",
    onClick: () => handleUpload(),
    disabled: uploadDisabled,
    className: `font-medium rounded-sm text-sm px-5 py-2.5 text-center ${uploadDisabled ? 'bg-gray-500 text-gray-200' : 'text-white bg-indigo-500 hover:bg-indigo-800 focus:outline-none focus:ring-4 focus:ring-indigo-300 dark:bg-indigo-600 dark:hover:bg-indigo-700 dark:focus:ring-indigo-800'}`
  }, "Upload ", maxRows ?? totalNumberOfRows, " items"))))))) : /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("span", {
    className: `${uploaded ? 'mt-4' : 'mt-36'}`
  }, collectionItemsAmount && collectionItemsAmount > 0 ? `Select file to add items to your collection: ${collection}` : 'Your collection is empty, add items to populate it'), /*#__PURE__*/React.createElement("div", {
    className: "mt-4 w-96"
  }, /*#__PURE__*/React.createElement("div", {
    className: "flex items-center justify-center w-full"
  }, /*#__PURE__*/React.createElement("label", {
    htmlFor: "dropzone-file",
    className: "flex flex-col items-center justify-center w-full h-64 border-2 border-gray-300 border-dashed rounded-lg cursor-pointer bg-gray-50 dark:hover:bg-bray-800 dark:bg-gray-700 hover:bg-gray-100 dark:border-gray-600 dark:hover:border-gray-500 dark:hover:bg-gray-600"
  }, /*#__PURE__*/React.createElement("div", {
    className: "flex flex-col items-center justify-center pt-5 pb-6"
  }, /*#__PURE__*/React.createElement("svg", {
    "aria-hidden": "true",
    className: "w-10 h-10 mb-3 text-gray-400",
    fill: "none",
    stroke: "currentColor",
    viewBox: "0 0 24 24",
    xmlns: "http://www.w3.org/2000/svg"
  }, /*#__PURE__*/React.createElement("path", {
    strokeLinecap: "round",
    strokeLinejoin: "round",
    strokeWidth: "2",
    d: "M7 16a4 4 0 01-.88-7.903A5 5 0 1115.9 6L16 6a5 5 0 011 9.9M15 13l-3-3m0 0l-3 3m3-3v12"
  })), /*#__PURE__*/React.createElement("p", {
    className: "mb-2 text-sm text-gray-500 dark:text-gray-400"
  }, /*#__PURE__*/React.createElement("span", {
    className: "font-semibold"
  }, "Click to upload CSV"), ' ', "or drag and drop"), /*#__PURE__*/React.createElement("ul", {
    className: "w-full flex-wrap list-disc"
  }, /*#__PURE__*/React.createElement("li", {
    className: "text-xs text-gray-500 dark:text-gray-400 mt-2"
  }, "Up to 255 characters per cell"), /*#__PURE__*/React.createElement("li", {
    className: "text-xs text-gray-500 dark:text-gray-400 mt-2"
  }, "Up to 1GB"), /*#__PURE__*/React.createElement("li", {
    className: "text-xs text-gray-500 dark:text-gray-400 mt-2"
  }, "Up to 250 columns"))), /*#__PURE__*/React.createElement("input", {
    onDrop: handleDragOver,
    onChange: handleFileChange,
    id: "dropzone-file",
    type: "file",
    className: "hidden"
  })))))));
};

exports.OneAIAnalyticsApiWrapper = OneAIAnalyticsApiWrapper;
exports.OneAIAnalyticsStaticDataWrapper = OneAIAnalyticsStaticDataWrapper;
exports.OneAiAnalytics = OneAiAnalytics;
exports.OneAiUpload = OneAiUpload;
//# sourceMappingURL=oneai-analytics.cjs.development.js.map
